@@ -32,40 +32,23 @@ pacman --noconfirm -S --needed \
     mingw-w64-$MSYS2_ARCH-fribidi \
     mingw-w64-$MSYS2_ARCH-gst-plugins-bad-libs \
     mingw-w64-$MSYS2_ARCH-shared-mime-info \
-    mingw-w64-$MSYS2_ARCH-python-gobject
+    mingw-w64-$MSYS2_ARCH-python-gobject \
+    mingw-w64-$MSYS2_ARCH-shaderc \
+    mingw-w64-$MSYS2_ARCH-vulkan \
+    mingw-w64-$MSYS2_ARCH-vulkan-headers
 
 mkdir -p _ccache
 export CCACHE_BASEDIR="$(pwd)"
 export CCACHE_DIR="${CCACHE_BASEDIR}/_ccache"
 
-# https://gitlab.gnome.org/GNOME/gtk/-/issues/2243
-# https://gitlab.gnome.org/GNOME/gtk/-/issues/3002
-
-if ! pkg-config --atleast-version=2.66.0 glib-2.0; then
-    git clone https://gitlab.gnome.org/GNOME/glib.git _glib
-    meson setup _glib_build _glib
-    meson compile -C _glib_build
-    meson install -C _glib_build
-fi
-pkg-config --modversion glib-2.0
-
-if ! pkg-config --atleast-version=1.50.0 pango; then
-    git clone https://gitlab.gnome.org/GNOME/pango.git _pango
-    meson setup _pango_build _pango
-    meson compile -C _pango_build
-    meson install -C _pango_build
-fi
-pkg-config --modversion pango
-
 # Build
 ccache --zero-stats
 ccache --show-stats
 export CCACHE_DISABLE=true
-meson \
+meson setup \
     -Dx11-backend=false \
     -Dwayland-backend=false \
     -Dwin32-backend=true \
-    -Dvulkan=disabled \
     -Dintrospection=enabled \
     -Dgtk:werror=true \
     _build
