@@ -43,7 +43,7 @@ update_image (void)
   cairo_t *cr;
   GdkPixbuf *pixbuf;
   GdkPixbuf *pixbuf2;
-  const char *hint;
+  GdkTexture *texture;
   cairo_font_options_t *fopt;
   cairo_hint_style_t hintstyle;
   cairo_hint_metrics_t hintmetrics;
@@ -54,22 +54,27 @@ update_image (void)
     context = gtk_widget_create_pango_context (image);
 
   text = gtk_editable_get_text (GTK_EDITABLE (entry));
-  desc = gtk_font_chooser_get_font_desc (GTK_FONT_CHOOSER (font_button));
+  desc = gtk_font_dialog_button_get_font_desc (GTK_FONT_DIALOG_BUTTON (font_button));
 
   fopt = cairo_font_options_copy (pango_cairo_context_get_font_options (context));
 
-  hint = gtk_combo_box_get_active_id (GTK_COMBO_BOX (hinting));
-  hintstyle = CAIRO_HINT_STYLE_DEFAULT;
-  if (hint)
+  switch (gtk_drop_down_get_selected (GTK_DROP_DOWN (hinting)))
     {
-      if (strcmp (hint, "none") == 0)
-        hintstyle = CAIRO_HINT_STYLE_NONE;
-      else if (strcmp (hint, "slight") == 0)
-        hintstyle = CAIRO_HINT_STYLE_SLIGHT;
-      else if (strcmp (hint, "medium") == 0)
-        hintstyle = CAIRO_HINT_STYLE_MEDIUM;
-      else if (strcmp (hint, "full") == 0)
-        hintstyle = CAIRO_HINT_STYLE_FULL;
+    case 0:
+      hintstyle = CAIRO_HINT_STYLE_NONE;
+      break;
+    case 1:
+      hintstyle = CAIRO_HINT_STYLE_SLIGHT;
+      break;
+    case 2:
+      hintstyle = CAIRO_HINT_STYLE_MEDIUM;
+      break;
+    case 3:
+      hintstyle = CAIRO_HINT_STYLE_FULL;
+      break;
+    default:
+      hintstyle = CAIRO_HINT_STYLE_DEFAULT;
+      break;
     }
   cairo_font_options_set_hint_style (fopt, hintstyle);
 
@@ -116,8 +121,17 @@ update_image (void)
       cairo_destroy (cr);
       g_object_unref (layout);
 
-      pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, cairo_image_surface_get_width (surface), cairo_image_surface_get_height (surface));
-      pixbuf2 = gdk_pixbuf_scale_simple (pixbuf, gdk_pixbuf_get_width (pixbuf) * scale, gdk_pixbuf_get_height (pixbuf) * scale, GDK_INTERP_NEAREST);
+      pixbuf = gdk_pixbuf_new_from_data (cairo_image_surface_get_data (surface),
+                                         GDK_COLORSPACE_RGB, TRUE, 8,
+                                         cairo_image_surface_get_width (surface),
+                                         cairo_image_surface_get_height (surface),
+                                         cairo_image_surface_get_stride (surface),
+                                         NULL, NULL);
+
+      pixbuf2 = gdk_pixbuf_scale_simple (pixbuf,
+                                         gdk_pixbuf_get_width (pixbuf) * scale,
+                                         gdk_pixbuf_get_height (pixbuf) * scale,
+                                         GDK_INTERP_NEAREST);
 
       g_object_unref (pixbuf);
       cairo_surface_destroy (surface);
@@ -274,17 +288,25 @@ retry:
       pango_layout_iter_free (iter);
       g_object_unref (layout);
 
-      pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, cairo_image_surface_get_width (surface), cairo_image_surface_get_height (surface));
-      pixbuf2 = gdk_pixbuf_scale_simple (pixbuf, gdk_pixbuf_get_width (pixbuf) * scale, gdk_pixbuf_get_height (pixbuf) * scale, GDK_INTERP_NEAREST);
+      pixbuf = gdk_pixbuf_new_from_data (cairo_image_surface_get_data (surface),
+                                         GDK_COLORSPACE_RGB, TRUE, 8,
+                                         cairo_image_surface_get_width (surface),
+                                         cairo_image_surface_get_height (surface),
+                                         cairo_image_surface_get_stride (surface),
+                                         NULL, NULL);
+
+      pixbuf2 = gdk_pixbuf_scale_simple (pixbuf,
+                                         gdk_pixbuf_get_width (pixbuf) * scale,
+                                         gdk_pixbuf_get_height (pixbuf) * scale,
+                                         GDK_INTERP_NEAREST);
       g_object_unref (pixbuf);
       cairo_surface_destroy (surface);
     }
 
-  gtk_picture_set_pixbuf (GTK_PICTURE (image), pixbuf2);
-
+  texture = gdk_texture_new_for_pixbuf (pixbuf2);
+  gtk_picture_set_paintable (GTK_PICTURE (image), GDK_PAINTABLE (texture));
   g_object_unref (pixbuf2);
-
-  pango_font_description_free (desc);
+  g_object_unref (texture);
 }
 
 static gboolean fading = FALSE;
@@ -418,7 +440,7 @@ do_fontrendering (GtkWidget *do_widget)
       g_signal_connect (down_button, "clicked", G_CALLBACK (scale_down), NULL);
       g_signal_connect (entry, "notify::text", G_CALLBACK (update_image), NULL);
       g_signal_connect (font_button, "notify::font-desc", G_CALLBACK (update_image), NULL);
-      g_signal_connect (hinting, "notify::active", G_CALLBACK (update_image), NULL);
+      g_signal_connect (hinting, "notify::selected", G_CALLBACK (update_image), NULL);
       g_signal_connect (anti_alias, "notify::active", G_CALLBACK (update_image), NULL);
       g_signal_connect (hint_metrics, "notify::active", G_CALLBACK (update_image), NULL);
       g_signal_connect (text_radio, "notify::active", G_CALLBACK (update_image), NULL);
@@ -433,7 +455,7 @@ do_fontrendering (GtkWidget *do_widget)
     }
 
   if (!gtk_widget_get_visible (window))
-    gtk_widget_show (window);
+    gtk_widget_set_visible (window, TRUE);
   else
     gtk_window_destroy (GTK_WINDOW (window));
 
