@@ -48,14 +48,16 @@
 #define VERTICES_PER_TOOTH 34
 #define GEAR_VERTEX_STRIDE 6
 
-#ifndef HAVE_SINCOS
-static void
-sincos (double x, double *_sin, double *_cos)
+static inline void
+_sincos (double x, double *_sin, double *_cos)
 {
+#ifdef HAVE_SINCOS
+  sincos (x, _sin, _cos);
+#else
   *_sin = sin (x);
   *_cos = cos (x);
-}
 #endif
+}
 
 /**
  * Struct describing the vertices in triangle strip
@@ -307,11 +309,11 @@ create_gear (GLfloat inner_radius,
     struct point p[7];
 
     /* Calculate needed sin/cos for various angles */
-    sincos(i * 2.0 * G_PI / teeth + da * 0, &s[0], &c[0]);
-    sincos(i * 2.0 * M_PI / teeth + da * 1, &s[1], &c[1]);
-    sincos(i * 2.0 * M_PI / teeth + da * 2, &s[2], &c[2]);
-    sincos(i * 2.0 * M_PI / teeth + da * 3, &s[3], &c[3]);
-    sincos(i * 2.0 * M_PI / teeth + da * 4, &s[4], &c[4]);
+    _sincos(i * 2.0 * G_PI / teeth + da * 0, &s[0], &c[0]);
+    _sincos(i * 2.0 * M_PI / teeth + da * 1, &s[1], &c[1]);
+    _sincos(i * 2.0 * M_PI / teeth + da * 2, &s[2], &c[2]);
+    _sincos(i * 2.0 * M_PI / teeth + da * 3, &s[3], &c[3]);
+    _sincos(i * 2.0 * M_PI / teeth + da * 4, &s[4], &c[4]);
 
     GEAR_POINT(p[0], r2, 1);
     GEAR_POINT(p[1], r2, 2);
@@ -520,7 +522,7 @@ void perspective(GLfloat *m, GLfloat fovy, GLfloat aspect, GLfloat zNear, GLfloa
    identity(tmp);
 
    deltaZ = zFar - zNear;
-   sincos(radians, &sine, &cosine);
+   _sincos(radians, &sine, &cosine);
 
    if ((deltaZ == 0) || (sine == 0) || (aspect == 0))
       return;
@@ -735,7 +737,6 @@ gtk_gears_realize (GtkWidget *widget)
   GtkGLArea *glarea = GTK_GL_AREA (widget);
   GtkGears *gears = GTK_GEARS (widget);
   GtkGearsPrivate *priv = gtk_gears_get_instance_private (gears);
-  GdkGLContext *context;
   GLuint vao, v, f, program;
   const char *p;
   char msg[512];
@@ -746,8 +747,6 @@ gtk_gears_realize (GtkWidget *widget)
   if (gtk_gl_area_get_error (glarea) != NULL)
     return;
 
-  context = gtk_gl_area_get_context (glarea);
-
   glEnable (GL_CULL_FACE);
   glEnable (GL_DEPTH_TEST);
 
@@ -757,7 +756,7 @@ gtk_gears_realize (GtkWidget *widget)
   priv->vao = vao;
 
   /* Compile the vertex shader */
-  if (gdk_gl_context_get_use_es (context))
+  if (gtk_gl_area_get_api (glarea) == GDK_GL_API_GLES)
     p = vertex_shader_gles;
   else
     p = vertex_shader_gl;
@@ -768,7 +767,7 @@ gtk_gears_realize (GtkWidget *widget)
   g_debug ("vertex shader info: %s\n", msg);
 
   /* Compile the fragment shader */
-  if (gdk_gl_context_get_use_es (context))
+  if (gtk_gl_area_get_api (glarea) == GDK_GL_API_GLES)
     p = fragment_shader_gles;
   else
     p = fragment_shader_gl;
@@ -841,24 +840,24 @@ gtk_gears_unrealize (GtkWidget *widget)
   GtkGearsPrivate *priv = gtk_gears_get_instance_private ((GtkGears *) widget);
 
   gtk_gl_area_make_current (glarea);
-  if (gtk_gl_area_get_error (glarea) != NULL)
-    return;
+  if (gtk_gl_area_get_error (glarea) == NULL)
+    {
+      /* Release the resources associated with OpenGL */
+      if (priv->gear_vbo[0] != 0)
+        glDeleteBuffers (1, &(priv->gear_vbo[0]));
 
-  /* Release the resources associated with OpenGL */
-  if (priv->gear_vbo[0] != 0)
-    glDeleteBuffers (1, &(priv->gear_vbo[0]));
+      if (priv->gear_vbo[1] != 0)
+        glDeleteBuffers (1, &(priv->gear_vbo[1]));
 
-  if (priv->gear_vbo[1] != 0)
-    glDeleteBuffers (1, &(priv->gear_vbo[1]));
+      if (priv->gear_vbo[2] != 0)
+        glDeleteBuffers (1, &(priv->gear_vbo[2]));
 
-  if (priv->gear_vbo[2] != 0)
-    glDeleteBuffers (1, &(priv->gear_vbo[2]));
+      if (priv->vao != 0)
+        glDeleteVertexArrays (1, &priv->vao);
 
-  if (priv->vao != 0)
-    glDeleteVertexArrays (1, &priv->vao);
-
-  if (priv->program != 0)
-    glDeleteProgram (priv->program);
+      if (priv->program != 0)
+        glDeleteProgram (priv->program);
+    }
 
   priv->ModelViewProjectionMatrix_location = 0;
   priv->NormalMatrix_location = 0;
