@@ -23,9 +23,19 @@
 #include "gdkglcontext.h"
 #include "gdkdrawcontextprivate.h"
 #include "gdkglversionprivate.h"
+#include "gdkdmabufprivate.h"
 
 G_BEGIN_DECLS
 
+typedef enum {
+  GDK_GL_FEATURE_DEBUG                      = 1 << 0,
+  GDK_GL_FEATURE_UNPACK_SUBIMAGE            = 1 << 1,
+  GDK_GL_FEATURE_VERTEX_HALF_FLOAT          = 1 << 2,
+  GDK_GL_FEATURE_SYNC                       = 1 << 3,
+  GDK_GL_FEATURE_BASE_INSTANCE              = 1 << 4,
+  GDK_GL_FEATURE_BUFFER_STORAGE             = 1 << 5,
+} GdkGLFeatures;
+
 typedef enum {
   GDK_GL_NONE = 0,
   GDK_GL_EGL,
@@ -34,6 +44,17 @@ typedef enum {
   GDK_GL_CGL
 } GdkGLBackend;
 
+typedef enum {
+  /* The format is supported for glTexImage2D() */
+  GDK_GL_FORMAT_USABLE = 1 << 0,
+  /* The format can be rendered to.
+   * GL/GLES spec term: "color-renderable" */
+  GDK_GL_FORMAT_RENDERABLE = 1 << 1,
+  /* GL_LINEAR/GL_MIPMAP_LINEAR can be used for textures in this format.
+   * GLES spec term: "texture-filterable" */
+  GDK_GL_FORMAT_FILTERABLE = 1 << 2
+} GdkGLMemoryFlags;
+
 /* The maximum amount of buffers we track update regions for.
  * Note that this is equal to the max buffer age value we
  * can provide a damage region for */
@@ -130,7 +151,6 @@ void                    gdk_gl_context_get_matching_version     (GdkGLContext
                                                                  gboolean                legacy,
                                                                  GdkGLVersion           *out_version);
 
-gboolean                gdk_gl_context_has_unpack_subimage      (GdkGLContext    *context);
 void                    gdk_gl_context_push_debug_group         (GdkGLContext    *context,
                                                                  const char      *message);
 void                    gdk_gl_context_push_debug_group_printf  (GdkGLContext    *context,
@@ -147,15 +167,27 @@ void                    gdk_gl_context_label_object_printf      (GdkGLContext
                                                                  const char      *format,
                                                                 ...)  G_GNUC_PRINTF (4, 5);
 
-gboolean                gdk_gl_context_has_debug                (GdkGLContext    *self) G_GNUC_PURE;
+const char *            gdk_gl_context_get_glsl_version_string  (GdkGLContext    *self);
+
+GdkGLMemoryFlags        gdk_gl_context_get_format_flags         (GdkGLContext    *self,
+                                                                 GdkMemoryFormat  format) G_GNUC_PURE;
+gboolean                gdk_gl_context_has_feature              (GdkGLContext    *self,
+                                                                 GdkGLFeatures    feature) G_GNUC_PURE;
 
 gboolean                gdk_gl_context_use_es_bgra              (GdkGLContext    *context);
 
-gboolean                gdk_gl_context_has_vertex_half_float    (GdkGLContext    *self) G_GNUC_PURE;
-
-gboolean                gdk_gl_context_has_sync                 (GdkGLContext    *self) G_GNUC_PURE;
+gboolean                gdk_gl_context_has_vertex_arrays        (GdkGLContext    *self) G_GNUC_PURE;
 
 double                  gdk_gl_context_get_scale                (GdkGLContext    *self);
 
-G_END_DECLS
+guint                   gdk_gl_context_import_dmabuf            (GdkGLContext    *self,
+                                                                 int              width,
+                                                                 int              height,
+                                                                 const GdkDmabuf *dmabuf,
+                                                                 gboolean        *external);
 
+gboolean                gdk_gl_context_export_dmabuf            (GdkGLContext    *self,
+                                                                 unsigned int     texture_id,
+                                                                 GdkDmabuf       *dmabuf);
+
+G_END_DECLS
