@@ -113,6 +113,33 @@ gdk_toplevel_notify_compute_size (GdkToplevel     *toplevel,
   gdk_toplevel_size_validate (size);
 }
 
+static void
+gdk_toplevel_default_export_handle (GdkToplevel          *toplevel,
+                                    GCancellable         *cancellable,
+                                    GAsyncReadyCallback   callback,
+                                    gpointer              user_data)
+{
+  GTask *task;
+
+  task = g_task_new (toplevel, cancellable, callback, user_data);
+  g_task_return_pointer (task, NULL, NULL);
+  g_object_unref (task);
+}
+
+static char *
+gdk_toplevel_default_export_handle_finish (GdkToplevel   *toplevel,
+                                           GAsyncResult  *result,
+                                           GError       **error)
+{
+  return g_task_propagate_pointer (G_TASK (result), error);
+}
+
+static void
+gdk_toplevel_default_unexport_handle (GdkToplevel *toplevel,
+                                      const char  *handle)
+{
+}
+
 static void
 gdk_toplevel_default_init (GdkToplevelInterface *iface)
 {
@@ -125,6 +152,9 @@ gdk_toplevel_default_init (GdkToplevelInterface *iface)
   iface->inhibit_system_shortcuts = gdk_toplevel_default_inhibit_system_shortcuts;
   iface->restore_system_shortcuts = gdk_toplevel_default_restore_system_shortcuts;
   iface->titlebar_gesture = gdk_toplevel_default_titlebar_gesture;
+  iface->export_handle = gdk_toplevel_default_export_handle;
+  iface->export_handle_finish = gdk_toplevel_default_export_handle_finish;
+  iface->unexport_handle = gdk_toplevel_default_unexport_handle;
 
   /**
    * GdkToplevel:state: (attributes org.gtk.Property.get=gdk_toplevel_get_state)
@@ -232,15 +262,16 @@ gdk_toplevel_default_init (GdkToplevelInterface *iface)
   /**
    * GdkToplevel::compute-size:
    * @toplevel: a `GdkToplevel`
-   * @size: (type Gdk.ToplevelSize) (out caller-allocates): a `GdkToplevelSize`
+   * @size: (type Gdk.ToplevelSize): a `GdkToplevelSize`
    *
    * Emitted when the size for the surface needs to be computed, when
    * it is present.
    *
-   * It will normally be emitted during or after [method@Gdk.Toplevel.present],
-   * depending on the configuration received by the windowing system.
-   * It may also be emitted at any other point in time, in response
-   * to the windowing system spontaneously changing the configuration.
+   * This signal will normally be emitted during or after a call to
+   * [method@Gdk.Toplevel.present], depending on the configuration
+   * received by the windowing system. It may also be emitted at any
+   * other point in time, in response to the windowing system
+   * spontaneously changing the configuration of the toplevel surface.
    *
    * It is the responsibility of the toplevel user to handle this signal
    * and compute the desired size of the toplevel, given the information
@@ -346,8 +377,9 @@ gdk_toplevel_lower (GdkToplevel *toplevel)
  *
  * Sets keyboard focus to @surface.
  *
- * In most cases, [method@Gtk.Window.present_with_time] should be
- * used on a [class@Gtk.Window], rather than calling this function.
+ * In most cases, [gtk_window_present_with_time()](../gtk4/method.Window.present_with_time.html)
+ * should be used on a [GtkWindow](../gtk4/class.Window.html), rather than
+ * calling this function.
  */
 void
 gdk_toplevel_focus (GdkToplevel *toplevel,
@@ -406,8 +438,8 @@ gdk_toplevel_set_title (GdkToplevel *toplevel,
  * Sets the startup notification ID.
  *
  * When using GTK, typically you should use
- * [method@Gtk.Window.set_startup_id] instead of this
- * low-level function.
+ * [gtk_window_set_startup_id()](../gtk4/method.Window.set_startup_id.html)
+ * instead of this low-level function.
  */
 void
 gdk_toplevel_set_startup_id (GdkToplevel *toplevel,
@@ -430,8 +462,8 @@ gdk_toplevel_set_startup_id (GdkToplevel *toplevel,
  * allows the window manager to do things like center @surface
  * on @parent and keep @surface above @parent.
  *
- * See [method@Gtk.Window.set_transient_for] if you’re using
- * [class@Gtk.Window] or [class@Gtk.Dialog].
+ * See [gtk_window_set_transient_for()](../gtk4/method.Window.set_transient_for.html)
+ * if you’re using [GtkWindow](../gtk4/class.Window.html).
  */
 void
 gdk_toplevel_set_transient_for (GdkToplevel *toplevel,
@@ -724,3 +756,74 @@ gdk_toplevel_titlebar_gesture (GdkToplevel        *toplevel,
   return GDK_TOPLEVEL_GET_IFACE (toplevel)->titlebar_gesture (toplevel,
                                                               gesture);
 }
+
+/*< private >
+ * gdk_toplevel_export_handle:
+ * @toplevel: a `GdkToplevel`
+ * @cancellable: (nullable): a `GCancellable`
+ * @callback: ithe callback to call when the handle has been exported
+ * @user_data: (closure callback): data to pass to @callback
+ *
+ * This function asynchronously obtains a handle for a toplevel surface
+ * that can be passed to other processes.
+ *
+ * When a handle has been obtained, @callback will be called, and can
+ * receive the handle via [method@Gdk.Toplevel.export_handle_finish].
+ *
+ * It is an error to call this function on a surface that is already
+ * exported.
+ *
+ * When the handle is no longer needed, [method@Gdk.Toplevel.unexport_handle]
+ * should be called to clean up resources.
+ *
+ * Since: 4.10
+ */
+void
+gdk_toplevel_export_handle (GdkToplevel         *toplevel,
+                            GCancellable        *cancellable,
+                            GAsyncReadyCallback  callback,
+                            gpointer             user_data)
+{
+  GDK_TOPLEVEL_GET_IFACE (toplevel)->export_handle (toplevel, cancellable, callback, user_data);
+}
+
+/*< private >
+ * gdk_toplevel_export_handle_finish:
+ * @toplevel: a `GdkToplevel`
+ * @result: the `GAsyncResult`
+ * @error: return location for an error
+ *
+ * Finishes the [method@Gdk.Toplevel.export_handle] call and
+ * returns the resulting handle.
+ *
+ * Returns: (nullable) (transfer full): the exported handle,
+ *   or `NULL` and @error is set
+ *
+ * Since: 4.10
+ */
+char *
+gdk_toplevel_export_handle_finish (GdkToplevel   *toplevel,
+                                   GAsyncResult  *result,
+                                   GError       **error)
+{
+  return GDK_TOPLEVEL_GET_IFACE (toplevel)->export_handle_finish (toplevel, result, error);
+}
+
+/*< private >
+ * gdk_toplevel_unexport_handle:
+ * @toplevel: a `GdkToplevel`
+ * @handle: the handle to unexport
+ *
+ * Destroys the handle that was obtained with [method@Gdk.Toplevel.export_handle].
+ *
+ * It is an error to call this function on a surface that
+ * does not have a handle.
+ *
+ * Since: 4.10
+ */
+void
+gdk_toplevel_unexport_handle (GdkToplevel *toplevel,
+                              const char  *handle)
+{
+  GDK_TOPLEVEL_GET_IFACE (toplevel)->unexport_handle (toplevel, handle);
+}
