@@ -32,7 +32,7 @@
 
 #define GDK_IS_MACOS_WINDOW(obj) ([obj isKindOfClass:[GdkMacosWindow class]])
 
-@interface GdkMacosWindow : NSWindow {
+@interface GdkMacosWindow : NSWindow <NSDraggingSource, NSDraggingDestination> {
   GdkMacosSurface *gdk_surface;
 
   BOOL             inMove;
@@ -49,10 +49,6 @@
 
   EdgeSnapping     snapping;
 
-  NSRect           lastUnmaximizedFrame;
-  NSRect           lastMaximizedFrame;
-  NSRect           lastUnfullscreenFrame;
-  BOOL             inMaximizeTransition;
   BOOL             inFullscreenTransition;
 }
 
