@@ -27,11 +27,11 @@
 
 #include "gdkmacossurface-private.h"
 
-#include "gdkdebug.h"
+#include "gdkdebugprivate.h"
 #include "gdkdeviceprivate.h"
 #include "gdkdisplay.h"
 #include "gdkeventsprivate.h"
-#include "gdkframeclockidleprivate.h"
+#include "gdkframeclockprivate.h"
 #include "gdkseatprivate.h"
 #include "gdksurfaceprivate.h"
 
@@ -43,8 +43,8 @@
 #include "gdkmacosglcontext-private.h"
 #include "gdkmacosmonitor-private.h"
 #include "gdkmacospopupsurface-private.h"
-#include "gdkmacostoplevelsurface-private.h"
 #include "gdkmacosutils-private.h"
+#include "gdkmacostoplevelsurface-private.h"
 
 G_DEFINE_ABSTRACT_TYPE (GdkMacosSurface, gdk_macos_surface, GDK_TYPE_SURFACE)
 
@@ -56,14 +56,6 @@ enum {
 
 static GParamSpec *properties [LAST_PROP];
 
-static gboolean
-window_is_fullscreen (GdkMacosSurface *self)
-{
-  g_assert (GDK_IS_MACOS_SURFACE (self));
-
-  return ([self->window styleMask] & NSWindowStyleMaskFullScreen) != 0;
-}
-
 void
 _gdk_macos_surface_request_frame (GdkMacosSurface *self)
 {
@@ -195,7 +187,6 @@ gdk_macos_surface_hide (GdkSurface *surface)
 {
   GdkMacosSurface *self = (GdkMacosSurface *)surface;
   GdkSeat *seat;
-  gboolean was_mapped;
   gboolean was_key;
 
   g_assert (GDK_IS_MACOS_SURFACE (self));
@@ -204,7 +195,6 @@ gdk_macos_surface_hide (GdkSurface *surface)
 
   _gdk_macos_surface_cancel_frame (self);
 
-  was_mapped = GDK_SURFACE_IS_MAPPED (surface);
   was_key = [self->window isKeyWindow];
 
   seat = gdk_display_get_default_seat (surface->display);
@@ -236,8 +226,8 @@ gdk_macos_surface_hide (GdkSurface *surface)
     }
 }
 
-static int
-gdk_macos_surface_get_scale_factor (GdkSurface *surface)
+static double
+gdk_macos_surface_get_scale (GdkSurface *surface)
 {
   GdkMacosSurface *self = (GdkMacosSurface *)surface;
 
@@ -246,32 +236,6 @@ gdk_macos_surface_get_scale_factor (GdkSurface *surface)
   return [self->window backingScaleFactor];
 }
 
-void
-_gdk_macos_surface_set_shadow (GdkMacosSurface *surface,
-                               int              top,
-                               int              right,
-                               int              bottom,
-                               int              left)
-{
-  GdkMacosSurface *self = (GdkMacosSurface *)surface;
-
-  g_assert (GDK_IS_MACOS_SURFACE (self));
-
-  if (self->shadow_top == top &&
-      self->shadow_right == right &&
-      self->shadow_bottom == bottom &&
-      self->shadow_left == left)
-    return;
-
-  self->shadow_top = top;
-  self->shadow_right = right;
-  self->shadow_bottom = bottom;
-  self->shadow_left = left;
-
-  if (top || right || bottom || left)
-    [self->window setHasShadow:NO];
-}
-
 static void
 gdk_macos_surface_begin_frame (GdkMacosSurface *self)
 {
@@ -285,14 +249,12 @@ gdk_macos_surface_end_frame (GdkMacosSurface *self)
 {
   GdkFrameTimings *timings;
   GdkFrameClock *frame_clock;
-  GdkDisplay *display;
 
   g_assert (GDK_IS_MACOS_SURFACE (self));
 
   if (GDK_SURFACE_DESTROYED (self))
     return;
 
-  display = gdk_surface_get_display (GDK_SURFACE (self));
   frame_clock = gdk_surface_get_frame_clock (GDK_SURFACE (self));
 
   if ((timings = gdk_frame_clock_get_current_timings (frame_clock)))
@@ -420,11 +382,6 @@ gdk_macos_surface_drag_begin (GdkSurface         *surface,
   GdkMacosSurface *drag_surface;
   GdkMacosDrag *drag;
   GdkCursor *cursor;
-  GdkSeat *seat;
-  double px;
-  double py;
-  int sx;
-  int sy;
 
   g_assert (GDK_IS_MACOS_SURFACE (self));
   g_assert (GDK_IS_MACOS_TOPLEVEL_SURFACE (self) ||
@@ -432,13 +389,7 @@ gdk_macos_surface_drag_begin (GdkSurface         *surface,
   g_assert (GDK_IS_MACOS_DEVICE (device));
   g_assert (GDK_IS_CONTENT_PROVIDER (content));
 
-  seat = gdk_device_get_seat (device);
-  gdk_macos_device_query_state (device, surface, NULL, &px, &py, NULL);
-  _gdk_macos_surface_get_root_coords (GDK_MACOS_SURFACE (surface), &sx, &sy);
-  drag_surface = _gdk_macos_surface_new (GDK_MACOS_DISPLAY (surface->display),
-                                         GDK_SURFACE_TEMP,
-                                         surface,
-                                         sx, sy, 1, 1);
+  drag_surface = _gdk_macos_drag_surface_new (GDK_MACOS_DISPLAY (surface->display));
   drag = g_object_new (GDK_TYPE_MACOS_DRAG,
                        "drag-surface", drag_surface,
                        "surface", surface,
@@ -452,7 +403,7 @@ gdk_macos_surface_drag_begin (GdkSurface         *surface,
                                 gdk_drag_get_selected_action (GDK_DRAG (drag)));
   gdk_drag_set_cursor (GDK_DRAG (drag), cursor);
 
-  if (!_gdk_macos_drag_begin (drag))
+  if (!_gdk_macos_drag_begin (drag, content, self->window))
     {
       g_object_unref (drag);
       return NULL;
@@ -533,8 +484,14 @@ gdk_macos_surface_constructed (GObject *object)
                                G_CONNECT_SWAPPED);
     }
 
+  gdk_surface_freeze_updates (GDK_SURFACE (self));
+  _gdk_macos_surface_monitor_changed (self);
+
   if (self->window != NULL)
     _gdk_macos_surface_configure (self);
+
+  _gdk_macos_display_surface_added (GDK_MACOS_DISPLAY (gdk_surface_get_display (GDK_SURFACE (self))),
+                                    self);
 }
 
 static void
@@ -556,26 +513,6 @@ gdk_macos_surface_get_property (GObject    *object,
     }
 }
 
-static void
-gdk_macos_surface_set_property (GObject      *object,
-                                guint         prop_id,
-                                const GValue *value,
-                                GParamSpec   *pspec)
-{
-  GdkMacosSurface *self = GDK_MACOS_SURFACE (object);
-
-  switch (prop_id)
-    {
-    case PROP_NATIVE:
-      self->window = g_value_get_pointer (value);
-      [self->window setGdkSurface:self];
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
-}
-
 static void
 gdk_macos_surface_class_init (GdkMacosSurfaceClass *klass)
 {
@@ -584,14 +521,13 @@ gdk_macos_surface_class_init (GdkMacosSurfaceClass *klass)
 
   object_class->constructed = gdk_macos_surface_constructed;
   object_class->get_property = gdk_macos_surface_get_property;
-  object_class->set_property = gdk_macos_surface_set_property;
 
   surface_class->destroy = gdk_macos_surface_destroy;
   surface_class->drag_begin = gdk_macos_surface_drag_begin;
   surface_class->get_device_state = gdk_macos_surface_get_device_state;
   surface_class->get_geometry = gdk_macos_surface_get_geometry;
   surface_class->get_root_coords = gdk_macos_surface_get_root_coords;
-  surface_class->get_scale_factor = gdk_macos_surface_get_scale_factor;
+  surface_class->get_scale = gdk_macos_surface_get_scale;
   surface_class->hide = gdk_macos_surface_hide;
   surface_class->set_input_region = gdk_macos_surface_set_input_region;
   surface_class->set_opaque_region = gdk_macos_surface_set_opaque_region;
@@ -603,7 +539,7 @@ gdk_macos_surface_class_init (GdkMacosSurfaceClass *klass)
    */
   properties [PROP_NATIVE] =
     g_param_spec_pointer ("native", NULL, NULL,
-                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, LAST_PROP, properties);
 }
@@ -617,78 +553,6 @@ gdk_macos_surface_init (GdkMacosSurface *self)
   self->monitors = g_ptr_array_new_with_free_func (g_object_unref);
 }
 
-GdkMacosSurface *
-_gdk_macos_surface_new (GdkMacosDisplay   *display,
-                        GdkSurfaceType     surface_type,
-                        GdkSurface        *parent,
-                        int                x,
-                        int                y,
-                        int                width,
-                        int                height)
-{
-  GdkFrameClock *frame_clock;
-  GdkMacosSurface *ret;
-
-  g_return_val_if_fail (GDK_IS_MACOS_DISPLAY (display), NULL);
-
-  if (parent != NULL)
-    frame_clock = g_object_ref (parent->frame_clock);
-  else
-    frame_clock = _gdk_frame_clock_idle_new ();
-
-  switch (surface_type)
-    {
-    case GDK_SURFACE_TOPLEVEL:
-      ret = _gdk_macos_toplevel_surface_new (display, parent, frame_clock, x, y, width, height);
-      break;
-
-    case GDK_SURFACE_POPUP:
-      ret = _gdk_macos_popup_surface_new (display, parent, frame_clock, x, y, width, height);
-      break;
-
-    case GDK_SURFACE_TEMP:
-      ret = _gdk_macos_drag_surface_new (display, frame_clock, x, y, width, height);
-      break;
-
-    default:
-      g_warn_if_reached ();
-      ret = NULL;
-    }
-
-  if (ret != NULL)
-    {
-      gdk_surface_freeze_updates (GDK_SURFACE (ret));
-      _gdk_macos_surface_monitor_changed (ret);
-    }
-
-  g_object_unref (frame_clock);
-
-  return g_steal_pointer (&ret);
-}
-
-void
-_gdk_macos_surface_get_shadow (GdkMacosSurface *self,
-                               int             *top,
-                               int             *right,
-                               int             *bottom,
-                               int             *left)
-{
-
-  g_return_if_fail (GDK_IS_MACOS_SURFACE (self));
-
-  if (top)
-    *top = self->shadow_top;
-
-  if (left)
-    *left = self->shadow_left;
-
-  if (bottom)
-    *bottom = self->shadow_bottom;
-
-  if (right)
-    *right = self->shadow_right;
-}
-
 gboolean
 _gdk_macos_surface_is_opaque (GdkMacosSurface *self)
 {
@@ -761,6 +625,16 @@ _gdk_macos_surface_get_native (GdkMacosSurface *self)
   return (NSWindow *)self->window;
 }
 
+void
+_gdk_macos_surface_set_native (GdkMacosSurface *self,
+                               GdkMacosWindow  *window)
+{
+  g_assert (self->window == NULL);
+
+  self->window = window;
+  [self->window setGdkSurface:self];
+}
+
 /**
  * gdk_macos_surface_get_native_window: (attributes org.gtk.Method.get_property=native)
  * @self: a #GdkMacosSurface
@@ -827,7 +701,7 @@ _gdk_macos_surface_update_fullscreen_state (GdkMacosSurface *self)
   g_return_if_fail (GDK_IS_MACOS_SURFACE (self));
 
   state = GDK_SURFACE (self)->state;
-  is_fullscreen = window_is_fullscreen (self);
+  is_fullscreen = ([self->window styleMask] & NSWindowStyleMaskFullScreen) != 0;
   was_fullscreen = (state & GDK_TOPLEVEL_STATE_FULLSCREEN) != 0;
 
   if (is_fullscreen != was_fullscreen)
@@ -944,7 +818,8 @@ _gdk_macos_surface_synthesize_null_key (GdkMacosSurface *self)
                              0,
                              FALSE,
                              &translated,
-                             &no_lock);
+                             &no_lock,
+                             NULL);
   _gdk_event_queue_append (display, event);
 }
 
@@ -1152,10 +1027,9 @@ _gdk_macos_surface_monitor_changed (GdkMacosSurface *self)
 
   if (g_set_object (&self->best_monitor, best))
     {
-      GDK_NOTE (MISC,
-                g_message ("Surface \"%s\" moved to monitor \"%s\"",
-                           self->title ? self->title : "unknown",
-                           gdk_monitor_get_connector (best)));
+      GDK_DEBUG (MISC, "Surface \"%s\" moved to monitor \"%s\"",
+                       self->title ? self->title : "unknown",
+                       gdk_monitor_get_connector (best));
 
       _gdk_macos_surface_configure (self);
 
@@ -1174,16 +1048,12 @@ _gdk_macos_surface_monitor_changed (GdkMacosSurface *self)
 
           g_set_object (&child->best_monitor, best);
 
-          area.x = self->root_x + GDK_SURFACE (child)->x + child->shadow_left;
-          area.y = self->root_y + GDK_SURFACE (child)->y + child->shadow_top;
-          area.width = GDK_SURFACE (child)->width - child->shadow_left - child->shadow_right;
-          area.height = GDK_SURFACE (child)->height - child->shadow_top - child->shadow_bottom;
+          area.x = self->root_x + GDK_SURFACE (child)->x;
+          area.y = self->root_y + GDK_SURFACE (child)->y;
+          area.width = GDK_SURFACE (child)->width;
+          area.height = GDK_SURFACE (child)->height;
 
           _gdk_macos_monitor_clamp (GDK_MACOS_MONITOR (best), &area);
-
-          area.x -= child->shadow_left;
-          area.y -= child->shadow_top;
-
           _gdk_macos_surface_move (child, area.x, area.y);
           gdk_surface_invalidate_rect (GDK_SURFACE (child), NULL);
         }
