@@ -23,6 +23,7 @@
 
 #include "gdkmacostoplevelsurface-private.h"
 
+#include "gdkframeclockidleprivate.h"
 #include "gdkseatprivate.h"
 #include "gdktoplevelprivate.h"
 
@@ -95,37 +96,26 @@ _gdk_macos_toplevel_surface_unminimize (GdkMacosToplevelSurface *self)
     [window deminiaturize:window];
 }
 
-static void
-_gdk_macos_toplevel_surface_present (GdkToplevel       *toplevel,
-                                     GdkToplevelLayout *layout)
+static gboolean
+_gdk_macos_toplevel_surface_compute_size (GdkSurface *surface)
 {
-  GdkSurface *surface = GDK_SURFACE (toplevel);
-  GdkMacosToplevelSurface *self = (GdkMacosToplevelSurface *)toplevel;
-  NSWindow *nswindow = _gdk_macos_surface_get_native (GDK_MACOS_SURFACE (self));
-  GdkDisplay *display = gdk_surface_get_display (surface);
-  GdkMonitor *monitor;
+  GdkMacosToplevelSurface *self = (GdkMacosToplevelSurface *)surface;
+  GdkMacosSurface *macos_surface = (GdkMacosSurface *)surface;
   GdkToplevelSize size;
+  GdkDisplay *display;
+  GdkMonitor *monitor;
   int bounds_width, bounds_height;
-  int width, height;
   GdkGeometry geometry;
   GdkSurfaceHints mask;
-  NSWindowStyleMask style_mask;
-  gboolean maximize;
-  gboolean fullscreen;
 
   g_assert (GDK_IS_MACOS_TOPLEVEL_SURFACE (self));
-  g_assert (GDK_IS_MACOS_WINDOW (nswindow));
 
-  if (layout != self->layout)
-    {
-      g_clear_pointer (&self->layout, gdk_toplevel_layout_unref);
-      self->layout = gdk_toplevel_layout_copy (layout);
-    }
+  if (!GDK_MACOS_SURFACE (surface)->geometry_dirty)
+    return FALSE;
 
-  _gdk_macos_toplevel_surface_attach_to_parent (self);
-
-  style_mask = [nswindow styleMask];
+  GDK_MACOS_SURFACE (surface)->geometry_dirty = FALSE;
 
+  display = gdk_surface_get_display (surface);
   monitor = gdk_display_get_monitor_at_surface (display, surface);
 
   if (monitor)
@@ -143,60 +133,121 @@ _gdk_macos_toplevel_surface_present (GdkToplevel       *toplevel,
     }
 
   gdk_toplevel_size_init (&size, bounds_width, bounds_height);
-  gdk_toplevel_notify_compute_size (toplevel, &size);
+  gdk_toplevel_notify_compute_size (GDK_TOPLEVEL (surface), &size);
+
   g_warn_if_fail (size.width > 0);
   g_warn_if_fail (size.height > 0);
-  width = size.width;
-  height = size.height;
 
-  if (gdk_toplevel_layout_get_resizable (layout))
+  if (self->layout != NULL &&
+      gdk_toplevel_layout_get_resizable (self->layout))
     {
       geometry.min_width = size.min_width;
       geometry.min_height = size.min_height;
       mask = GDK_HINT_MIN_SIZE;
-
-      /* Only set 'Resizable' mask to get native resize zones if the window is
-       * titled, otherwise we do this internally for CSD and do not need
-       * NSWindow to do it for us. Additionally, it can mess things up when
-       * doing a window resize since it can cause mouseDown to get passed
-       * through to the next window.
-       */
-      if ((style_mask & NSWindowStyleMaskTitled) != 0)
-        style_mask |= NSWindowStyleMaskResizable;
-      else
-        style_mask &= ~NSWindowStyleMaskResizable;
     }
   else
     {
-      geometry.max_width = geometry.min_width = width;
-      geometry.max_height = geometry.min_height = height;
+      geometry.max_width = geometry.min_width = size.width;
+      geometry.max_height = geometry.min_height = size.height;
       mask = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE;
-
-      style_mask &= ~NSWindowStyleMaskResizable;
     }
 
+  _gdk_macos_surface_set_geometry_hints (macos_surface, &geometry, mask);
+
+  if (surface->state & (GDK_TOPLEVEL_STATE_FULLSCREEN |
+                        GDK_TOPLEVEL_STATE_MAXIMIZED |
+                        GDK_TOPLEVEL_STATE_TILED |
+                        GDK_TOPLEVEL_STATE_TOP_TILED |
+                        GDK_TOPLEVEL_STATE_RIGHT_TILED |
+                        GDK_TOPLEVEL_STATE_BOTTOM_TILED |
+                        GDK_TOPLEVEL_STATE_LEFT_TILED |
+                        GDK_TOPLEVEL_STATE_MINIMIZED) ||
+      [macos_surface->window inLiveResize])
+    return FALSE;
+
+  /* If we delayed a user resize until the beginning of the frame,
+   * apply it now so we can start processing updates for it.
+   */
+  if (macos_surface->next_layout.width > 0 &&
+      macos_surface->next_layout.height > 0)
+    {
+      int root_x = macos_surface->next_layout.root_x;
+      int root_y = macos_surface->next_layout.root_y;
+      int width = macos_surface->next_layout.width;
+      int height = macos_surface->next_layout.height;
+
+      gdk_surface_constrain_size (&geometry, mask,
+                                  width, height,
+                                  &width, &height);
+
+      macos_surface->next_layout.width = 0;
+      macos_surface->next_layout.height = 0;
+
+      _gdk_macos_surface_move_resize (macos_surface,
+                                      root_x, root_y,
+                                      width, height);
+
+      return FALSE;
+    }
+
+  gdk_surface_constrain_size (&geometry, mask,
+                              size.width, size.height,
+                              &size.width, &size.height);
+
+  if ((size.width != self->last_computed_width ||
+       size.height != self->last_computed_height) &&
+      (size.width != surface->width ||
+       size.height != surface->height))
+    {
+      self->last_computed_width = size.width;
+      self->last_computed_height = size.height;
+
+      _gdk_macos_surface_resize (macos_surface, size.width, size.height);
+    }
+
+  return FALSE;
+}
+
+static void
+_gdk_macos_toplevel_surface_present (GdkToplevel       *toplevel,
+                                     GdkToplevelLayout *layout)
+{
+  GdkSurface *surface = GDK_SURFACE (toplevel);
+  GdkMacosToplevelSurface *self = (GdkMacosToplevelSurface *)toplevel;
+  NSWindow *nswindow = _gdk_macos_surface_get_native (GDK_MACOS_SURFACE (self));
+  GdkDisplay *display = gdk_surface_get_display (surface);
+  NSWindowStyleMask style_mask;
+  gboolean maximize;
+  gboolean fullscreen;
+
+  g_assert (GDK_IS_MACOS_TOPLEVEL_SURFACE (self));
+  g_assert (GDK_IS_MACOS_WINDOW (nswindow));
+
+  if (layout != self->layout)
+    {
+      g_clear_pointer (&self->layout, gdk_toplevel_layout_unref);
+      self->layout = gdk_toplevel_layout_copy (layout);
+    }
+
+  _gdk_macos_toplevel_surface_attach_to_parent (self);
+  _gdk_macos_toplevel_surface_compute_size (surface);
+
+  /* Only set 'Resizable' mask to get native resize zones if the window is
+   * titled, otherwise we do this internally for CSD and do not need
+   * NSWindow to do it for us. Additionally, it can mess things up when
+   * doing a window resize since it can cause mouseDown to get passed
+   * through to the next window.
+   */
+  style_mask = [nswindow styleMask];
+  if (gdk_toplevel_layout_get_resizable (layout) &&
+      (style_mask & NSWindowStyleMaskTitled) != 0)
+    style_mask |= NSWindowStyleMaskResizable;
+  else
+    style_mask &= ~NSWindowStyleMaskResizable;
+
   if (style_mask != [nswindow styleMask])
     [nswindow setStyleMask:style_mask];
 
-  if (size.shadow.is_valid)
-    _gdk_macos_surface_set_shadow (GDK_MACOS_SURFACE (surface),
-                                   size.shadow.top,
-                                   size.shadow.right,
-                                   size.shadow.bottom,
-                                   size.shadow.left);
-
-  _gdk_macos_surface_set_geometry_hints (GDK_MACOS_SURFACE (self), &geometry, mask);
-  gdk_surface_constrain_size (&geometry, mask, width, height, &width, &height);
-
-  GDK_NOTE (MISC,
-            g_message ("Resizing \"%s\" to %dx%d",
-                       GDK_MACOS_SURFACE (self)->title ?
-                         GDK_MACOS_SURFACE (self)->title :
-                         "untitled",
-                       width, height));
-
-  _gdk_macos_surface_resize (GDK_MACOS_SURFACE (self), width, height);
-
   /* Maximized state */
   if (gdk_toplevel_layout_get_maximized (layout, &maximize))
     {
@@ -210,7 +261,30 @@ _gdk_macos_toplevel_surface_present (GdkToplevel       *toplevel,
   if (gdk_toplevel_layout_get_fullscreen (layout, &fullscreen))
     {
       if (fullscreen)
-        _gdk_macos_toplevel_surface_fullscreen (self);
+        {
+          GdkMonitor *fullscreen_monitor =
+            gdk_toplevel_layout_get_fullscreen_monitor (layout);
+
+          if (fullscreen_monitor)
+            {
+              int x = 0, y = 0;
+
+              _gdk_macos_display_position_surface (GDK_MACOS_DISPLAY (display),
+                                                   GDK_MACOS_SURFACE (self),
+                                                   fullscreen_monitor,
+                                                   &x, &y);
+
+              GDK_DEBUG (MISC, "Moving toplevel \"%s\" to %d,%d",
+                         GDK_MACOS_SURFACE (self)->title ?
+                         GDK_MACOS_SURFACE (self)->title :
+                         "untitled",
+                         x, y);
+
+              _gdk_macos_surface_move (GDK_MACOS_SURFACE (self), x, y);
+            }
+
+          _gdk_macos_toplevel_surface_fullscreen (self);
+        }
       else
         _gdk_macos_toplevel_surface_unfullscreen (self);
     }
@@ -223,14 +297,14 @@ _gdk_macos_toplevel_surface_present (GdkToplevel       *toplevel,
 
       _gdk_macos_display_position_surface (GDK_MACOS_DISPLAY (display),
                                            GDK_MACOS_SURFACE (self),
+                                           gdk_toplevel_layout_get_fullscreen_monitor (layout),
                                            &x, &y);
 
-      GDK_NOTE (MISC,
-                g_message ("Placing new toplevel \"%s\" at %d,%d",
-                           GDK_MACOS_SURFACE (self)->title ?
-                             GDK_MACOS_SURFACE (self)->title :
-                             "untitled",
-                           x, y));
+      GDK_DEBUG (MISC, "Placing new toplevel \"%s\" at %d,%d",
+                       GDK_MACOS_SURFACE (self)->title ?
+                       GDK_MACOS_SURFACE (self)->title :
+                       "untitled",
+                       x, y);
 
       _gdk_macos_surface_move (GDK_MACOS_SURFACE (self), x, y);
     }
@@ -379,125 +453,6 @@ _gdk_macos_toplevel_surface_hide (GdkSurface *surface)
   GDK_SURFACE_CLASS (_gdk_macos_toplevel_surface_parent_class)->hide (surface);
 }
 
-static gboolean
-_gdk_macos_toplevel_surface_compute_size (GdkSurface *surface)
-{
-  GdkMacosToplevelSurface *self = (GdkMacosToplevelSurface *)surface;
-  GdkMacosSurface *macos_surface = (GdkMacosSurface *)surface;
-  GdkToplevelSize size;
-  GdkDisplay *display;
-  GdkMonitor *monitor;
-  int bounds_width, bounds_height;
-  GdkGeometry geometry;
-  GdkSurfaceHints mask;
-
-  g_assert (GDK_IS_MACOS_TOPLEVEL_SURFACE (self));
-
-  if (!GDK_MACOS_SURFACE (surface)->geometry_dirty)
-    return FALSE;
-
-  GDK_MACOS_SURFACE (surface)->geometry_dirty = FALSE;
-
-  display = gdk_surface_get_display (surface);
-  monitor = gdk_display_get_monitor_at_surface (display, surface);
-
-  if (monitor)
-    {
-      GdkRectangle workarea;
-
-      gdk_macos_monitor_get_workarea (monitor, &workarea);
-      bounds_width = workarea.width;
-      bounds_height = workarea.height;
-    }
-  else
-    {
-      bounds_width = G_MAXINT;
-      bounds_height = G_MAXINT;
-    }
-
-  gdk_toplevel_size_init (&size, bounds_width, bounds_height);
-  gdk_toplevel_notify_compute_size (GDK_TOPLEVEL (surface), &size);
-
-  g_warn_if_fail (size.width > 0);
-  g_warn_if_fail (size.height > 0);
-
-  if (self->layout != NULL &&
-      gdk_toplevel_layout_get_resizable (self->layout))
-    {
-      geometry.min_width = size.min_width;
-      geometry.min_height = size.min_height;
-      mask = GDK_HINT_MIN_SIZE;
-    }
-  else
-    {
-      geometry.max_width = geometry.min_width = size.width;
-      geometry.max_height = geometry.min_height = size.height;
-      mask = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE;
-    }
-
-  if (size.shadow.is_valid)
-    _gdk_macos_surface_set_shadow (macos_surface,
-                                   size.shadow.top,
-                                   size.shadow.right,
-                                   size.shadow.bottom,
-                                   size.shadow.left);
-
-  _gdk_macos_surface_set_geometry_hints (macos_surface, &geometry, mask);
-
-  if (surface->state & (GDK_TOPLEVEL_STATE_FULLSCREEN |
-                        GDK_TOPLEVEL_STATE_MAXIMIZED |
-                        GDK_TOPLEVEL_STATE_TILED |
-                        GDK_TOPLEVEL_STATE_TOP_TILED |
-                        GDK_TOPLEVEL_STATE_RIGHT_TILED |
-                        GDK_TOPLEVEL_STATE_BOTTOM_TILED |
-                        GDK_TOPLEVEL_STATE_LEFT_TILED |
-                        GDK_TOPLEVEL_STATE_MINIMIZED) ||
-      [macos_surface->window inLiveResize])
-    return FALSE;
-
-  /* If we delayed a user resize until the beginning of the frame,
-   * apply it now so we can start processing updates for it.
-   */
-  if (macos_surface->next_layout.width > 0 &&
-      macos_surface->next_layout.height > 0)
-    {
-      int root_x = macos_surface->next_layout.root_x;
-      int root_y = macos_surface->next_layout.root_y;
-      int width = macos_surface->next_layout.width;
-      int height = macos_surface->next_layout.height;
-
-      gdk_surface_constrain_size (&geometry, mask,
-                                  width, height,
-                                  &width, &height);
-
-      macos_surface->next_layout.width = 0;
-      macos_surface->next_layout.height = 0;
-
-      _gdk_macos_surface_move_resize (macos_surface,
-                                      root_x, root_y,
-                                      width, height);
-
-      return FALSE;
-    }
-
-  gdk_surface_constrain_size (&geometry, mask,
-                              size.width, size.height,
-                              &size.width, &size.height);
-
-  if ((size.width != self->last_computed_width ||
-       size.height != self->last_computed_height) &&
-      (size.width != surface->width ||
-       size.height != surface->height))
-    {
-      self->last_computed_width = size.width;
-      self->last_computed_height = size.height;
-
-      _gdk_macos_surface_resize (macos_surface, size.width, size.height);
-    }
-
-  return FALSE;
-}
-
 static void
 _gdk_macos_toplevel_surface_request_layout (GdkSurface *surface)
 {
@@ -627,12 +582,60 @@ _gdk_macos_toplevel_surface_set_property (GObject      *object,
     }
 }
 
+static void
+_gdk_macos_toplevel_surface_constructed (GObject *object)
+{
+  GDK_BEGIN_MACOS_ALLOC_POOL;
+
+  GdkMacosWindow *window;
+  GdkMacosToplevelSurface *self = GDK_MACOS_TOPLEVEL_SURFACE (object);
+  GdkSurface *surface = GDK_SURFACE (self);
+  GdkMacosDisplay *display = GDK_MACOS_DISPLAY (gdk_surface_get_display (surface));
+  GdkFrameClock *frame_clock;
+  NSUInteger style_mask;
+  NSRect content_rect;
+  NSRect visible_frame;
+  NSScreen *screen;
+  int nx;
+  int ny;
+
+  style_mask = (NSWindowStyleMaskTitled |
+                NSWindowStyleMaskClosable |
+                NSWindowStyleMaskMiniaturizable |
+                NSWindowStyleMaskResizable);
+
+  _gdk_macos_display_to_display_coords (display, 0, 100, &nx, &ny);
+
+  screen = _gdk_macos_display_get_screen_at_display_coords (display, nx, ny);
+  visible_frame = [screen visibleFrame];
+  content_rect = NSMakeRect (nx - visible_frame.origin.x, ny - visible_frame.origin.y, 100, 100);
+  window = [[GdkMacosWindow alloc] initWithContentRect:content_rect
+                                             styleMask:style_mask
+                                               backing:NSBackingStoreBuffered
+                                                 defer:NO
+                                                screen:screen];
+
+  _gdk_macos_surface_set_native (GDK_MACOS_SURFACE (self), window);
+
+  /* Allow NSWindow to go fullscreen */
+  [window setCollectionBehavior:NSWindowCollectionBehaviorFullScreenPrimary];
+
+  frame_clock = _gdk_frame_clock_idle_new ();
+  gdk_surface_set_frame_clock (surface, frame_clock);
+  g_object_unref (frame_clock);
+
+  GDK_END_MACOS_ALLOC_POOL;
+
+  G_OBJECT_CLASS (_gdk_macos_toplevel_surface_parent_class)->constructed (object);
+}
+
 static void
 _gdk_macos_toplevel_surface_class_init (GdkMacosToplevelSurfaceClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GdkSurfaceClass *surface_class = GDK_SURFACE_CLASS (klass);
 
+  object_class->constructed = _gdk_macos_toplevel_surface_constructed;
   object_class->get_property = _gdk_macos_toplevel_surface_get_property;
   object_class->set_property = _gdk_macos_toplevel_surface_set_property;
 
@@ -650,66 +653,6 @@ _gdk_macos_toplevel_surface_init (GdkMacosToplevelSurface *self)
   self->decorated = TRUE;
 }
 
-GdkMacosSurface *
-_gdk_macos_toplevel_surface_new (GdkMacosDisplay *display,
-                                 GdkSurface      *parent,
-                                 GdkFrameClock   *frame_clock,
-                                 int              x,
-                                 int              y,
-                                 int              width,
-                                 int              height)
-{
-  GDK_BEGIN_MACOS_ALLOC_POOL;
-
-  GdkMacosWindow *window;
-  GdkMacosSurface *self;
-  NSUInteger style_mask;
-  NSRect content_rect;
-  NSRect visible_frame;
-  NSScreen *screen;
-  int nx;
-  int ny;
-
-  g_return_val_if_fail (GDK_IS_MACOS_DISPLAY (display), NULL);
-  g_return_val_if_fail (!frame_clock || GDK_IS_FRAME_CLOCK (frame_clock), NULL);
-  g_return_val_if_fail (!parent || GDK_IS_MACOS_SURFACE (parent), NULL);
-
-  style_mask = (NSWindowStyleMaskTitled |
-                NSWindowStyleMaskClosable |
-                NSWindowStyleMaskMiniaturizable |
-                NSWindowStyleMaskResizable);
-
-  if (parent != NULL)
-    {
-      x += GDK_MACOS_SURFACE (parent)->root_x;
-      y += GDK_MACOS_SURFACE (parent)->root_y;
-    }
-
-  _gdk_macos_display_to_display_coords (display, x, y + height, &nx, &ny);
-
-  screen = _gdk_macos_display_get_screen_at_display_coords (display, nx, ny);
-  visible_frame = [screen visibleFrame];
-  content_rect = NSMakeRect (nx - visible_frame.origin.x, ny - visible_frame.origin.y, width, height);
-  window = [[GdkMacosWindow alloc] initWithContentRect:content_rect
-                                             styleMask:style_mask
-                                               backing:NSBackingStoreBuffered
-                                                 defer:NO
-                                                screen:screen];
-
-  /* Allow NSWindow to go fullscreen */
-  [window setCollectionBehavior:NSWindowCollectionBehaviorFullScreenPrimary];
-
-  self = g_object_new (GDK_TYPE_MACOS_TOPLEVEL_SURFACE,
-                       "display", display,
-                       "frame-clock", frame_clock,
-                       "native", window,
-                       NULL);
-
-  GDK_END_MACOS_ALLOC_POOL;
-
-  return g_steal_pointer (&self);
-}
-
 void
 _gdk_macos_toplevel_surface_attach_to_parent (GdkMacosToplevelSurface *self)
 {
@@ -738,6 +681,7 @@ _gdk_macos_toplevel_surface_attach_to_parent (GdkMacosToplevelSurface *self)
       _gdk_macos_display_clear_sorting (GDK_MACOS_DISPLAY (surface->display));
       _gdk_macos_display_position_surface (GDK_MACOS_DISPLAY (surface->display),
                                            GDK_MACOS_SURFACE (surface),
+                                           NULL,
                                            &x, &y);
       _gdk_macos_surface_move (GDK_MACOS_SURFACE (surface), x, y);
     }
