@@ -48,12 +48,12 @@ G_DEFINE_TYPE (GdkWaylandGLContext, gdk_wayland_gl_context, GDK_TYPE_GL_CONTEXT)
 
 static void
 gdk_wayland_gl_context_begin_frame (GdkDrawContext *draw_context,
-                                    gboolean        prefers_high_depth,
+                                    GdkMemoryDepth  depth,
                                     cairo_region_t *region)
 {
   gdk_wayland_surface_ensure_wl_egl_window (gdk_draw_context_get_surface (draw_context));
 
-  GDK_DRAW_CONTEXT_CLASS (gdk_wayland_gl_context_parent_class)->begin_frame (draw_context, prefers_high_depth, region);
+  GDK_DRAW_CONTEXT_CLASS (gdk_wayland_gl_context_parent_class)->begin_frame (draw_context, depth, region);
 }
 
 static void
@@ -75,11 +75,29 @@ gdk_wayland_gl_context_end_frame (GdkDrawContext *draw_context,
       WL_SURFACE_OFFSET_SINCE_VERSION)
     wl_surface_offset (impl->display_server.wl_surface, dx, dy);
 
+  /* We should do this when setting up the EGLSurface, but we don't make_current then */
+  eglSwapInterval (gdk_display_get_egl_display (gdk_draw_context_get_display (draw_context)), 0);
+
   GDK_DRAW_CONTEXT_CLASS (gdk_wayland_gl_context_parent_class)->end_frame (draw_context, painted);
 
   gdk_wayland_surface_notify_committed (surface);
 }
 
+static void
+gdk_wayland_gl_context_empty_frame (GdkDrawContext *draw_context)
+{
+  GdkSurface *surface = gdk_draw_context_get_surface (draw_context);
+
+  if (gdk_wayland_surface_needs_commit (surface))
+    {
+      gdk_wayland_surface_sync (surface);
+      gdk_wayland_surface_request_frame (surface);
+
+      gdk_wayland_surface_commit (surface);
+      gdk_wayland_surface_notify_committed (surface);
+    }
+}
+
 static void
 gdk_wayland_gl_context_class_init (GdkWaylandGLContextClass *klass)
 {
@@ -88,6 +106,7 @@ gdk_wayland_gl_context_class_init (GdkWaylandGLContextClass *klass)
 
   draw_context_class->begin_frame = gdk_wayland_gl_context_begin_frame;
   draw_context_class->end_frame = gdk_wayland_gl_context_end_frame;
+  draw_context_class->empty_frame = gdk_wayland_gl_context_empty_frame;
 
   context_class->backend_type = GDK_GL_EGL;
 }
