@@ -1145,7 +1145,6 @@ fetch_net_wm_check_window (GdkX11Screen *x11_screen)
 {
   GdkDisplay *display;
   Window window;
-  guint64 now;
   int error;
 
   display = x11_screen->display;
@@ -1155,11 +1154,6 @@ fetch_net_wm_check_window (GdkX11Screen *x11_screen)
   if (x11_screen->wmspec_check_window != None)
     return; /* already have it */
 
-  now = g_get_monotonic_time ();
-
-  if ((now - x11_screen->last_wmspec_check_time) / 1e6 < 15)
-    return; /* we've checked recently */
-
   window = get_net_supporting_wm_check (x11_screen, x11_screen->xroot_window);
   if (window == None)
     return;
@@ -1184,7 +1178,6 @@ fetch_net_wm_check_window (GdkX11Screen *x11_screen)
         return;
 
       x11_screen->wmspec_check_window = window;
-      x11_screen->last_wmspec_check_time = now;
       x11_screen->need_refetch_net_supported = TRUE;
       x11_screen->need_refetch_wm_name = TRUE;
 
