@@ -105,7 +105,7 @@
  * ```
  *
  * This samples a texture (e.g. u_texture1) at the specified
- * coordinates, and containes some helper ifdefs to ensure that
+ * coordinates, and contains some helper ifdefs to ensure that
  * it works on all OpenGL versions.
  *
  * You can compile the shader yourself using [method@Gsk.GLShader.compile],
@@ -444,9 +444,7 @@ gsk_gl_shader_class_init (GskGLShaderClass *klass)
    * The source code for the shader, as a `GBytes`.
    */
   gsk_gl_shader_properties[GLSHADER_PROP_SOURCE] =
-    g_param_spec_boxed ("source",
-                        "Source",
-                        "The sourcecode for the shader",
+    g_param_spec_boxed ("source", NULL, NULL,
                         G_TYPE_BYTES,
                         G_PARAM_READWRITE |
                         G_PARAM_CONSTRUCT_ONLY |
@@ -461,9 +459,7 @@ gsk_gl_shader_class_init (GskGLShaderClass *klass)
    * will be %NULL.
    */
   gsk_gl_shader_properties[GLSHADER_PROP_RESOURCE] =
-    g_param_spec_string ("resource",
-                         "Resources",
-                         "Resource path to the source code",
+    g_param_spec_string ("resource", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY |
@@ -635,6 +631,7 @@ gsk_gl_shader_get_uniform_name (GskGLShader *shader,
                                 int          idx)
 {
   g_return_val_if_fail (GSK_IS_GL_SHADER (shader), NULL);
+  g_return_val_if_fail (0 <= idx && idx < shader->uniforms->len, NULL);
 
   return g_array_index (shader->uniforms, GskGLUniform, idx).name;
 }
@@ -679,6 +676,7 @@ gsk_gl_shader_get_uniform_type (GskGLShader *shader,
                                 int          idx)
 {
   g_return_val_if_fail (GSK_IS_GL_SHADER (shader), 0);
+  g_return_val_if_fail (0 <= idx && idx < shader->uniforms->len, 0);
 
   return g_array_index (shader->uniforms, GskGLUniform, idx).type;
 }
@@ -697,6 +695,7 @@ gsk_gl_shader_get_uniform_offset (GskGLShader *shader,
                                   int          idx)
 {
   g_return_val_if_fail (GSK_IS_GL_SHADER (shader), 0);
+  g_return_val_if_fail (0 <= idx && idx < shader->uniforms->len, 0);
 
   return g_array_index (shader->uniforms, GskGLUniform, idx).offset;
 }
