@@ -24,6 +24,7 @@
 
 #include "gtkaccessibleprivate.h"
 #include "gtkaccessibleattributesetprivate.h"
+#include "gtkaccessibletext.h"
 
 G_BEGIN_DECLS
 
@@ -78,7 +79,8 @@ typedef enum {
   GTK_ACCESSIBLE_STATE_CHANGE_HIDDEN   = 1 << GTK_ACCESSIBLE_STATE_HIDDEN,
   GTK_ACCESSIBLE_STATE_CHANGE_INVALID  = 1 << GTK_ACCESSIBLE_STATE_INVALID,
   GTK_ACCESSIBLE_STATE_CHANGE_PRESSED  = 1 << GTK_ACCESSIBLE_STATE_PRESSED,
-  GTK_ACCESSIBLE_STATE_CHANGE_SELECTED = 1 << GTK_ACCESSIBLE_STATE_SELECTED
+  GTK_ACCESSIBLE_STATE_CHANGE_SELECTED = 1 << GTK_ACCESSIBLE_STATE_SELECTED,
+  GTK_ACCESSIBLE_STATE_CHANGE_VISITED = 1 << GTK_ACCESSIBLE_STATE_VISITED
 } GtkAccessibleStateChange;
 
 struct _GtkATContext
@@ -126,6 +128,18 @@ struct _GtkATContextClass
 
   void (* realize)       (GtkATContext *self);
   void (* unrealize)     (GtkATContext *self);
+
+  void (* announce)      (GtkATContext *self,
+                          const char   *message,
+                          GtkAccessibleAnnouncementPriority priority);
+
+  /* Text interface */
+  void (* update_caret_position) (GtkATContext *self);
+  void (* update_selection_bound) (GtkATContext *self);
+  void (* update_text_contents) (GtkATContext *self,
+                                 GtkAccessibleTextContentChange change,
+                                 unsigned int start,
+                                 unsigned int end);
 };
 
 GtkATContext *          gtk_at_context_clone                    (GtkATContext          *self,
@@ -192,4 +206,17 @@ void
 gtk_at_context_set_next_accessible_sibling (GtkATContext *self,
                                             GtkAccessible *sibling);
 
+void gtk_at_context_announce (GtkATContext                     *self,
+                              const char                       *message,
+                              GtkAccessibleAnnouncementPriority priority);
+void
+gtk_at_context_update_caret_position (GtkATContext *self);
+void
+gtk_at_context_update_selection_bound (GtkATContext *self);
+void
+gtk_at_context_update_text_contents (GtkATContext *self,
+                                     GtkAccessibleTextContentChange change,
+                                     unsigned int start,
+                                     unsigned int end);
+
 G_END_DECLS
