@@ -19,7 +19,6 @@
 #include "gtkcenterlayout.h"
 
 #include "gtkcsspositionvalueprivate.h"
-#include "gtkintl.h"
 #include "gtklayoutchild.h"
 #include "gtkprivate.h"
 #include "gtksizerequest.h"
@@ -43,6 +42,7 @@ struct _GtkCenterLayout
 
   GtkBaselinePosition baseline_pos;
   GtkOrientation orientation;
+  gboolean shrink_center_last;
 
   union {
     struct {
@@ -54,6 +54,14 @@ struct _GtkCenterLayout
   };
 };
 
+enum {
+  PROP_0,
+  PROP_SHRINK_CENTER_LAST,
+  LAST_PROP
+};
+
+static GParamSpec *props[LAST_PROP] = { NULL, };
+
 G_DEFINE_TYPE (GtkCenterLayout, gtk_center_layout, GTK_TYPE_LAYOUT_MANAGER)
 
 static int
@@ -98,16 +106,6 @@ gtk_center_layout_get_request_mode (GtkLayoutManager *layout_manager,
            : GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
 }
 
-static gboolean
-get_expand (GtkWidget      *widget,
-            GtkOrientation  orientation)
-{
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    return gtk_widget_get_hexpand (widget);
-  else
-    return gtk_widget_get_vexpand (widget);
-}
-
 static void
 gtk_center_layout_distribute (GtkCenterLayout  *self,
                               int               for_size,
@@ -147,22 +145,31 @@ gtk_center_layout_distribute (GtkCenterLayout  *self,
 
   if (self->center_widget)
     {
-      center_size = CLAMP (size - needed_spacing - (sizes[0].minimum_size + sizes[2].minimum_size), sizes[1].minimum_size, sizes[1].natural_size);
-      center_expand = get_expand (self->center_widget, self->orientation);
+      int natural_size;
+
+      avail = size - needed_spacing - (sizes[0].minimum_size + sizes[2].minimum_size);
+
+      if (self->shrink_center_last)
+        natural_size = sizes[1].natural_size;
+      else
+        natural_size = CLAMP (size - needed_spacing - (sizes[0].natural_size + sizes[2].natural_size), sizes[1].minimum_size, sizes[1].natural_size);
+
+      center_size = CLAMP (avail, sizes[1].minimum_size, natural_size);
+      center_expand = gtk_widget_compute_expand (self->center_widget, self->orientation);
     }
 
   if (self->start_widget)
     {
-      avail = MIN ((size - needed_spacing - center_size) / 2, size - needed_spacing - (center_size + sizes[2].minimum_size));
+      avail = size - needed_spacing - (center_size + sizes[2].minimum_size);
       start_size = CLAMP (avail, sizes[0].minimum_size, sizes[0].natural_size);
-      start_expand = get_expand (self->start_widget, self->orientation);
+      start_expand = gtk_widget_compute_expand (self->start_widget, self->orientation);
     }
 
    if (self->end_widget)
     {
-      avail = MIN ((size - needed_spacing - center_size) / 2, size - needed_spacing - (center_size + sizes[0].minimum_size));
+      avail = size - needed_spacing - (center_size + sizes[0].minimum_size);
       end_size = CLAMP (avail, sizes[2].minimum_size, sizes[2].natural_size);
-      end_expand = get_expand (self->end_widget, self->orientation);
+      end_expand = gtk_widget_compute_expand (self->end_widget, self->orientation);
     }
 
   if (self->center_widget)
@@ -275,6 +282,8 @@ gtk_center_layout_measure_opposite (GtkCenterLayout *self,
   int total_nat, above_nat, below_nat;
   GtkWidget *child[3];
   GtkRequestedSize sizes[3];
+  gboolean have_baseline = FALSE;
+  gboolean align_baseline = FALSE;
   int i;
 
   child[0] = self->start_widget;
@@ -298,27 +307,33 @@ gtk_center_layout_measure_opposite (GtkCenterLayout *self,
                           &child_min, &child_nat,
                           &child_min_baseline, &child_nat_baseline);
 
-      if (child_min_baseline >= 0)
+      total_min = MAX (total_min, child_min);
+      total_nat = MAX (total_nat, child_nat);
+
+      if (orientation == GTK_ORIENTATION_VERTICAL && child_min_baseline >= 0)
         {
+          have_baseline = TRUE;
+          if (gtk_widget_get_valign (child[i]) == GTK_ALIGN_BASELINE_FILL ||
+              gtk_widget_get_valign (child[i]) == GTK_ALIGN_BASELINE_CENTER)
+            align_baseline = TRUE;
+
           below_min = MAX (below_min, child_min - child_min_baseline);
           above_min = MAX (above_min, child_min_baseline);
           below_nat = MAX (below_nat, child_nat - child_nat_baseline);
           above_nat = MAX (above_nat, child_nat_baseline);
         }
-      else
-        {
-          total_min = MAX (total_min, child_min);
-          total_nat = MAX (total_nat, child_nat);
-        }
    }
 
-  if (above_min >= 0)
+  if (have_baseline)
     {
       int min_baseline = -1;
       int nat_baseline = -1;
 
-      total_min = MAX (total_min, above_min + below_min);
-      total_nat = MAX (total_nat, above_nat + below_nat);
+      if (align_baseline)
+        {
+          total_min = MAX (total_min, above_min + below_min);
+          total_nat = MAX (total_nat, above_nat + below_nat);
+        }
 
       switch (self->baseline_pos)
         {
@@ -438,7 +453,9 @@ gtk_center_layout_allocate (GtkLayoutManager *layout_manager,
 
       for (i = 0; i < 3; i++)
         {
-          if (child[i] && gtk_widget_get_valign (child[i]) == GTK_ALIGN_BASELINE)
+          if (child[i] &&
+              (gtk_widget_get_valign (child[i]) == GTK_ALIGN_BASELINE_FILL ||
+               gtk_widget_get_valign (child[i]) == GTK_ALIGN_BASELINE_CENTER))
             {
               int child_min_height, child_nat_height;
               int child_min_baseline, child_nat_baseline;
@@ -523,14 +540,92 @@ gtk_center_layout_allocate (GtkLayoutManager *layout_manager,
     }
 }
 
+static void
+gtk_center_layout_set_property (GObject      *object,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
+{
+  GtkCenterLayout *self = GTK_CENTER_LAYOUT (object);
+
+  switch (prop_id)
+    {
+    case PROP_SHRINK_CENTER_LAST:
+      gtk_center_layout_set_shrink_center_last (self, g_value_get_boolean (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+gtk_center_layout_get_property (GObject    *object,
+                                guint       prop_id,
+                                GValue     *value,
+                                GParamSpec *pspec)
+{
+  GtkCenterLayout *self = GTK_CENTER_LAYOUT (object);
+
+  switch (prop_id)
+    {
+    case PROP_SHRINK_CENTER_LAST:
+      g_value_set_boolean (value, self->shrink_center_last);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+gtk_center_layout_dispose (GObject *object)
+{
+  GtkCenterLayout *self = GTK_CENTER_LAYOUT (object);
+
+  g_clear_object (&self->start_widget);
+  g_clear_object (&self->center_widget);
+  g_clear_object (&self->end_widget);
+
+  G_OBJECT_CLASS (gtk_center_layout_parent_class)->dispose (object);
+}
+
 static void
 gtk_center_layout_class_init (GtkCenterLayoutClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkLayoutManagerClass *layout_class = GTK_LAYOUT_MANAGER_CLASS (klass);
 
+  object_class->get_property = gtk_center_layout_get_property;
+  object_class->set_property = gtk_center_layout_set_property;
+  object_class->dispose = gtk_center_layout_dispose;
+
   layout_class->get_request_mode = gtk_center_layout_get_request_mode;
   layout_class->measure = gtk_center_layout_measure;
   layout_class->allocate = gtk_center_layout_allocate;
+
+  /**
+   * GtkCenterLayout:shrink-center-last: (attributes org.gtk.Property.get=gtk_center_layout_get_shrink_center_last org.gtk.Property.set=gtk_center_layout_set_shrink_center_last)
+   *
+   * Whether to shrink the center widget after other children.
+   *
+   * By default, when there's no space to give all three children their
+   * natural widths, the start and end widgets start shrinking and the
+   * center child keeps natural width until they reach minimum width.
+   *
+   * If set to `FALSE`, start and end widgets keep natural width and the
+   * center widget starts shrinking instead.
+   *
+   * Since: 4.12
+   */
+  props[PROP_SHRINK_CENTER_LAST] =
+      g_param_spec_boolean ("shrink-center-last", NULL, NULL,
+                            TRUE,
+                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
+
+  g_object_class_install_properties (object_class, LAST_PROP, props);
 }
 
 static void
@@ -538,6 +633,7 @@ gtk_center_layout_init (GtkCenterLayout *self)
 {
   self->orientation = GTK_ORIENTATION_HORIZONTAL;
   self->baseline_pos = GTK_BASELINE_POSITION_CENTER;
+  self->shrink_center_last = TRUE;
 }
 
 /**
@@ -641,18 +737,15 @@ gtk_center_layout_set_start_widget (GtkCenterLayout *self,
   g_return_if_fail (GTK_IS_CENTER_LAYOUT (self));
   g_return_if_fail (widget == NULL || GTK_IS_WIDGET (widget));
 
-  if (self->start_widget == widget)
-    return;
-
-  self->start_widget = widget;
-  gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
+  if (g_set_object (&self->start_widget, widget))
+    gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
 }
 
 /**
  * gtk_center_layout_get_start_widget:
  * @self: a `GtkCenterLayout`
  *
- * Returns the start widget fo the layout.
+ * Returns the start widget of the layout.
  *
  * Returns: (nullable) (transfer none): The current start widget of @self
  */
@@ -680,11 +773,8 @@ gtk_center_layout_set_center_widget (GtkCenterLayout *self,
   g_return_if_fail (GTK_IS_CENTER_LAYOUT (self));
   g_return_if_fail (widget == NULL || GTK_IS_WIDGET (widget));
 
-  if (self->center_widget == widget)
-    return;
-
-  self->center_widget = widget;
-  gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
+  if (g_set_object (&self->center_widget, widget))
+    gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
 }
 
 /**
@@ -706,7 +796,7 @@ gtk_center_layout_get_center_widget (GtkCenterLayout *self)
 /**
  * gtk_center_layout_set_end_widget:
  * @self: a `GtkCenterLayout`
- * @widget: (nullable) (transfer none): the new end widget
+ * @widget: (nullable): the new end widget
  *
  * Sets the new end widget of @self.
  *
@@ -719,11 +809,8 @@ gtk_center_layout_set_end_widget (GtkCenterLayout *self,
   g_return_if_fail (GTK_IS_CENTER_LAYOUT (self));
   g_return_if_fail (widget == NULL || GTK_IS_WIDGET (widget));
 
-  if (self->end_widget == widget)
-    return;
-
-  self->end_widget = widget;
-  gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
+  if (g_set_object (&self->end_widget, widget))
+    gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
 }
 
 /**
@@ -741,3 +828,55 @@ gtk_center_layout_get_end_widget (GtkCenterLayout *self)
 
   return self->end_widget;
 }
+
+/**
+ * gtk_center_layout_set_shrink_center_last: (attributes org.gtk.Method.set_property=shrink-center-last)
+ * @self: a `GtkCenterLayout`
+ * @shrink_center_last: whether to shrink the center widget after others
+ *
+ * Sets whether to shrink the center widget after other children.
+ *
+ * By default, when there's no space to give all three children their
+ * natural widths, the start and end widgets start shrinking and the
+ * center child keeps natural width until they reach minimum width.
+ *
+ * If set to `FALSE`, start and end widgets keep natural width and the
+ * center widget starts shrinking instead.
+ *
+ * Since: 4.12
+ */
+void
+gtk_center_layout_set_shrink_center_last (GtkCenterLayout *self,
+                                          gboolean         shrink_center_last)
+{
+  g_return_if_fail (GTK_IS_CENTER_LAYOUT (self));
+
+  shrink_center_last = !!shrink_center_last;
+
+  if (shrink_center_last == self->shrink_center_last)
+    return;
+
+  self->shrink_center_last = shrink_center_last;
+
+  gtk_layout_manager_layout_changed (GTK_LAYOUT_MANAGER (self));
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SHRINK_CENTER_LAST]);
+}
+
+/**
+ * gtk_center_layout_get_shrink_center_last: (attributes org.gtk.Method.get_property=shrink-center-last)
+ * @self: a `GtkCenterLayout`
+ *
+ * Gets whether @self shrinks the center widget after other children.
+ *
+ * Returns: whether to shrink the center widget after others
+ *
+ * Since: 4.12
+ */
+gboolean
+gtk_center_layout_get_shrink_center_last (GtkCenterLayout *self)
+{
+  g_return_val_if_fail (GTK_IS_CENTER_LAYOUT (self), FALSE);
+
+  return self->shrink_center_last;
+}
