@@ -25,6 +25,7 @@
 
 #include "gtkcomposetable.h"
 #include "gtkimcontextsimple.h"
+#include "gtkprivate.h"
 
 
 #define GTK_COMPOSE_TABLE_MAGIC "GtkComposeTable"
@@ -230,7 +231,7 @@ parse_compose_sequence (const char *seq,
       char *start = words[i];
       char *end = strchr (words[i], '>');
       char *match;
-      gunichar codepoint;
+      guint keyval;
 
       if (words[i][0] == '\0')
              continue;
@@ -247,18 +248,24 @@ parse_compose_sequence (const char *seq,
 
       if (is_codepoint (match))
         {
-          codepoint = (gunichar) g_ascii_strtoll (match + 1, NULL, 16);
-          sequence[n] = codepoint;
+          keyval = gdk_unicode_to_keyval ((gunichar) g_ascii_strtoll (match + 1, NULL, 16));
+          if (keyval > 0xffff)
+            g_warning ("Can't handle >16bit keyvals");
+
+          sequence[n] = (guint16) keyval;
           sequence[n + 1] = 0;
         }
       else
         {
-          codepoint = (gunichar) gdk_keyval_from_name (match);
-          sequence[n] = codepoint;
+          keyval = gdk_keyval_from_name (match);
+          if (keyval > 0xffff)
+            g_warning ("Can't handle >16bit keyvals");
+
+          sequence[n] = (guint16) keyval;
           sequence[n + 1] = 0;
         }
 
-      if (codepoint == GDK_KEY_VoidSymbol)
+      if (keyval == GDK_KEY_VoidSymbol)
         g_warning ("Could not get code point of keysym %s", match);
       g_free (match);
       n++;
@@ -284,6 +291,20 @@ fail:
 static void parser_parse_file (GtkComposeParser *parser,
                                const char       *path);
 
+char *
+gtk_compose_table_get_x11_compose_file_dir (void)
+{
+  char * compose_file_dir;
+
+#if defined (X11_DATA_PREFIX)
+  compose_file_dir = g_strdup (X11_DATA_PREFIX "/share/X11/locale");
+#else
+  compose_file_dir = g_build_filename (_gtk_get_datadir (), "X11", "locale", NULL);
+#endif
+
+  return compose_file_dir;
+}
+
 /* Substitute %H, %L and %S */
 static char *
 handle_substitutions (const char *start,
@@ -305,6 +326,9 @@ handle_substitutions (const char *start,
         }
       else
         {
+          char *x11_compose_file_dir;
+          char *path;
+
           switch (p[1])
             {
             case 'H':
@@ -313,11 +337,17 @@ handle_substitutions (const char *start,
               break;
             case 'L':
               p++;
-              g_string_append_printf (s, "/usr/share/X11/locale/%s/Compose", locale_name);
+              x11_compose_file_dir = gtk_compose_table_get_x11_compose_file_dir ();
+              path = g_build_filename (x11_compose_file_dir, locale_name, "Compose", NULL);
+              g_string_append (s, path);
+              g_free (path);
+              g_free (x11_compose_file_dir);
               break;
             case 'S':
               p++;
-              g_string_append (s, "/usr/share/X11/locale");
+              x11_compose_file_dir = gtk_compose_table_get_x11_compose_file_dir ();
+              g_string_append (s, x11_compose_file_dir);
+              g_free (x11_compose_file_dir);
               break;
             default: ;
               /* do nothing, next iteration handles p[1] */
@@ -939,6 +969,7 @@ parser_get_compose_table (GtkComposeParser *parser)
           if (char_data->len >= 0x8000)
             {
               g_warning ("GTK can't handle compose tables this large (%s)", parser->compose_file ? parser->compose_file : "");
+              g_free (data);
               g_string_free (char_data, TRUE);
               return NULL;
             }
@@ -955,6 +986,7 @@ parser_get_compose_table (GtkComposeParser *parser)
           current_first = (guint16)sequence[0];
 
           data[first_pos] = (guint16)sequence[0];
+
           for (i = 1; i < index_rowstride; i++)
             data[first_pos + i] = rest_pos;
         }
@@ -973,6 +1005,14 @@ parser_get_compose_table (GtkComposeParser *parser)
 
       rest_pos += len;
 
+      if (rest_pos >= 0x8000)
+        {
+          g_warning ("GTK can't handle compose tables this large (%s)", parser->compose_file ? parser->compose_file : "");
+          g_free (data);
+          g_string_free (char_data, TRUE);
+          return NULL;
+        }
+
       for (i = len + 1; i < index_rowstride; i++)
         data[first_pos + i] = rest_pos;
 
@@ -1297,12 +1337,27 @@ gtk_compose_table_check (const GtkComposeTable *table,
   if (!seq_index)
     return FALSE;
 
-  if (n_compose == 1)
-    return TRUE;
-
-  seq = NULL;
   match = FALSE;
-  value = 0;
+
+  if (n_compose == 1)
+    {
+      if (seq_index[2] - seq_index[1] > 0)
+        {
+          seq = table->data + seq_index[1];
+
+          value = seq[0];
+
+          if ((value & (1 << 15)) != 0)
+            g_string_append (output, &table->char_data[value & ~(1 << 15)]);
+          else
+            g_string_append_unichar (output, value);
+
+          if (compose_match)
+            *compose_match = TRUE;
+        }
+
+      return TRUE;
+    }
 
   for (i = n_compose - 1; i < table->max_seq_len; i++)
     {
@@ -1403,11 +1458,9 @@ gtk_compose_table_foreach (const GtkComposeTable      *table,
 {
   int index_stride = table->max_seq_len + 2;
   gunichar *sequence;
-  int seqno;
 
   sequence = g_newa (gunichar, table->max_seq_len + 1);
 
-  seqno = 0;
   for (int idx = 0; idx < table->n_index_size; idx++)
     {
       const guint16 *seq_index = table->data + (idx * index_stride);
@@ -1446,7 +1499,6 @@ gtk_compose_table_foreach (const GtkComposeTable      *table,
                 }
 
               callback (sequence, len, value, data);
-              seqno++;
             }
         }
     }
