@@ -162,9 +162,12 @@ gtk_entry_buffer_normal_insert_text (GtkEntryBuffer *buffer,
 
       /* Could be a password, so can't leave stuff in memory. */
       et_new = g_malloc (pv->normal_text_size);
-      memcpy (et_new, pv->normal_text, MIN (prev_size, pv->normal_text_size));
-      trash_area (pv->normal_text, prev_size);
-      g_free (pv->normal_text);
+      if (pv->normal_text)
+        {
+          memcpy (et_new, pv->normal_text, MIN (prev_size, pv->normal_text_size));
+          trash_area (pv->normal_text, prev_size);
+          g_free (pv->normal_text);
+        }
       pv->normal_text = et_new;
     }
 
@@ -388,6 +391,9 @@ gtk_entry_buffer_class_init (GtkEntryBufferClass *klass)
                                          G_TYPE_UINT,
                                          G_TYPE_STRING,
                                          G_TYPE_UINT);
+  g_signal_set_va_marshaller (signals[INSERTED_TEXT],
+                              GTK_TYPE_ENTRY_BUFFER,
+                              _gtk_marshal_VOID__UINT_STRING_UINTv);
 
   /**
    * GtkEntryBuffer::deleted-text:
@@ -409,6 +415,9 @@ gtk_entry_buffer_class_init (GtkEntryBufferClass *klass)
                                          G_TYPE_NONE, 2,
                                          G_TYPE_UINT,
                                          G_TYPE_UINT);
+  g_signal_set_va_marshaller (signals[DELETED_TEXT],
+                              GTK_TYPE_ENTRY_BUFFER,
+                              _gtk_marshal_VOID__UINT_UINTv);
 }
 
 /* --------------------------------------------------------------------------------
