@@ -23,6 +23,7 @@
 
 #include "gtkbitset.h"
 #include "gtkprivate.h"
+#include "gtksectionmodelprivate.h"
 
 /**
  * GtkFilterListModel:
@@ -33,9 +34,11 @@
  * It hides some elements from the other model according to
  * criteria given by a `GtkFilter`.
  *
- * The model can be set up to do incremental searching, so that
+ * The model can be set up to do incremental filtering, so that
  * filtering long lists doesn't block the UI. See
  * [method@Gtk.FilterListModel.set_incremental] for details.
+ *
+ * `GtkFilterListModel` passes through sections from the underlying model.
  */
 
 enum {
@@ -135,8 +138,100 @@ gtk_filter_list_model_model_init (GListModelInterface *iface)
   iface->get_item = gtk_filter_list_model_get_item;
 }
 
+static void
+gtk_filter_list_model_get_section (GtkSectionModel *model,
+                                   guint            position,
+                                   guint           *out_start,
+                                   guint           *out_end)
+{
+  GtkFilterListModel *self = GTK_FILTER_LIST_MODEL (model);
+  guint n_items;
+  guint pos, start, end;
+
+  switch (self->strictness)
+    {
+    case GTK_FILTER_MATCH_NONE:
+      *out_start = 0;
+      *out_end = G_MAXUINT;
+      return;
+
+    case GTK_FILTER_MATCH_ALL:
+      gtk_list_model_get_section (self->model, position, out_start, out_end);
+      return;
+
+    case GTK_FILTER_MATCH_SOME:
+      n_items = gtk_bitset_get_size (self->matches);
+      if (position >= n_items)
+        {
+          *out_start = n_items;
+          *out_end = G_MAXUINT;
+          return;
+        }
+      if (!GTK_IS_SECTION_MODEL (self->model))
+        {
+          *out_start = 0;
+          *out_end = n_items;
+          return;
+        }
+      break;
+
+    default:
+      g_assert_not_reached ();
+    }
+
+  /* if we get here, we have a section model, and are MATCH_SOME */
+
+  pos = gtk_bitset_get_nth (self->matches, position);
+  gtk_section_model_get_section (GTK_SECTION_MODEL (self->model), pos, &start, &end);
+  if (start == 0)
+    *out_start = 0;
+  else
+    *out_start = gtk_bitset_get_size_in_range (self->matches, 0, start - 1);
+  *out_end = *out_start + gtk_bitset_get_size_in_range (self->matches, start, end - 1);
+}
+
+static void
+gtk_filter_list_model_sections_changed_cb (GtkSectionModel *model,
+                                           unsigned int     position,
+                                           unsigned int     n_items,
+                                           gpointer         user_data)
+{
+  GtkFilterListModel *self = GTK_FILTER_LIST_MODEL (user_data);
+  unsigned int start, end;
+
+  switch (self->strictness)
+    {
+    case GTK_FILTER_MATCH_NONE:
+      return;
+
+    case GTK_FILTER_MATCH_ALL:
+      gtk_section_model_sections_changed (GTK_SECTION_MODEL (self), position, n_items);
+      break;
+
+    case GTK_FILTER_MATCH_SOME:
+      if (position > 0)
+        start = gtk_bitset_get_size_in_range (self->matches, 0, position - 1);
+      else
+        start = 0;
+      end = gtk_bitset_get_size_in_range (self->matches, 0, position + n_items - 1);
+      if (end - start > 0)
+        gtk_section_model_sections_changed (GTK_SECTION_MODEL (self), start, end - start);
+      break;
+
+    default:
+      g_assert_not_reached ();
+    }
+}
+
+static void
+gtk_filter_list_model_section_model_init (GtkSectionModelInterface *iface)
+{
+  iface->get_section = gtk_filter_list_model_get_section;
+}
+
 G_DEFINE_TYPE_WITH_CODE (GtkFilterListModel, gtk_filter_list_model, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, gtk_filter_list_model_model_init))
+                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, gtk_filter_list_model_model_init)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SECTION_MODEL, gtk_filter_list_model_section_model_init))
 
 static gboolean
 gtk_filter_list_model_run_filter_on_item (GtkFilterListModel *self,
@@ -145,7 +240,7 @@ gtk_filter_list_model_run_filter_on_item (GtkFilterListModel *self,
   gpointer item;
   gboolean visible;
 
-  /* all other cases should have beeen optimized away */
+  /* all other cases should have been optimized away */
   g_assert (self->strictness == GTK_FILTER_MATCH_SOME);
 
   item = g_list_model_get_item (self->model, position);
@@ -164,7 +259,7 @@ gtk_filter_list_model_run_filter (GtkFilterListModel *self,
   gboolean more;
 
   g_return_if_fail (GTK_IS_FILTER_LIST_MODEL (self));
-  
+
   if (self->pending == NULL)
     return;
 
@@ -180,9 +275,6 @@ gtk_filter_list_model_run_filter (GtkFilterListModel *self,
     gtk_bitset_remove_range_closed (self->pending, 0, pos - 1);
   else
     g_clear_pointer (&self->pending, gtk_bitset_unref);
-  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PENDING]);
-
-  return;
 }
 
 static void
@@ -237,6 +329,7 @@ gtk_filter_list_model_run_filter_cb (gpointer data)
     gtk_filter_list_model_stop_filtering (self);
 
   gtk_filter_list_model_emit_items_changed_for_changes (self, old);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PENDING]);
 
   return G_SOURCE_CONTINUE;
 }
@@ -355,7 +448,7 @@ gtk_filter_list_model_set_property (GObject      *object,
     }
 }
 
-static void 
+static void
 gtk_filter_list_model_get_property (GObject     *object,
                                     guint        prop_id,
                                     GValue      *value,
@@ -403,6 +496,7 @@ gtk_filter_list_model_clear_model (GtkFilterListModel *self)
 
   gtk_filter_list_model_stop_filtering (self);
   g_signal_handlers_disconnect_by_func (self->model, gtk_filter_list_model_items_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (self->model, gtk_filter_list_model_sections_changed_cb, self);
   g_clear_object (&self->model);
   if (self->matches)
     gtk_bitset_remove_all (self->matches);
@@ -503,7 +597,7 @@ gtk_filter_list_model_refilter (GtkFilterListModel *self,
     case GTK_FILTER_MATCH_SOME:
       {
         GtkBitset *old, *pending;
-      
+
         if (self->matches == NULL)
           {
             if (self->strictness == GTK_FILTER_MATCH_ALL)
@@ -767,6 +861,8 @@ gtk_filter_list_model_set_model (GtkFilterListModel *self,
     {
       self->model = g_object_ref (model);
       g_signal_connect (model, "items-changed", G_CALLBACK (gtk_filter_list_model_items_changed_cb), self);
+      if (GTK_IS_SECTION_MODEL (model))
+        g_signal_connect (model, "sections-changed", G_CALLBACK (gtk_filter_list_model_sections_changed_cb), self);
       if (removed == 0)
         {
           self->strictness = GTK_FILTER_MATCH_NONE;
@@ -857,6 +953,7 @@ gtk_filter_list_model_set_incremental (GtkFilterListModel *self,
       gtk_filter_list_model_stop_filtering (self);
 
       gtk_filter_list_model_emit_items_changed_for_changes (self, old);
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PENDING]);
     }
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_INCREMENTAL]);
