@@ -23,12 +23,12 @@
 
 #include "gtkbitset.h"
 #include "gtklistbaseprivate.h"
+#include "gtklistheaderwidgetprivate.h"
 #include "gtklistitemmanagerprivate.h"
-#include "gtkmain.h"
-#include "gtkprivate.h"
-#include "gtkrbtreeprivate.h"
-#include "gtkwidgetprivate.h"
+#include "gtklistitemwidgetprivate.h"
 #include "gtkmultiselection.h"
+#include "gtktypebuiltins.h"
+#include "gtkwidgetprivate.h"
 
 /* Maximum number of list items created by the listview.
  * For debugging, you can set this to G_MAXUINT to ensure
@@ -141,29 +141,16 @@
  * items use the %GTK_ACCESSIBLE_ROLE_LIST_ITEM role.
  */
 
-typedef struct _ListRow ListRow;
-typedef struct _ListRowAugment ListRowAugment;
-
-struct _ListRow
-{
-  GtkListItemManagerItem parent;
-  guint height; /* per row */
-};
-
-struct _ListRowAugment
-{
-  GtkListItemManagerItemAugment parent;
-  guint height; /* total */
-};
-
 enum
 {
   PROP_0,
+  PROP_ENABLE_RUBBERBAND,
   PROP_FACTORY,
+  PROP_HEADER_FACTORY,
   PROP_MODEL,
   PROP_SHOW_SEPARATORS,
   PROP_SINGLE_CLICK_ACTIVATE,
-  PROP_ENABLE_RUBBERBAND,
+  PROP_TAB_BEHAVIOR,
 
   N_PROPS
 };
@@ -178,226 +165,196 @@ G_DEFINE_TYPE (GtkListView, gtk_list_view, GTK_TYPE_LIST_BASE)
 static GParamSpec *properties[N_PROPS] = { NULL, };
 static guint signals[LAST_SIGNAL] = { 0 };
 
-static void G_GNUC_UNUSED
-dump (GtkListView *self)
+static GtkListTile *
+gtk_list_view_split (GtkListBase *base,
+                     GtkListTile *tile,
+                     guint        n_items)
 {
-  ListRow *row;
-  guint n_widgets, n_list_rows;
+  GtkListView *self = GTK_LIST_VIEW (base);
+  GtkListTile *new_tile;
+  int spacing, row_height;
 
-  n_widgets = 0;
-  n_list_rows = 0;
-  //g_print ("ANCHOR: %u - %u\n", self->anchor_start, self->anchor_end);
-  for (row = gtk_list_item_manager_get_first (self->item_manager);
-       row;
-       row = gtk_rb_tree_node_get_next (row))
-    {
-      if (row->parent.widget)
-        n_widgets++;
-      n_list_rows++;
-      g_print ("  %4u%s (%upx)\n", row->parent.n_items, row->parent.widget ? " (widget)" : "", row->height);
-    }
+  gtk_list_base_get_border_spacing (GTK_LIST_BASE (self), NULL, &spacing);
+  row_height = (tile->area.height - (tile->n_items - 1) * spacing) / tile->n_items;
 
-  g_print ("  => %u widgets in %u list rows\n", n_widgets, n_list_rows);
+  new_tile = gtk_list_tile_split (self->item_manager, tile, n_items);
+  gtk_list_tile_set_area_size (self->item_manager,
+                               tile,
+                               tile->area.width,
+                               row_height * tile->n_items + spacing * (tile->n_items - 1));
+  gtk_list_tile_set_area (self->item_manager,
+                          new_tile,
+                          &(GdkRectangle) {
+                            tile->area.x,
+                            tile->area.y + tile->area.height + spacing,
+                            tile->area.width,
+                            row_height * new_tile->n_items + spacing * (new_tile->n_items - 1)
+                          });
+
+  return new_tile;
 }
 
 static void
-list_row_augment (GtkRbTree *tree,
-                  gpointer   node_augment,
-                  gpointer   node,
-                  gpointer   left,
-                  gpointer   right)
+gtk_list_view_prepare_section (GtkListBase *base,
+                               GtkListTile *tile,
+                               guint        position)
 {
-  ListRow *row = node;
-  ListRowAugment *aug = node_augment;
-
-  gtk_list_item_manager_augment_node (tree, node_augment, node, left, right);
-
-  aug->height = row->height * row->parent.n_items;
-
-  if (left)
-    {
-      ListRowAugment *left_aug = gtk_rb_tree_get_augment (tree, left);
-
-      aug->height += left_aug->height;
-    }
-
-  if (right)
-    {
-      ListRowAugment *right_aug = gtk_rb_tree_get_augment (tree, right);
-
-      aug->height += right_aug->height;
-    }
 }
 
-static ListRow *
-gtk_list_view_get_row_at_y (GtkListView *self,
-                            int          y,
-                            int         *offset)
+/* We define the listview as **inert** when the factory isn't used. */
+static gboolean
+gtk_list_view_is_inert (GtkListView *self)
 {
-  ListRow *row, *tmp;
+  GtkWidget *widget = GTK_WIDGET (self);
 
-  row = gtk_list_item_manager_get_root (self->item_manager);
+  return !gtk_widget_get_visible (widget) ||
+         gtk_widget_get_root (widget) == NULL;
+}
 
-  while (row)
+static void
+gtk_list_view_update_factories_with (GtkListView        *self,
+                                     GtkListItemFactory *factory,
+                                     GtkListItemFactory *header_factory)
+{
+  GtkListTile *tile;
+
+  for (tile = gtk_list_item_manager_get_first (self->item_manager);
+       tile != NULL;
+       tile = gtk_rb_tree_node_get_next (tile))
     {
-      tmp = gtk_rb_tree_node_get_left (row);
-      if (tmp)
+      switch (tile->type)
         {
-          ListRowAugment *aug = gtk_list_item_manager_get_item_augment (self->item_manager, tmp);
-          if (y < aug->height)
-            {
-              row = tmp;
-              continue;
-            }
-          y -= aug->height;
+        case GTK_LIST_TILE_ITEM:
+          if (tile->widget)
+            gtk_list_factory_widget_set_factory (GTK_LIST_FACTORY_WIDGET (tile->widget), factory);
+          break;
+        case GTK_LIST_TILE_HEADER:
+          if (tile->widget)
+            gtk_list_header_widget_set_factory (GTK_LIST_HEADER_WIDGET (tile->widget), header_factory);
+          break;
+        case GTK_LIST_TILE_UNMATCHED_HEADER:
+        case GTK_LIST_TILE_FOOTER:
+        case GTK_LIST_TILE_UNMATCHED_FOOTER:
+        case GTK_LIST_TILE_REMOVED:
+          g_assert (tile->widget == NULL);
+          break;
+        default:
+          g_assert_not_reached();
+          break;
         }
-
-      if (y < row->height * row->parent.n_items)
-        break;
-      y -= row->height * row->parent.n_items;
-
-      row = gtk_rb_tree_node_get_right (row);
     }
-
-  if (offset)
-    *offset = row ? y : 0;
-
-  return row;
 }
 
-static int
-list_row_get_y (GtkListView *self,
-                ListRow     *row)
+static void
+gtk_list_view_update_factories (GtkListView *self)
 {
-  ListRow *parent, *left;
-  int y;
+  gtk_list_view_update_factories_with (self,
+                                       gtk_list_view_is_inert (self) ? NULL : self->factory,
+                                       gtk_list_view_is_inert (self) ? NULL : self->header_factory);
+}
 
-  left = gtk_rb_tree_node_get_left (row);
-  if (left)
-    {
-      ListRowAugment *aug = gtk_list_item_manager_get_item_augment (self->item_manager, left);
-      y = aug->height;
-    }
+static void
+gtk_list_view_clear_factories (GtkListView *self)
+{
+  gtk_list_view_update_factories_with (self, NULL, NULL);
+}
+
+static GtkListItemBase *
+gtk_list_view_create_list_widget (GtkListBase *base)
+{
+  GtkListView *self = GTK_LIST_VIEW (base);
+  GtkListItemFactory *factory;
+  GtkWidget *result;
+
+  if (gtk_list_view_is_inert (self))
+    factory = NULL;
   else
-    y = 0; 
+    factory = self->factory;
 
-  for (parent = gtk_rb_tree_node_get_parent (row);
-       parent != NULL;
-       parent = gtk_rb_tree_node_get_parent (row))
-    {
-      left = gtk_rb_tree_node_get_left (parent);
+  result = gtk_list_item_widget_new (factory,
+                                     "row",
+                                     GTK_ACCESSIBLE_ROLE_LIST_ITEM);
 
-      if (left != row)
-        {
-          if (left)
-            {
-              ListRowAugment *aug = gtk_list_item_manager_get_item_augment (self->item_manager, left);
-              y += aug->height;
-            }
-          y += parent->height * parent->parent.n_items;
-        }
+  gtk_list_factory_widget_set_single_click_activate (GTK_LIST_FACTORY_WIDGET (result), self->single_click_activate);
 
-      row = parent;
-    }
-
-  return y ;
+  return GTK_LIST_ITEM_BASE (result);
 }
 
-static int
-gtk_list_view_get_list_height (GtkListView *self)
+static GtkListHeaderBase *
+gtk_list_view_create_header_widget (GtkListBase *base)
 {
-  ListRow *row;
-  ListRowAugment *aug;
+  GtkListView *self = GTK_LIST_VIEW (base);
+  GtkListItemFactory *factory;
 
-  row = gtk_list_item_manager_get_root (self->item_manager);
-  if (row == NULL)
-    return 0;
+  if (gtk_list_view_is_inert (self))
+    factory = NULL;
+  else
+    factory = self->header_factory;
 
-  aug = gtk_list_item_manager_get_item_augment (self->item_manager, row);
-  return aug->height;
+  return GTK_LIST_HEADER_BASE (gtk_list_header_widget_new (factory));
 }
 
 static gboolean
-gtk_list_view_get_allocation_along (GtkListBase *base,
-                                    guint        pos,
-                                    int         *offset,
-                                    int         *size)
+gtk_list_view_get_allocation (GtkListBase  *base,
+                              guint         pos,
+                              GdkRectangle *area)
 {
   GtkListView *self = GTK_LIST_VIEW (base);
-  ListRow *row;
-  guint skip;
-  int y;
+  GtkListTile *tile;
+  guint offset;
 
-  row = gtk_list_item_manager_get_nth (self->item_manager, pos, &skip);
-  if (row == NULL)
+  tile = gtk_list_item_manager_get_nth (self->item_manager, pos, &offset);
+  if (tile == NULL)
+    return FALSE;
+
+  *area = tile->area;
+  if (area->width || area->height)
     {
+      if (tile->n_items)
+        area->height /= tile->n_items;
       if (offset)
-        *offset = 0;
-      if (size)
-        *size = 0;
-      return FALSE;
+        area->y += offset * area->height;
+    }
+  else
+    {
+      /* item is not allocated yet */
+      GtkListTile *other;
+      int spacing;
+
+      gtk_list_base_get_border_spacing (GTK_LIST_BASE (self), NULL, &spacing);
+
+      for (other = gtk_rb_tree_node_get_previous (tile);
+           other;
+           other = gtk_rb_tree_node_get_previous (other))
+        {
+          if (other->area.width || other->area.height)
+            {
+              area->x = other->area.x;
+              area->width = other->area.width;
+              area->y = other->area.y + other->area.height + spacing;
+              break;
+            }
+        }
+      if (other == NULL)
+        {
+          for (other = gtk_rb_tree_node_get_next (tile);
+               other;
+               other = gtk_rb_tree_node_get_next (other))
+            {
+              if (other->area.width || other->area.height)
+                {
+                  area->x = other->area.x;
+                  area->width = other->area.width;
+                  area->y = MAX (0, other->area.y - spacing);
+                  break;
+                }
+            }
+        }
     }
 
-  y = list_row_get_y (self, row);
-  y += skip * row->height;
-
-  if (offset)
-    *offset = y;
-  if (size)
-    *size = row->height;
-
   return TRUE;
 }
 
-static gboolean
-gtk_list_view_get_allocation_across (GtkListBase *base,
-                                     guint        pos,
-                                     int         *offset,
-                                     int         *size)
-{
-  GtkListView *self = GTK_LIST_VIEW (base);
-
-  if (offset)
-    *offset = 0;
-  if (size)
-    *size = self->list_width;
-
-  return TRUE;
-}
-
-static GtkBitset *
-gtk_list_view_get_items_in_rect (GtkListBase                 *base,
-                                 const cairo_rectangle_int_t *rect)
-{
-  GtkListView *self = GTK_LIST_VIEW (base);
-  guint first, last, n_items;
-  GtkBitset *result;
-  ListRow *row;
-
-  result = gtk_bitset_new_empty ();
-
-  if (rect->y >= gtk_list_view_get_list_height (self))
-    return result;
-
-  n_items = gtk_list_base_get_n_items (base);
-  if (n_items == 0)
-    return result;
-
-  row = gtk_list_view_get_row_at_y (self, rect->y, NULL);
-  if (row)
-    first = gtk_list_item_manager_get_item_position (self->item_manager, row);
-  else
-    first = rect->y < 0 ? 0 : n_items - 1;
-  row = gtk_list_view_get_row_at_y (self, rect->y + rect->height, NULL);
-  if (row)
-    last = gtk_list_item_manager_get_item_position (self->item_manager, row);
-  else
-    last = rect->y + rect->height < 0 ? 0 : n_items - 1;
-
-  gtk_bitset_add_range_closed (result, first, last);
-  return result;
-}
-
 static guint
 gtk_list_view_move_focus_along (GtkListBase *base,
                                 guint        pos,
@@ -415,39 +372,84 @@ gtk_list_view_move_focus_along (GtkListBase *base,
 
 static gboolean
 gtk_list_view_get_position_from_allocation (GtkListBase           *base,
-                                            int                    across,
-                                            int                    along,
+                                            int                    x,
+                                            int                    y,
                                             guint                 *pos,
                                             cairo_rectangle_int_t *area)
 {
   GtkListView *self = GTK_LIST_VIEW (base);
-  ListRow *row;
-  int remaining;
+  GtkListTile *tile;
 
-  if (across >= self->list_width)
+  tile = gtk_list_item_manager_get_nearest_tile (self->item_manager, x, y);
+  if (tile == NULL)
     return FALSE;
 
-  along = CLAMP (along, 0, gtk_list_view_get_list_height (self) - 1);
-
-  row = gtk_list_view_get_row_at_y (self, along, &remaining);
-  if (row == NULL)
-    return FALSE;
-
-  *pos = gtk_list_item_manager_get_item_position (self->item_manager, row);
-  g_assert (remaining < row->height * row->parent.n_items);
-  *pos += remaining / row->height;
-
-  if (area)
+  while (tile && tile->n_items == 0)
+    tile = gtk_rb_tree_node_get_previous (tile);
+  if (tile == NULL)
     {
-      area->x = 0;
-      area->width = self->list_width;
-      area->y = along - remaining % row->height;
-      area->height = row->height;
+      tile = gtk_list_item_manager_get_first (self->item_manager);
+      while (tile && tile->n_items == 0)
+        tile = gtk_rb_tree_node_get_next (tile);
+      if (tile == NULL)
+        return FALSE;
+    }
+
+  *pos = gtk_list_tile_get_position (self->item_manager, tile);
+  if (area)
+    *area = tile->area;
+
+  if (tile->n_items > 1)
+    {
+      int row_height, tile_pos, spacing;
+
+      gtk_list_base_get_border_spacing (GTK_LIST_BASE (self), NULL, &spacing);
+      row_height = (tile->area.height - (tile->n_items - 1) * spacing) / tile->n_items;
+      if (y >= tile->area.y + tile->area.height)
+        tile_pos = tile->n_items - 1;
+      else
+        tile_pos = (y - tile->area.y) / (row_height + spacing);
+
+      *pos += tile_pos;
+      if (area)
+        {
+          area->y = tile->area.y + tile_pos * (row_height + spacing);
+          area->height = row_height;
+        }
     }
 
   return TRUE;
 }
 
+static GtkBitset *
+gtk_list_view_get_items_in_rect (GtkListBase                 *base,
+                                 const cairo_rectangle_int_t *rect)
+{
+  guint first, last;
+  cairo_rectangle_int_t area;
+  GtkBitset *result;
+
+  result = gtk_bitset_new_empty ();
+
+  if (!gtk_list_view_get_position_from_allocation (base, rect->x, rect->y, &first, &area))
+    return result;
+  if (area.y + area.height < rect->y)
+    first++;
+
+  if (!gtk_list_view_get_position_from_allocation (base,
+                                                   rect->x + rect->width - 1,
+                                                   rect->y + rect->height - 1,
+                                                   &last, &area))
+    return result;
+  if (area.y >= rect->y + rect->height)
+    last--;
+
+  if (last >= first)
+    gtk_bitset_add_range_closed (result, first, last);
+
+  return result;
+}
+
 static guint
 gtk_list_view_move_focus_across (GtkListBase *base,
                                  guint        pos,
@@ -483,7 +485,7 @@ gtk_list_view_measure_across (GtkWidget      *widget,
                               int            *natural)
 {
   GtkListView *self = GTK_LIST_VIEW (widget);
-  ListRow *row;
+  GtkListTile *tile;
   int min, nat, child_min, child_nat;
   /* XXX: Figure out how to split a given height into per-row heights.
    * Good luck! */
@@ -492,15 +494,15 @@ gtk_list_view_measure_across (GtkWidget      *widget,
   min = 0;
   nat = 0;
 
-  for (row = gtk_list_item_manager_get_first (self->item_manager);
-       row != NULL;
-       row = gtk_rb_tree_node_get_next (row))
+  for (tile = gtk_list_item_manager_get_first (self->item_manager);
+       tile != NULL;
+       tile = gtk_rb_tree_node_get_next (tile))
     {
       /* ignore unavailable rows */
-      if (row->parent.widget == NULL)
+      if (tile->widget == NULL)
         continue;
 
-      gtk_widget_measure (row->parent.widget,
+      gtk_widget_measure (tile->widget,
                           orientation, for_size,
                           &child_min, &child_nat, NULL, NULL);
       min = MAX (min, child_min);
@@ -519,10 +521,15 @@ gtk_list_view_measure_list (GtkWidget      *widget,
                             int            *natural)
 {
   GtkListView *self = GTK_LIST_VIEW (widget);
-  ListRow *row;
-  int min, nat, child_min, child_nat;
+  GtkListTile *tile;
+  int min, nat, child_min, child_nat, spacing;
   GArray *min_heights, *nat_heights;
-  guint n_unknown;
+  guint n_unknown, n_items;
+
+  n_items = gtk_list_base_get_n_items (GTK_LIST_BASE (self));
+  if (n_items == 0)
+    return;
+  gtk_list_base_get_border_spacing (GTK_LIST_BASE (self), NULL, &spacing);
 
   min_heights = g_array_new (FALSE, FALSE, sizeof (int));
   nat_heights = g_array_new (FALSE, FALSE, sizeof (int));
@@ -530,23 +537,26 @@ gtk_list_view_measure_list (GtkWidget      *widget,
   min = 0;
   nat = 0;
 
-  for (row = gtk_list_item_manager_get_first (self->item_manager);
-       row != NULL;
-       row = gtk_rb_tree_node_get_next (row))
+  for (tile = gtk_list_item_manager_get_first (self->item_manager);
+       tile != NULL;
+       tile = gtk_rb_tree_node_get_next (tile))
     {
-      if (row->parent.widget)
+      if (tile->widget)
         {
-          gtk_widget_measure (row->parent.widget,
+          gtk_widget_measure (tile->widget,
                               orientation, for_size,
                               &child_min, &child_nat, NULL, NULL);
-          g_array_append_val (min_heights, child_min);
-          g_array_append_val (nat_heights, child_nat);
+          if (tile->type == GTK_LIST_TILE_ITEM)
+            {
+              g_array_append_val (min_heights, child_min);
+              g_array_append_val (nat_heights, child_nat);
+            }
           min += child_min;
           nat += child_nat;
         }
       else
         {
-          n_unknown += row->parent.n_items;
+          n_unknown += tile->n_items;
         }
     }
 
@@ -558,8 +568,8 @@ gtk_list_view_measure_list (GtkWidget      *widget,
   g_array_free (min_heights, TRUE);
   g_array_free (nat_heights, TRUE);
 
-  *minimum = min;
-  *natural = nat;
+  *minimum = min + spacing * (n_items - 1);
+  *natural = nat + spacing * (n_items - 1);
 }
 
 static void
@@ -586,10 +596,9 @@ gtk_list_view_size_allocate (GtkWidget *widget,
                              int        baseline)
 {
   GtkListView *self = GTK_LIST_VIEW (widget);
-  ListRow *row;
+  GtkListTile *tile;
   GArray *heights;
-  int min, nat, row_height;
-  int x, y;
+  int min, nat, row_height, y, list_width, spacing;
   GtkOrientation orientation, opposite_orientation;
   GtkScrollablePolicy scroll_policy, opposite_scroll_policy;
 
@@ -597,97 +606,118 @@ gtk_list_view_size_allocate (GtkWidget *widget,
   opposite_orientation = OPPOSITE_ORIENTATION (orientation);
   scroll_policy = gtk_list_base_get_scroll_policy (GTK_LIST_BASE (self), orientation);
   opposite_scroll_policy = gtk_list_base_get_scroll_policy (GTK_LIST_BASE (self), opposite_orientation);
+  gtk_list_base_get_border_spacing (GTK_LIST_BASE (self), NULL, &spacing);
+
+  gtk_list_item_manager_gc_tiles (self->item_manager);
 
   /* step 0: exit early if list is empty */
-  if (gtk_list_item_manager_get_root (self->item_manager) == NULL)
+  tile = gtk_list_item_manager_get_first (self->item_manager);
+  if (tile == NULL)
     {
-      gtk_list_base_update_adjustments (GTK_LIST_BASE (self), 0, 0, 0, 0, &x, &y);
+      gtk_list_base_allocate (GTK_LIST_BASE (self));
       return;
     }
 
   /* step 1: determine width of the list */
-  gtk_widget_measure (widget, opposite_orientation,
-                      -1,
-                      &min, &nat, NULL, NULL);
-  self->list_width = orientation == GTK_ORIENTATION_VERTICAL ? width : height;
+  gtk_list_view_measure_across (widget, opposite_orientation,
+                                -1,
+                                &min, &nat);
+  list_width = orientation == GTK_ORIENTATION_VERTICAL ? width : height;
   if (opposite_scroll_policy == GTK_SCROLL_MINIMUM)
-    self->list_width = MAX (min, self->list_width);
+    list_width = MAX (min, list_width);
   else
-    self->list_width = MAX (nat, self->list_width);
+    list_width = MAX (nat, list_width);
 
-  /* step 2: determine height of known list items */
+  /* step 2: determine height of known list items and gc the list */
   heights = g_array_new (FALSE, FALSE, sizeof (int));
 
-  for (row = gtk_list_item_manager_get_first (self->item_manager);
-       row != NULL;
-       row = gtk_rb_tree_node_get_next (row))
+  for (;
+       tile != NULL;
+       tile = gtk_rb_tree_node_get_next (tile))
     {
-      if (row->parent.widget == NULL)
+      if (tile->widget == NULL)
         continue;
 
-      gtk_widget_measure (row->parent.widget, orientation,
-                          self->list_width,
+      gtk_widget_measure (tile->widget, orientation,
+                          list_width,
                           &min, &nat, NULL, NULL);
       if (scroll_policy == GTK_SCROLL_MINIMUM)
         row_height = min;
       else
         row_height = nat;
-      if (row->height != row_height)
-        {
-          row->height = row_height;
-          gtk_rb_tree_node_mark_dirty (row);
-        }
-      g_array_append_val (heights, row_height);
+      gtk_list_tile_set_area_size (self->item_manager, tile, list_width, row_height);
+      if (tile->type == GTK_LIST_TILE_ITEM)
+        g_array_append_val (heights, row_height);
     }
 
-  /* step 3: determine height of unknown items */
+  /* step 3: determine height of unknown items and set the positions */
   row_height = gtk_list_view_get_unknown_row_height (self, heights);
   g_array_free (heights, TRUE);
 
-  for (row = gtk_list_item_manager_get_first (self->item_manager);
-       row != NULL;
-       row = gtk_rb_tree_node_get_next (row))
+  y = 0;
+  for (tile = gtk_list_item_manager_get_first (self->item_manager);
+       tile != NULL;
+       tile = gtk_rb_tree_node_get_next (tile))
     {
-      if (row->parent.widget)
-        continue;
-
-      if (row->height != row_height)
+      gtk_list_tile_set_area_position (self->item_manager, tile, 0, y);
+      if (tile->widget == NULL)
         {
-          row->height = row_height;
-          gtk_rb_tree_node_mark_dirty (row);
-        }
-    }
-
-  /* step 3: update the adjustments */
-  gtk_list_base_update_adjustments (GTK_LIST_BASE (self),
-                                    self->list_width,
-                                    gtk_list_view_get_list_height (self),
-                                    gtk_widget_get_size (widget, opposite_orientation),
-                                    gtk_widget_get_size (widget, orientation),
-                                    &x, &y);
-  x = -x;
-  y = -y;
-
-  /* step 4: actually allocate the widgets */
-
-  for (row = gtk_list_item_manager_get_first (self->item_manager);
-       row != NULL;
-       row = gtk_rb_tree_node_get_next (row))
-    {
-      if (row->parent.widget)
-        {
-          gtk_list_base_size_allocate_child (GTK_LIST_BASE (self),
-                                             row->parent.widget,
-                                             x,
-                                             y,
-                                             self->list_width,
-                                             row->height);
+          gtk_list_tile_set_area_size (self->item_manager,
+                                       tile,
+                                       list_width,
+                                       row_height * tile->n_items
+                                       + spacing * (tile->n_items - 1));
         }
 
-      y += row->height * row->parent.n_items;
+      y += tile->area.height + spacing;
     }
 
-  gtk_list_base_allocate_rubberband (GTK_LIST_BASE (self));
+  /* step 4: allocate the rest */
+  gtk_list_base_allocate (GTK_LIST_BASE (self));
+}
+
+static void
+gtk_list_view_root (GtkWidget *widget)
+{
+  GtkListView *self = GTK_LIST_VIEW (widget);
+
+  GTK_WIDGET_CLASS (gtk_list_view_parent_class)->root (widget);
+
+  if (!gtk_list_view_is_inert (self))
+    gtk_list_view_update_factories (self);
+}
+
+static void
+gtk_list_view_unroot (GtkWidget *widget)
+{
+  GtkListView *self = GTK_LIST_VIEW (widget);
+
+  if (!gtk_list_view_is_inert (self))
+    gtk_list_view_clear_factories (self);
+
+  GTK_WIDGET_CLASS (gtk_list_view_parent_class)->unroot (widget);
+}
+
+static void
+gtk_list_view_show (GtkWidget *widget)
+{
+  GtkListView *self = GTK_LIST_VIEW (widget);
+
+  GTK_WIDGET_CLASS (gtk_list_view_parent_class)->show (widget);
+
+  if (!gtk_list_view_is_inert (self))
+    gtk_list_view_update_factories (self);
+}
+
+static void
+gtk_list_view_hide (GtkWidget *widget)
+{
+  GtkListView *self = GTK_LIST_VIEW (widget);
+
+  if (!gtk_list_view_is_inert (self))
+    gtk_list_view_clear_factories (self);
+
+  GTK_WIDGET_CLASS (gtk_list_view_parent_class)->hide (widget);
 }
 
 static void
@@ -697,6 +727,9 @@ gtk_list_view_dispose (GObject *object)
 
   self->item_manager = NULL;
 
+  g_clear_object (&self->factory);
+  g_clear_object (&self->header_factory);
+
   G_OBJECT_CLASS (gtk_list_view_parent_class)->dispose (object);
 }
 
@@ -710,8 +743,16 @@ gtk_list_view_get_property (GObject    *object,
 
   switch (property_id)
     {
+    case PROP_ENABLE_RUBBERBAND:
+      g_value_set_boolean (value, gtk_list_base_get_enable_rubberband (GTK_LIST_BASE (self)));
+      break;
+
     case PROP_FACTORY:
-      g_value_set_object (value, gtk_list_item_manager_get_factory (self->item_manager));
+      g_value_set_object (value, self->factory);
+      break;
+
+    case PROP_HEADER_FACTORY:
+      g_value_set_object (value, self->header_factory);
       break;
 
     case PROP_MODEL:
@@ -723,11 +764,11 @@ gtk_list_view_get_property (GObject    *object,
       break;
 
     case PROP_SINGLE_CLICK_ACTIVATE:
-      g_value_set_boolean (value, gtk_list_item_manager_get_single_click_activate (self->item_manager));
+      g_value_set_boolean (value, self->single_click_activate);
       break;
 
-    case PROP_ENABLE_RUBBERBAND:
-      g_value_set_boolean (value, gtk_list_base_get_enable_rubberband (GTK_LIST_BASE (self)));
+    case PROP_TAB_BEHAVIOR:
+      g_value_set_enum (value, gtk_list_base_get_tab_behavior (GTK_LIST_BASE (self)));
       break;
 
     default:
@@ -746,10 +787,18 @@ gtk_list_view_set_property (GObject      *object,
 
   switch (property_id)
     {
+    case PROP_ENABLE_RUBBERBAND:
+      gtk_list_view_set_enable_rubberband (self, g_value_get_boolean (value));
+      break;
+
     case PROP_FACTORY:
       gtk_list_view_set_factory (self, g_value_get_object (value));
       break;
 
+    case PROP_HEADER_FACTORY:
+      gtk_list_view_set_header_factory (self, g_value_get_object (value));
+      break;
+
     case PROP_MODEL:
       gtk_list_view_set_model (self, g_value_get_object (value));
       break;
@@ -762,8 +811,8 @@ gtk_list_view_set_property (GObject      *object,
       gtk_list_view_set_single_click_activate (self, g_value_get_boolean (value));
       break;
 
-    case PROP_ENABLE_RUBBERBAND:
-      gtk_list_view_set_enable_rubberband (self, g_value_get_boolean (value));
+    case PROP_TAB_BEHAVIOR:
+      gtk_list_view_set_tab_behavior (self, g_value_get_enum (value));
       break;
 
     default:
@@ -797,13 +846,11 @@ gtk_list_view_class_init (GtkListViewClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  list_base_class->list_item_name = "row";
-  list_base_class->list_item_role = GTK_ACCESSIBLE_ROLE_LIST_ITEM;
-  list_base_class->list_item_size = sizeof (ListRow);
-  list_base_class->list_item_augment_size = sizeof (ListRowAugment);
-  list_base_class->list_item_augment_func = list_row_augment;
-  list_base_class->get_allocation_along = gtk_list_view_get_allocation_along;
-  list_base_class->get_allocation_across = gtk_list_view_get_allocation_across;
+  list_base_class->split = gtk_list_view_split;
+  list_base_class->create_list_widget = gtk_list_view_create_list_widget;
+  list_base_class->prepare_section = gtk_list_view_prepare_section;
+  list_base_class->create_header_widget = gtk_list_view_create_header_widget;
+  list_base_class->get_allocation = gtk_list_view_get_allocation;
   list_base_class->get_items_in_rect = gtk_list_view_get_items_in_rect;
   list_base_class->get_position_from_allocation = gtk_list_view_get_position_from_allocation;
   list_base_class->move_focus_along = gtk_list_view_move_focus_along;
@@ -811,11 +858,25 @@ gtk_list_view_class_init (GtkListViewClass *klass)
 
   widget_class->measure = gtk_list_view_measure;
   widget_class->size_allocate = gtk_list_view_size_allocate;
+  widget_class->root = gtk_list_view_root;
+  widget_class->unroot = gtk_list_view_unroot;
+  widget_class->show = gtk_list_view_show;
+  widget_class->hide = gtk_list_view_hide;
 
   gobject_class->dispose = gtk_list_view_dispose;
   gobject_class->get_property = gtk_list_view_get_property;
   gobject_class->set_property = gtk_list_view_set_property;
 
+  /**
+   * GtkListView:enable-rubberband: (attributes org.gtk.Property.get=gtk_list_view_get_enable_rubberband org.gtk.Property.set=gtk_list_view_set_enable_rubberband)
+   *
+   * Allow rubberband selection.
+   */
+  properties[PROP_ENABLE_RUBBERBAND] =
+    g_param_spec_boolean ("enable-rubberband", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * GtkListView:factory: (attributes org.gtk.Property.get=gtk_list_view_get_factory org.gtk.Property.set=gtk_list_view_set_factory)
    *
@@ -826,6 +887,18 @@ gtk_list_view_class_init (GtkListViewClass *klass)
                          GTK_TYPE_LIST_ITEM_FACTORY,
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * GtkListView:header-factory: (attributes org.gtk.Property.get=gtk_list_view_get_header_factory org.gtk.Property.set=gtk_list_view_set_header_factory)
+   *
+   * Factory for creating header widgets.
+   *
+   * Since: 4.12
+   */
+  properties[PROP_HEADER_FACTORY] =
+    g_param_spec_object ("header-factory", NULL, NULL,
+                         GTK_TYPE_LIST_ITEM_FACTORY,
+                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
   /**
    * GtkListView:model: (attributes org.gtk.Property.get=gtk_list_view_get_model org.gtk.Property.set=gtk_list_view_set_model)
    *
@@ -857,14 +930,17 @@ gtk_list_view_class_init (GtkListViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkListView:enable-rubberband: (attributes org.gtk.Property.get=gtk_list_view_get_enable_rubberband org.gtk.Property.set=gtk_list_view_set_enable_rubberband)
+   * GtkListView:tab-behavior: (attributes org.gtk.Property.get=gtk_list_view_get_tab_behavior org.gtk.Property.set=gtk_list_view_set_tab_behavior)
    *
-   * Allow rubberband selection.
+   * Behavior of the <kbd>Tab</kbd> key
+   *
+   * Since: 4.12
    */
-  properties[PROP_ENABLE_RUBBERBAND] =
-    g_param_spec_boolean ("enable-rubberband", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+  properties[PROP_TAB_BEHAVIOR] =
+    g_param_spec_enum ("tab-behavior", NULL, NULL,
+                       GTK_TYPE_LIST_TAB_BEHAVIOR,
+                       GTK_LIST_TAB_ALL,
+                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (gobject_class, N_PROPS, properties);
 
@@ -1014,7 +1090,7 @@ gtk_list_view_get_factory (GtkListView *self)
 {
   g_return_val_if_fail (GTK_IS_LIST_VIEW (self), NULL);
 
-  return gtk_list_item_manager_get_factory (self->item_manager);
+  return self->factory;
 }
 
 /**
@@ -1031,14 +1107,77 @@ gtk_list_view_set_factory (GtkListView        *self,
   g_return_if_fail (GTK_IS_LIST_VIEW (self));
   g_return_if_fail (factory == NULL || GTK_IS_LIST_ITEM_FACTORY (factory));
 
-  if (factory == gtk_list_item_manager_get_factory (self->item_manager))
+  if (!g_set_object (&self->factory, factory))
     return;
 
-  gtk_list_item_manager_set_factory (self->item_manager, factory);
+  gtk_list_view_update_factories (self);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_FACTORY]);
 }
 
+/**
+ * gtk_list_view_get_header_factory: (attributes org.gtk.Method.get_property=header-factory)
+ * @self: a `GtkListView`
+ *
+ * Gets the factory that's currently used to populate section headers.
+ *
+ * Returns: (nullable) (transfer none): The factory in use
+ *
+ * Since: 4.12
+ */
+GtkListItemFactory *
+gtk_list_view_get_header_factory (GtkListView *self)
+{
+  g_return_val_if_fail (GTK_IS_LIST_VIEW (self), NULL);
+
+  return self->header_factory;
+}
+
+/**
+ * gtk_list_view_set_header_factory: (attributes org.gtk.Method.set_property=header-factory)
+ * @self: a `GtkListView`
+ * @factory: (nullable) (transfer none): the factory to use
+ *
+ * Sets the `GtkListItemFactory` to use for populating the
+ * [class@Gtk.ListHeader] objects used in section headers.
+ *
+ * If this factory is set to %NULL, the list will not show section headers.
+ *
+ * Since: 4.12
+ */
+void
+gtk_list_view_set_header_factory (GtkListView        *self,
+                                  GtkListItemFactory *factory)
+{
+  gboolean had_sections;
+
+  g_return_if_fail (GTK_IS_LIST_VIEW (self));
+  g_return_if_fail (factory == NULL || GTK_IS_LIST_ITEM_FACTORY (factory));
+
+  had_sections = gtk_list_item_manager_get_has_sections (self->item_manager);
+
+  if (!g_set_object (&self->header_factory, factory))
+    return;
+
+  gtk_list_item_manager_set_has_sections (self->item_manager, factory != NULL);
+
+  if (!gtk_list_view_is_inert (self) &&
+      had_sections && gtk_list_item_manager_get_has_sections (self->item_manager))
+    {
+      GtkListTile *tile;
+
+      for (tile = gtk_list_item_manager_get_first (self->item_manager);
+           tile != NULL;
+           tile = gtk_rb_tree_node_get_next (tile))
+        {
+          if (tile->widget && tile->type == GTK_LIST_TILE_HEADER)
+            gtk_list_header_widget_set_factory (GTK_LIST_HEADER_WIDGET (tile->widget), factory);
+        }
+    }
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_HEADER_FACTORY]);
+}
+
 /**
  * gtk_list_view_set_show_separators: (attributes org.gtk.Method.set_property=show-separators)
  * @self: a `GtkListView`
@@ -1095,12 +1234,22 @@ void
 gtk_list_view_set_single_click_activate (GtkListView *self,
                                          gboolean     single_click_activate)
 {
+  GtkListTile *tile;
+
   g_return_if_fail (GTK_IS_LIST_VIEW (self));
 
-  if (single_click_activate == gtk_list_item_manager_get_single_click_activate (self->item_manager))
+  if (single_click_activate == self->single_click_activate)
     return;
 
-  gtk_list_item_manager_set_single_click_activate (self->item_manager, single_click_activate);
+  self->single_click_activate = single_click_activate;
+
+  for (tile = gtk_list_item_manager_get_first (self->item_manager);
+       tile != NULL;
+       tile = gtk_rb_tree_node_get_next (tile))
+    {
+      if (tile->widget && tile->type == GTK_LIST_TILE_ITEM)
+        gtk_list_factory_widget_set_single_click_activate (GTK_LIST_FACTORY_WIDGET (tile->widget), single_click_activate);
+    }
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SINGLE_CLICK_ACTIVATE]);
 }
@@ -1119,7 +1268,7 @@ gtk_list_view_get_single_click_activate (GtkListView *self)
 {
   g_return_val_if_fail (GTK_IS_LIST_VIEW (self), FALSE);
 
-  return gtk_list_item_manager_get_single_click_activate (self->item_manager);
+  return self->single_click_activate;
 }
 
 /**
@@ -1158,3 +1307,74 @@ gtk_list_view_get_enable_rubberband (GtkListView *self)
 
   return gtk_list_base_get_enable_rubberband (GTK_LIST_BASE (self));
 }
+
+/**
+ * gtk_list_view_set_tab_behavior: (attributes org.gtk.Method.set_property=tab-behavior)
+ * @self: a `GtkListView`
+ * @tab_behavior: The desired tab behavior
+ *
+ * Sets the behavior of the <kbd>Tab</kbd> and <kbd>Shift</kbd>+<kbd>Tab</kbd> keys.
+ *
+ * Since: 4.12
+ */
+void
+gtk_list_view_set_tab_behavior (GtkListView        *self,
+                                GtkListTabBehavior  tab_behavior)
+{
+  g_return_if_fail (GTK_IS_LIST_VIEW (self));
+
+  if (tab_behavior == gtk_list_base_get_tab_behavior (GTK_LIST_BASE (self)))
+    return;
+
+  gtk_list_base_set_tab_behavior (GTK_LIST_BASE (self), tab_behavior);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_TAB_BEHAVIOR]);
+}
+
+/**
+ * gtk_list_view_get_tab_behavior: (attributes org.gtk.Method.get_property=tab-behavior)
+ * @self: a `GtkListView`
+ *
+ * Gets the behavior set for the <kbd>Tab</kbd> key.
+ *
+ * Returns: The behavior of the <kbd>Tab</kbd> key
+ *
+ * Since: 4.12
+ */
+GtkListTabBehavior
+gtk_list_view_get_tab_behavior (GtkListView *self)
+{
+  g_return_val_if_fail (GTK_IS_LIST_VIEW (self), FALSE);
+
+  return gtk_list_base_get_tab_behavior (GTK_LIST_BASE (self));
+}
+
+/**
+ * gtk_list_view_scroll_to:
+ * @self: The listview to scroll in
+ * @pos: position of the item. Must be less than the number of
+ *   items in the view.
+ * @flags: actions to perform
+ * @scroll: (nullable) (transfer full): details of how to perform
+ *   the scroll operation or %NULL to scroll into view 
+ *
+ * Scrolls to the item at the given position and performs the actions
+ * specified in @flags.
+ *
+ * This function works no matter if the listview is shown or focused.
+ * If it isn't, then the changes will take effect once that happens.
+ *
+ * Since: 4.12
+ */
+void
+gtk_list_view_scroll_to (GtkListView        *self,
+                         guint               pos,
+                         GtkListScrollFlags  flags,
+                         GtkScrollInfo      *scroll)
+{
+  g_return_if_fail (GTK_IS_LIST_VIEW (self));
+  g_return_if_fail (pos < gtk_list_base_get_n_items (GTK_LIST_BASE (self)));
+
+  gtk_list_base_scroll_to (GTK_LIST_BASE (self), pos, flags, scroll);
+}
+
