@@ -22,6 +22,7 @@
 #include "gtkmaplistmodel.h"
 
 #include "gtkrbtreeprivate.h"
+#include "gtksectionmodel.h"
 #include "gtkprivate.h"
 
 /**
@@ -54,6 +55,8 @@
  *
  * `GtkMapListModel` will attempt to discard the mapped objects as soon as
  * they are no longer needed and recreate them if necessary.
+ *
+ * `GtkMapListModel` passes through sections from the underlying model.
  */
 
 enum {
@@ -207,8 +210,43 @@ gtk_map_list_model_model_init (GListModelInterface *iface)
   iface->get_item = gtk_map_list_model_get_item;
 }
 
+static void
+gtk_map_list_model_get_section (GtkSectionModel *model,
+                                guint            position,
+                                guint           *out_start,
+                                guint           *out_end)
+{
+  GtkMapListModel *self = GTK_MAP_LIST_MODEL (model);
+
+  if (GTK_IS_SECTION_MODEL (self->model))
+    {
+      gtk_section_model_get_section (GTK_SECTION_MODEL (self->model), position, out_start, out_end);
+      return;
+    }
+
+  *out_start = 0;
+  *out_end = self->model ? g_list_model_get_n_items (self->model) : 0;
+}
+
+static void
+gtk_map_list_model_sections_changed_cb (GtkSectionModel *model,
+                                        unsigned int     position,
+                                        unsigned int     n_items,
+                                        gpointer         user_data)
+{
+  gtk_section_model_sections_changed (GTK_SECTION_MODEL (user_data), position, n_items);
+}
+
+
+static void
+gtk_map_list_model_section_model_init (GtkSectionModelInterface *iface)
+{
+  iface->get_section = gtk_map_list_model_get_section;
+}
+
 G_DEFINE_TYPE_WITH_CODE (GtkMapListModel, gtk_map_list_model, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, gtk_map_list_model_model_init))
+                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, gtk_map_list_model_model_init)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SECTION_MODEL, gtk_map_list_model_section_model_init))
 
 static void
 gtk_map_list_model_items_changed_cb (GListModel      *model,
@@ -337,6 +375,7 @@ gtk_map_list_model_clear_model (GtkMapListModel *self)
   if (self->model == NULL)
     return;
 
+  g_signal_handlers_disconnect_by_func (self->model, gtk_map_list_model_sections_changed_cb, self);
   g_signal_handlers_disconnect_by_func (self->model, gtk_map_list_model_items_changed_cb, self);
   g_clear_object (&self->model);
 }
@@ -608,6 +647,9 @@ gtk_map_list_model_set_model (GtkMapListModel *self,
       self->model = g_object_ref (model);
       g_signal_connect (model, "items-changed", G_CALLBACK (gtk_map_list_model_items_changed_cb), self);
       added = g_list_model_get_n_items (model);
+
+      if (GTK_IS_SECTION_MODEL (model))
+        g_signal_connect (model, "sections-changed", G_CALLBACK (gtk_map_list_model_sections_changed_cb), self);
     }
   else
     {
