@@ -95,9 +95,6 @@ struct _GtkPlacesView
   GtkSizeGroup                  *path_size_group;
   GtkSizeGroup                  *space_size_group;
 
-  GtkEntryCompletion            *address_entry_completion;
-  GtkListStore                  *completion_store;
-
   GCancellable                  *networks_fetching_cancellable;
 
   GtkPlacesViewRow              *row_for_action;
@@ -552,12 +549,9 @@ populate_servers (GtkPlacesView *view)
   while ((child = gtk_widget_get_first_child (GTK_WIDGET (view->recent_servers_listbox))))
     gtk_list_box_remove (GTK_LIST_BOX (view->recent_servers_listbox), child);
 
-  gtk_list_store_clear (view->completion_store);
-
   for (i = 0; i < num_uris; i++)
     {
       RemoveServerData *data;
-      GtkTreeIter iter;
       GtkWidget *row;
       GtkWidget *grid;
       GtkWidget *button;
@@ -568,14 +562,6 @@ populate_servers (GtkPlacesView *view)
       name = g_bookmark_file_get_title (server_list, uris[i], NULL);
       dup_uri = g_strdup (uris[i]);
 
-      /* add to the completion list */
-      gtk_list_store_append (view->completion_store, &iter);
-      gtk_list_store_set (view->completion_store,
-                          &iter,
-                          0, name,
-                          1, uris[i],
-                          -1);
-
       /* add to the recent servers listbox */
       row = gtk_list_box_row_new ();
 
@@ -884,20 +870,20 @@ update_network_state (GtkPlacesView *view)
        * otherwise just show the spinner in the header */
       if (!has_networks (view))
         {
-          gtk_widget_show (view->network_placeholder);
+          gtk_widget_set_visible (view->network_placeholder, TRUE);
           gtk_label_set_text (GTK_LABEL (view->network_placeholder_label),
                               _("Searching for network locations"));
         }
     }
   else if (!has_networks (view))
     {
-      gtk_widget_show (view->network_placeholder);
+      gtk_widget_set_visible (view->network_placeholder, TRUE);
       gtk_label_set_text (GTK_LABEL (view->network_placeholder_label),
                           _("No network locations found"));
     }
   else
     {
-      gtk_widget_hide (view->network_placeholder);
+      gtk_widget_set_visible (view->network_placeholder, FALSE);
     }
 }
 
@@ -1644,7 +1630,7 @@ populate_available_protocols_grid (GtkGrid *grid)
     }
 
   if (!has_any)
-    gtk_widget_hide (GTK_WIDGET (grid));
+    gtk_widget_set_visible (GTK_WIDGET (grid), FALSE);
 }
 
 static GMenuModel *
@@ -1706,17 +1692,18 @@ _popover_set_pointing_to_widget (GtkPopover *popover,
                                  GtkWidget  *target)
 {
   GtkWidget *parent;
-  double x, y, w, h;
+  graphene_point_t p;
+  double w, h;
 
   parent = gtk_widget_get_parent (GTK_WIDGET (popover));
 
-  if (!gtk_widget_translate_coordinates (target, parent, 0, 0, &x, &y))
+  if (!gtk_widget_compute_point (target, parent, &GRAPHENE_POINT_INIT (0, 0), &p))
     return;
 
-  w = gtk_widget_get_allocated_width (GTK_WIDGET (target));
-  h = gtk_widget_get_allocated_height (GTK_WIDGET (target));
+  w = gtk_widget_get_width (target);
+  h = gtk_widget_get_height (target);
 
-  gtk_popover_set_pointing_to (popover, &(GdkRectangle){x, y, w, h});
+  gtk_popover_set_pointing_to (popover, &(GdkRectangle){p.x, p.y, w, h});
 }
 
 static gboolean
@@ -1729,7 +1716,6 @@ real_popup_menu (GtkWidget *widget,
   GMount *mount;
   GFile *file;
   gboolean is_network;
-  double x_in_view, y_in_view;
 
   view = GTK_PLACES_VIEW (gtk_widget_get_ancestor (GTK_WIDGET (row), GTK_TYPE_PLACES_VIEW));
 
@@ -1767,10 +1753,13 @@ real_popup_menu (GtkWidget *widget,
     _popover_set_pointing_to_widget (GTK_POPOVER (view->popup_menu), GTK_WIDGET (row));
   else
     {
-      gtk_widget_translate_coordinates (widget, GTK_WIDGET (view),
-                                        x, y, &x_in_view, &y_in_view);
+      graphene_point_t p;
+      if (!gtk_widget_compute_point (widget, GTK_WIDGET (view),
+                                     &GRAPHENE_POINT_INIT (x, y),
+                                     &p))
+        graphene_point_init (&p, x, y);
       gtk_popover_set_pointing_to (GTK_POPOVER (view->popup_menu),
-                                   &(GdkRectangle){x_in_view, y_in_view, 0, 0});
+                                   &(GdkRectangle){p.x, p.y, 0, 0});
     }
 
   view->row_for_action = row;
@@ -1944,17 +1933,18 @@ on_address_entry_show_help_pressed (GtkPlacesView        *view,
                                     GtkEntry             *entry)
 {
   GdkRectangle rect;
-  double x, y;
+  graphene_point_t p;
 
   /* Setup the auxiliary popover's rectangle */
   gtk_entry_get_icon_area (GTK_ENTRY (view->address_entry),
                            GTK_ENTRY_ICON_SECONDARY,
                            &rect);
-  gtk_widget_translate_coordinates (view->address_entry, GTK_WIDGET (view),
-                                    rect.x, rect.y, &x, &y);
+  if (!gtk_widget_compute_point (view->address_entry, GTK_WIDGET (view),
+                                 &GRAPHENE_POINT_INIT (rect.x, rect.y), &p))
+    graphene_point_init (&p, rect.x, rect.y);
 
-  rect.x = x;
-  rect.y = y;
+  rect.x = p.x;
+  rect.y = p.y;
   gtk_popover_set_pointing_to (GTK_POPOVER (view->server_adresses_popover), &rect);
   gtk_widget_set_visible (view->server_adresses_popover, TRUE);
 }
@@ -1970,7 +1960,7 @@ on_recent_servers_listbox_row_activated (GtkPlacesView    *view,
 
   gtk_editable_set_text (GTK_EDITABLE (view->address_entry), uri);
 
-  gtk_widget_hide (view->recent_servers_popover);
+  gtk_widget_set_visible (view->recent_servers_popover, FALSE);
 }
 
 static void
@@ -2276,6 +2266,9 @@ gtk_places_view_class_init (GtkPlacesViewClass *klass)
                         G_TYPE_NONE, 2,
                         G_TYPE_STRING,
                         G_TYPE_STRING);
+  g_signal_set_va_marshaller (places_view_signals [SHOW_ERROR_MESSAGE],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_VOID__STRING_STRINGv);
 
   properties[PROP_LOADING] =
           g_param_spec_boolean ("loading", NULL, NULL,
@@ -2300,8 +2293,6 @@ gtk_places_view_class_init (GtkPlacesViewClass *klass)
 
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesView, actionbar);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesView, address_entry);
-  gtk_widget_class_bind_template_child (widget_class, GtkPlacesView, address_entry_completion);
-  gtk_widget_class_bind_template_child (widget_class, GtkPlacesView, completion_store);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesView, connect_button);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesView, listbox);
   gtk_widget_class_bind_template_child (widget_class, GtkPlacesView, recent_servers_listbox);
