@@ -1701,7 +1701,7 @@ gtk_range_render_trough (GtkGizmo    *gizmo,
     {
       GtkCssBoxes boxes;
       gtk_css_boxes_init (&boxes, GTK_WIDGET (gizmo));
-      gtk_snapshot_push_rounded_clip(snapshot, gtk_css_boxes_get_padding_box (&boxes));
+      gtk_snapshot_push_rounded_clip (snapshot, gtk_css_boxes_get_border_box (&boxes));
       gtk_widget_snapshot_child (GTK_WIDGET (gizmo), priv->highlight_widget, snapshot);
       gtk_snapshot_pop (snapshot);
     }
