@@ -22,12 +22,12 @@
 
 #include "gtkbox.h"
 #include "gtkbuildable.h"
-#include "gtkintl.h"
 #include "gtklabel.h"
 #include "gtkorientable.h"
 #include "gtkprivate.h"
 #include "gtkshortcutsshortcut.h"
 #include "gtksizegroup.h"
+#include "gtkaccessible.h"
 
 /**
  * GtkShortcutsGroup:
@@ -40,6 +40,14 @@
  * depending on the application context.
  *
  * This widget is only meant to be used with [class@Gtk.ShortcutsWindow].
+ *
+ * The recommended way to construct a `GtkShortcutsGroup` is with
+ * [class@Gtk.Builder], by using the `<child>` tag to populate a
+ * `GtkShortcutsGroup` with one or more [class@Gtk.ShortcutsShortcut]
+ * instances.
+ *
+ * If you need to add a shortcut programmatically, use
+ * [method@Gtk.ShortcutsGroup.add_shortcut].
  */
 
 struct _GtkShortcutsGroup
@@ -152,9 +160,8 @@ gtk_shortcuts_group_buildable_add_child (GtkBuildable *buildable,
 {
   if (GTK_IS_SHORTCUTS_SHORTCUT (child))
     {
-      gtk_box_append (GTK_BOX (buildable), GTK_WIDGET (child));
-      gtk_shortcuts_group_apply_accel_size_group (GTK_SHORTCUTS_GROUP (buildable), GTK_WIDGET (child));
-      gtk_shortcuts_group_apply_title_size_group (GTK_SHORTCUTS_GROUP (buildable), GTK_WIDGET (child));
+      gtk_shortcuts_group_add_shortcut (GTK_SHORTCUTS_GROUP (buildable),
+                                        GTK_SHORTCUTS_SHORTCUT (child));
     }
   else
     parent_buildable_iface->add_child (buildable, builder, child, type);
@@ -334,6 +341,7 @@ gtk_shortcuts_group_class_init (GtkShortcutsGroupClass *klass)
   g_object_class_install_properties (object_class, LAST_PROP, properties);
 
   gtk_widget_class_set_css_name (widget_class, I_("shortcuts-group"));
+  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GROUP);
 }
 
 static void
@@ -347,6 +355,7 @@ gtk_shortcuts_group_init (GtkShortcutsGroup *self)
   attrs = pango_attr_list_new ();
   pango_attr_list_insert (attrs, pango_attr_weight_new (PANGO_WEIGHT_BOLD));
   self->title = g_object_new (GTK_TYPE_LABEL,
+                              "accessible-role", GTK_ACCESSIBLE_ROLE_CAPTION,
                               "attributes", attrs,
                               "visible", TRUE,
                               "xalign", 0.0f,
@@ -354,4 +363,35 @@ gtk_shortcuts_group_init (GtkShortcutsGroup *self)
   pango_attr_list_unref (attrs);
 
   gtk_box_append (GTK_BOX (self), GTK_WIDGET (self->title));
+
+  gtk_accessible_update_relation (GTK_ACCESSIBLE (self),
+                                  GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->title, NULL,
+                                  -1);
+}
+
+/**
+ * gtk_shortcuts_group_add_shortcut:
+ * @self: a `GtkShortcutsGroup`
+ * @shortcut: the `GtkShortcutsShortcut` to add
+ *
+ * Adds a shortcut to the shortcuts group.
+ *
+ * This is the programmatic equivalent to using [class@Gtk.Builder] and a
+ * `<child>` tag to add the child. Adding children with other API is not
+ * appropriate as `GtkShortcutsGroup` manages its children internally.
+ *
+ * Since: 4.14
+ */
+void
+gtk_shortcuts_group_add_shortcut (GtkShortcutsGroup    *self,
+                                  GtkShortcutsShortcut *shortcut)
+{
+  g_return_if_fail (GTK_IS_SHORTCUTS_GROUP (self));
+  g_return_if_fail (GTK_IS_SHORTCUTS_SHORTCUT (shortcut));
+  g_return_if_fail (gtk_widget_get_parent (GTK_WIDGET (shortcut)) == NULL);
+
+  GtkWidget *widget = GTK_WIDGET (shortcut);
+  gtk_box_append (GTK_BOX (self), widget);
+  gtk_shortcuts_group_apply_accel_size_group (self, widget);
+  gtk_shortcuts_group_apply_title_size_group (self, widget);
 }
