@@ -22,6 +22,7 @@
 #include "gtksingleselection.h"
 
 #include "gtkbitset.h"
+#include "gtksectionmodelprivate.h"
 #include "gtkselectionmodel.h"
 
 /**
@@ -103,6 +104,23 @@ gtk_single_selection_list_model_init (GListModelInterface *iface)
   iface->get_item = gtk_single_selection_get_item;
 }
 
+static void
+gtk_single_selection_get_section (GtkSectionModel *model,
+                                  guint            position,
+                                  guint           *out_start,
+                                  guint           *out_end)
+{
+  GtkSingleSelection *self = GTK_SINGLE_SELECTION (model);
+
+  gtk_list_model_get_section (self->model, position, out_start, out_end);
+}
+
+static void
+gtk_single_selection_section_model_init (GtkSectionModelInterface *iface)
+{
+  iface->get_section = gtk_single_selection_get_section;
+}
+
 static gboolean
 gtk_single_selection_is_selected (GtkSelectionModel *model,
                                   guint              position)
@@ -167,6 +185,8 @@ gtk_single_selection_selection_model_init (GtkSelectionModelInterface *iface)
 G_DEFINE_TYPE_EXTENDED (GtkSingleSelection, gtk_single_selection, G_TYPE_OBJECT, 0,
                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL,
                                                gtk_single_selection_list_model_init)
+                        G_IMPLEMENT_INTERFACE (GTK_TYPE_SECTION_MODEL,
+                                               gtk_single_selection_section_model_init)
                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SELECTION_MODEL,
                                                gtk_single_selection_selection_model_init))
 
@@ -200,7 +220,6 @@ gtk_single_selection_items_changed_cb (GListModel         *model,
     {
       self->selected += added - removed;
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SELECTED]);
-      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SELECTED_ITEM]);
     }
   else
     {
@@ -216,7 +235,6 @@ gtk_single_selection_items_changed_cb (GListModel         *model,
                 {
                   self->selected = position + i;
                   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SELECTED]);
-                  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SELECTED_ITEM]);
                 }
 
               g_object_unref (item);
@@ -226,6 +244,8 @@ gtk_single_selection_items_changed_cb (GListModel         *model,
         }
       if (i == added)
         {
+          guint old_selected = self->selected;
+
           /* the item really was deleted */
           g_clear_object (&self->selected_item);
           if (self->autoselect)
@@ -266,7 +286,9 @@ gtk_single_selection_items_changed_cb (GListModel         *model,
               g_clear_object (&self->selected_item);
               self->selected = GTK_INVALID_LIST_POSITION;
             }
-          g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SELECTED]);
+          if (old_selected != self->selected)
+            g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SELECTED]);
+          /* the item was deleted above, so this is guaranteed to be new, even if the position didn't change */
           g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SELECTED_ITEM]);
         }
     }
@@ -278,15 +300,29 @@ gtk_single_selection_items_changed_cb (GListModel         *model,
   g_object_thaw_notify (G_OBJECT (self));
 }
 
+static void
+gtk_single_selection_sections_changed_cb (GtkSectionModel *model,
+                                          unsigned int     position,
+                                          unsigned int     n_items,
+                                          gpointer         user_data)
+{
+  GtkSingleSelection *self = GTK_SINGLE_SELECTION (user_data);
+
+  gtk_section_model_sections_changed (GTK_SECTION_MODEL (self), position, n_items);
+}
+
 static void
 gtk_single_selection_clear_model (GtkSingleSelection *self)
 {
   if (self->model == NULL)
     return;
 
-  g_signal_handlers_disconnect_by_func (self->model, 
+  g_signal_handlers_disconnect_by_func (self->model,
                                         gtk_single_selection_items_changed_cb,
                                         self);
+  g_signal_handlers_disconnect_by_func (self->model,
+                                        gtk_single_selection_sections_changed_cb,
+                                        self);
   g_clear_object (&self->model);
 }
 
@@ -429,7 +465,7 @@ gtk_single_selection_class_init (GtkSingleSelectionClass *klass)
   properties[PROP_MODEL] =
     g_param_spec_object ("model", NULL, NULL,
                          G_TYPE_LIST_MODEL,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkSingleSelection:n-items:
@@ -536,7 +572,7 @@ gtk_single_selection_set_model (GtkSingleSelection *self,
     return;
 
   g_object_freeze_notify (G_OBJECT (self));
-  
+
   n_items_before = self->model ? g_list_model_get_n_items (self->model) : 0;
   gtk_single_selection_clear_model (self);
 
@@ -545,6 +581,9 @@ gtk_single_selection_set_model (GtkSingleSelection *self,
       self->model = g_object_ref (model);
       g_signal_connect (self->model, "items-changed",
                         G_CALLBACK (gtk_single_selection_items_changed_cb), self);
+      if (GTK_IS_SECTION_MODEL (self->model))
+        g_signal_connect (self->model, "sections-changed",
+                          G_CALLBACK (gtk_single_selection_sections_changed_cb), self);
       gtk_single_selection_items_changed_cb (self->model,
                                              0,
                                              n_items_before,
