@@ -22,8 +22,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GTK_TEXT_VIEW_H__
-#define __GTK_TEXT_VIEW_H__
+#pragma once
 
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
@@ -432,6 +431,7 @@ PangoContext    *gtk_text_view_get_rtl_context        (GtkTextView      *text_vi
 GDK_AVAILABLE_IN_ALL
 PangoContext    *gtk_text_view_get_ltr_context        (GtkTextView      *text_view);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GtkTextView, g_object_unref)
+
 G_END_DECLS
 
-#endif /* __GTK_TEXT_VIEW_H__ */
