@@ -30,11 +30,13 @@
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtkscrollable.h"
+#include "gtkscrollinfoprivate.h"
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
 #include "gtkbuildable.h"
 #include "gtktext.h"
 
+#include <math.h>
 
 /**
  * GtkViewport:
@@ -54,7 +56,9 @@
  *
  * # Accessibility
  *
- * `GtkViewport` uses the %GTK_ACCESSIBLE_ROLE_GROUP role.
+ * Until GTK 4.10, `GtkViewport` used the `GTK_ACCESSIBLE_ROLE_GROUP` role.
+ *
+ * Starting from GTK 4.12, `GtkViewport` uses the `GTK_ACCESSIBLE_ROLE_GENERIC` role.
  */
 
 typedef struct _GtkViewportPrivate       GtkViewportPrivate;
@@ -331,10 +335,10 @@ gtk_viewport_class_init (GtkViewportClass *class)
                                    PROP_CHILD,
                                    g_param_spec_object ("child", NULL, NULL,
                                                         GTK_TYPE_WIDGET,
-                                                        GTK_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   gtk_widget_class_set_css_name (widget_class, I_("viewport"));
-  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GROUP);
+  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GENERIC);
 }
 
 static void
@@ -600,30 +604,12 @@ gtk_viewport_set_scroll_to_focus (GtkViewport *viewport,
   g_object_notify (G_OBJECT (viewport), "scroll-to-focus");
 }
 
-static void
-scroll_to_view (GtkAdjustment *adj,
-                double         pos,
-                double         size)
-{
-  double value, page_size;
-
-  value = gtk_adjustment_get_value (adj);
-  page_size = gtk_adjustment_get_page_size (adj);
-
-  if (pos < 0)
-    gtk_adjustment_animate_to_value (adj, value + pos);
-  else if (pos + size >= page_size)
-    gtk_adjustment_animate_to_value (adj, value + pos + size - page_size);
-}
-
 static void
 focus_change_handler (GtkWidget *widget)
 {
   GtkViewport *viewport = GTK_VIEWPORT (widget);
   GtkRoot *root;
   GtkWidget *focus_widget;
-  graphene_rect_t rect;
-  double x, y;
 
   if ((gtk_widget_get_state_flags (widget) & GTK_STATE_FLAG_FOCUS_WITHIN) == 0)
     return;
@@ -637,16 +623,7 @@ focus_change_handler (GtkWidget *widget)
   if (GTK_IS_TEXT (focus_widget))
     focus_widget = gtk_widget_get_parent (focus_widget);
 
-  if (!gtk_widget_compute_bounds (focus_widget, viewport->child, &rect))
-    return;
-
-  gtk_widget_translate_coordinates (viewport->child, widget,
-                                    rect.origin.x,
-                                    rect.origin.y,
-                                     &x, &y);
-
-  scroll_to_view (viewport->adjustment[GTK_ORIENTATION_HORIZONTAL], x, rect.size.width);
-  scroll_to_view (viewport->adjustment[GTK_ORIENTATION_VERTICAL], y, rect.size.height);
+  gtk_viewport_scroll_to (viewport, focus_widget, NULL);
 }
 
 static void
@@ -686,7 +663,7 @@ gtk_viewport_set_child (GtkViewport *viewport,
                         GtkWidget   *child)
 {
   g_return_if_fail (GTK_IS_VIEWPORT (viewport));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || viewport->child == child || gtk_widget_get_parent (child) == NULL);
 
   if (viewport->child == child)
     return;
@@ -718,3 +695,56 @@ gtk_viewport_get_child (GtkViewport *viewport)
   return viewport->child;
 }
 
+/**
+ * gtk_viewport_scroll_to:
+ * @viewport: a `GtkViewport`
+ * @descendant: a descendant widget of the viewport
+ * @scroll: (nullable) (transfer full): details of how to perform
+ *   the scroll operation or NULL to scroll into view
+ *
+ * Scrolls a descendant of the viewport into view.
+ *
+ * The viewport and the descendant must be visible and mapped for
+ * this function to work, otherwise no scrolling will be performed.
+ *
+ * Since: 4.12
+ **/
+void
+gtk_viewport_scroll_to (GtkViewport   *viewport,
+                        GtkWidget     *descendant,
+                        GtkScrollInfo *scroll)
+{
+  graphene_rect_t bounds;
+  int x, y;
+  double adj_x, adj_y;
+
+  g_return_if_fail (GTK_IS_VIEWPORT (viewport));
+  g_return_if_fail (GTK_IS_WIDGET (descendant));
+
+  if (!gtk_widget_compute_bounds (descendant, GTK_WIDGET (viewport), &bounds))
+    return;
+
+  adj_x = gtk_adjustment_get_value (viewport->adjustment[GTK_ORIENTATION_HORIZONTAL]);
+  adj_y = gtk_adjustment_get_value (viewport->adjustment[GTK_ORIENTATION_VERTICAL]);
+
+  gtk_scroll_info_compute_scroll (scroll,
+                                  &(GdkRectangle) {
+                                    floor (bounds.origin.x + adj_x),
+                                    floor (bounds.origin.y + adj_y),
+                                    ceil (bounds.origin.x + bounds.size.width) - floor (bounds.origin.x),
+                                    ceil (bounds.origin.y + bounds.size.height) - floor (bounds.origin.y)
+                                  },
+                                  &(GdkRectangle) {
+                                    adj_x,
+                                    adj_y,
+                                    gtk_widget_get_width (GTK_WIDGET (viewport)),
+                                    gtk_widget_get_height (GTK_WIDGET (viewport))
+                                  },
+                                  &x, &y);
+
+  gtk_adjustment_animate_to_value (viewport->adjustment[GTK_ORIENTATION_HORIZONTAL], x);
+  gtk_adjustment_animate_to_value (viewport->adjustment[GTK_ORIENTATION_VERTICAL], y);
+
+  g_clear_pointer (&scroll, gtk_scroll_info_unref);
+}
+
