@@ -22,13 +22,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GTK_WIDGET_PRIVATE_H__
-#define __GTK_WIDGET_PRIVATE_H__
+#pragma once
 
 #include "gtkwidget.h"
 
 #include "gtkactionmuxerprivate.h"
 #include "gtkatcontextprivate.h"
+#include "gtkborder.h"
 #include "gtkcsstypesprivate.h"
 #include "gtkeventcontrollerprivate.h"
 #include "gtklistlistmodelprivate.h"
@@ -154,7 +154,7 @@ struct _GtkWidgetPrivate
   GskTransform *allocated_transform;
   int allocated_width;
   int allocated_height;
-  int allocated_size_baseline;
+  int allocated_baseline;
 
   int width;
   int height;
@@ -316,9 +316,6 @@ gboolean          gtk_widget_focus_move                    (GtkWidget        *wi
                                                             GtkDirectionType  direction);
 void              gtk_widget_set_has_focus                 (GtkWidget        *widget,
                                                             gboolean          has_focus);
-void              gtk_widget_get_surface_allocation         (GtkWidget *widget,
-							     GtkAllocation *allocation);
-
 
 GtkWidget *       gtk_widget_common_ancestor               (GtkWidget *widget_a,
                                                             GtkWidget *widget_b);
@@ -446,15 +443,6 @@ _gtk_widget_get_display (GtkWidget *widget)
   return gtk_root_get_display (root);
 }
 
-static inline GtkStyleContext *
-_gtk_widget_get_style_context (GtkWidget *widget)
-{
-  if (G_LIKELY (widget->priv->context))
-    return widget->priv->context;
-
-  return gtk_widget_get_style_context (widget);
-}
-
 static inline gpointer
 _gtk_widget_peek_request_cache (GtkWidget *widget)
 {
@@ -493,4 +481,3 @@ _gtk_widget_is_sensitive (GtkWidget *widget)
 
 G_END_DECLS
 
-#endif /* __GTK_WIDGET_PRIVATE_H__ */
