@@ -20,7 +20,6 @@
 
 #include "gtkwindowhandle.h"
 
-#include "gtkactionmuxerprivate.h"
 #include "gtkbinlayout.h"
 #include "gtkbox.h"
 #include "gtkbuildable.h"
@@ -51,7 +50,9 @@
  *
  * # Accessibility
  *
- * `GtkWindowHandle` uses the %GTK_ACCESSIBLE_ROLE_GROUP role.
+ * Until GTK 4.10, `GtkWindowHandle` used the `GTK_ACCESSIBLE_ROLE_GROUP` role.
+ *
+ * Starting from GTK 4.12, `GtkWindowHandle` uses the `GTK_ACCESSIBLE_ROLE_GENERIC` role.
  */
 
 struct _GtkWindowHandle {
@@ -200,18 +201,21 @@ do_popup_fallback (GtkWindowHandle *self,
       GdkSurface *surface;
       double px, py;
       double nx, ny;
+      graphene_point_t p;
 
       native = gtk_widget_get_native (GTK_WIDGET (self));
       surface = gtk_native_get_surface (native);
       gdk_surface_get_device_position (surface, device, &px, &py, NULL);
       gtk_native_get_surface_transform (native, &nx, &ny);
 
-      gtk_widget_translate_coordinates (GTK_WIDGET (gtk_widget_get_native (GTK_WIDGET (self))),
-                                       GTK_WIDGET (self),
-                                       px - nx, py - ny,
-                                       &px, &py);
-      rect.x = px;
-      rect.y = py;
+      if (!gtk_widget_compute_point (GTK_WIDGET (gtk_widget_get_native (GTK_WIDGET (self))),
+                                     GTK_WIDGET (self),
+                                     &GRAPHENE_POINT_INIT (px - nx, py - ny),
+                                     &p))
+        graphene_point_init (&p, 0, 0);
+
+      rect.x = p.x;
+      rect.y = p.y;
     }
 
   gtk_popover_set_pointing_to (GTK_POPOVER (self->fallback_menu), &rect);
@@ -256,10 +260,17 @@ do_popup_fallback (GtkWindowHandle *self,
 
 static void
 do_popup (GtkWindowHandle *self,
-          GdkEvent        *event)
+          GtkGestureClick *gesture)
 {
   GdkSurface *surface =
     gtk_native_get_surface (gtk_widget_get_native (GTK_WIDGET (self)));
+  GdkEventSequence *sequence;
+  GdkEvent *event;
+
+  sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
+  event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
+  if (!event)
+    return;
 
   if (!gdk_toplevel_show_window_menu (GDK_TOPLEVEL (surface), event))
     do_popup_fallback (self, event);
@@ -267,7 +278,7 @@ do_popup (GtkWindowHandle *self,
 
 static gboolean
 perform_titlebar_action_fallback (GtkWindowHandle    *self,
-                                  GdkEvent           *event,
+                                  GtkGestureClick    *click_gesture,
                                   GdkTitlebarGesture  gesture)
 {
   GtkSettings *settings;
@@ -306,7 +317,7 @@ perform_titlebar_action_fallback (GtkWindowHandle    *self,
                                 "window.minimize",
                                 NULL);
   else if (g_str_equal (action, "menu"))
-    do_popup (self, event);
+    do_popup (self, click_gesture);
   else
     {
       g_warning ("Unsupported titlebar action %s", action);
@@ -320,7 +331,7 @@ perform_titlebar_action_fallback (GtkWindowHandle    *self,
 
 static gboolean
 perform_titlebar_action (GtkWindowHandle *self,
-                         GdkEvent        *event,
+			 GtkGestureClick *click_gesture,
                          guint            button,
                          int              n_press)
 {
@@ -349,7 +360,7 @@ perform_titlebar_action (GtkWindowHandle *self,
   if (gdk_toplevel_titlebar_gesture (GDK_TOPLEVEL (surface), gesture))
     return TRUE;
 
-  return perform_titlebar_action_fallback (self, event, gesture);
+  return perform_titlebar_action_fallback (self, click_gesture, gesture);
 }
 
 static void
@@ -360,17 +371,10 @@ click_gesture_pressed_cb (GtkGestureClick *gesture,
                           GtkWindowHandle *self)
 {
   GtkWidget *widget;
-  GdkEventSequence *sequence;
-  GdkEvent *event;
   guint button;
 
   widget = GTK_WIDGET (self);
-  sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
   button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
-  event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
-
-  if (!event)
-    return;
 
   if (n_press > 1)
     gtk_gesture_set_state (self->drag_gesture, GTK_EVENT_SEQUENCE_DENIED);
@@ -387,25 +391,25 @@ click_gesture_pressed_cb (GtkGestureClick *gesture,
     case GDK_BUTTON_PRIMARY:
       if (n_press == 2)
         {
-          perform_titlebar_action (self, event, button, n_press);
-          gtk_gesture_set_sequence_state (GTK_GESTURE (gesture),
-                                          sequence, GTK_EVENT_SEQUENCE_CLAIMED);
+          perform_titlebar_action (self, gesture, button, n_press);
+          gtk_gesture_set_state (GTK_GESTURE (gesture),
+				 GTK_EVENT_SEQUENCE_CLAIMED);
         }
       break;
 
     case GDK_BUTTON_SECONDARY:
-      if (perform_titlebar_action (self, event, button, n_press))
-        gtk_gesture_set_sequence_state (GTK_GESTURE (gesture),
-                                        sequence, GTK_EVENT_SEQUENCE_CLAIMED);
+      if (perform_titlebar_action (self, gesture, button, n_press))
+        gtk_gesture_set_state (GTK_GESTURE (gesture),
+			       GTK_EVENT_SEQUENCE_CLAIMED);
 
       gtk_event_controller_reset (GTK_EVENT_CONTROLLER (gesture));
       gtk_event_controller_reset (GTK_EVENT_CONTROLLER (self->drag_gesture));
       break;
 
     case GDK_BUTTON_MIDDLE:
-      if (perform_titlebar_action (self, event, button, n_press))
-        gtk_gesture_set_sequence_state (GTK_GESTURE (gesture),
-                                        sequence, GTK_EVENT_SEQUENCE_CLAIMED);
+      if (perform_titlebar_action (self, gesture, button, n_press))
+        gtk_gesture_set_state (GTK_GESTURE (gesture),
+			       GTK_EVENT_SEQUENCE_CLAIMED);
       break;
 
     default:
@@ -423,7 +427,7 @@ drag_gesture_update_cb (GtkGestureDrag  *gesture,
     {
       double start_x, start_y;
       double native_x, native_y;
-      double window_x, window_y;
+      graphene_point_t p;
       GtkNative *native;
       GdkSurface *surface;
 
@@ -433,21 +437,25 @@ drag_gesture_update_cb (GtkGestureDrag  *gesture,
 
       native = gtk_widget_get_native (GTK_WIDGET (self));
 
-      gtk_widget_translate_coordinates (GTK_WIDGET (self),
-                                        GTK_WIDGET (native),
-                                        start_x, start_y,
-                                        &window_x, &window_y);
+      if (!gtk_widget_compute_point (GTK_WIDGET (self),
+                                     GTK_WIDGET (native),
+                                     &GRAPHENE_POINT_INIT (start_x, start_y),
+                                     &p))
+        graphene_point_init (&p, start_x, start_y);
 
       gtk_native_get_surface_transform (native, &native_x, &native_y);
-      window_x += native_x;
-      window_y += native_y;
+      p.x += native_x;
+      p.y += native_y;
+
+      if (GTK_IS_WINDOW (native))
+        gtk_window_unfullscreen (GTK_WINDOW (native));
 
       surface = gtk_native_get_surface (native);
       if (GDK_IS_TOPLEVEL (surface))
         gdk_toplevel_begin_move (GDK_TOPLEVEL (surface),
                                  gtk_gesture_get_device (GTK_GESTURE (gesture)),
                                  gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture)),
-                                 window_x, window_y,
+                                 p.x, p.y,
                                  gtk_event_controller_get_current_event_time (GTK_EVENT_CONTROLLER (gesture)));
 
       gtk_event_controller_reset (GTK_EVENT_CONTROLLER (gesture));
@@ -541,7 +549,7 @@ gtk_window_handle_class_init (GtkWindowHandleClass *klass)
 
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, I_("windowhandle"));
-  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GROUP);
+  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GENERIC);
 }
 
 static void
@@ -622,7 +630,7 @@ gtk_window_handle_set_child (GtkWindowHandle *self,
                              GtkWidget       *child)
 {
   g_return_if_fail (GTK_IS_WINDOW_HANDLE (self));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || self->child == child || gtk_widget_get_parent (child) == NULL);
 
   if (self->child == child)
     return;
