@@ -25,7 +25,6 @@
 #include "type-info.h"
 
 #include "gtktypebuiltins.h"
-#include "gtktreeview.h"
 #include "gtkbuildable.h"
 #include "gtklabel.h"
 #include "gtkframe.h"
@@ -33,7 +32,7 @@
 #include "gtkmenubutton.h"
 #include "gtkwidgetprivate.h"
 #include "gtkbinlayout.h"
-
+#include "gtkwidgetprivate.h"
 
 struct _GtkInspectorMiscInfo
 {
@@ -57,12 +56,13 @@ struct _GtkInspectorMiscInfo
   GtkWidget *mnemonic_label;
   GtkWidget *request_mode_row;
   GtkWidget *request_mode;
+  GtkWidget *measure_info_row;
   GtkWidget *measure_row;
   GtkWidget *measure_expand_toggle;
   GtkWidget *measure_picture;
   GdkPaintable *measure_graph;
-  GtkWidget *allocated_size_row;
-  GtkWidget *allocated_size;
+  GtkWidget *bounds_row;
+  GtkWidget *bounds;
   GtkWidget *baseline_row;
   GtkWidget *baseline;
   GtkWidget *surface_row;
@@ -78,6 +78,8 @@ struct _GtkInspectorMiscInfo
   GtkWidget *tick_callback;
   GtkWidget *framerate_row;
   GtkWidget *framerate;
+  GtkWidget *scale_row;
+  GtkWidget *scale;
   GtkWidget *framecount_row;
   GtkWidget *framecount;
   GtkWidget *mapped_row;
@@ -139,24 +141,57 @@ state_flags_changed (GtkWidget *w, GtkStateFlags old_flags, GtkInspectorMiscInfo
   g_free (s);
 }
 
+static void
+update_measure_picture (GtkPicture      *picture,
+                        GtkToggleButton *toggle)
+{
+  GdkPaintable *paintable = gtk_picture_get_paintable (picture);
+
+  if (gtk_toggle_button_get_active (toggle) ||
+      (gdk_paintable_get_intrinsic_width (paintable) <= 200 &&
+       gdk_paintable_get_intrinsic_height (paintable) <= 100))
+    {
+      gtk_picture_set_can_shrink (picture, FALSE);
+      gtk_widget_set_size_request (GTK_WIDGET (picture), -1, -1);
+    }
+  else
+    {
+      gtk_picture_set_can_shrink (picture, TRUE);
+      gtk_widget_set_size_request (GTK_WIDGET (picture),
+                                   -1,
+                                   MIN (100, 200 / gdk_paintable_get_intrinsic_aspect_ratio (paintable)));
+    }
+}
+
+static void
+measure_graph_measure (GtkInspectorMiscInfo *sl)
+{
+  if (gtk_widget_get_visible (sl->measure_row))
+    gtk_inspector_measure_graph_measure (GTK_INSPECTOR_MEASURE_GRAPH (sl->measure_graph), GTK_WIDGET (sl->object));
+
+  update_measure_picture (GTK_PICTURE (sl->measure_picture), GTK_TOGGLE_BUTTON (sl->measure_expand_toggle));
+}
+
 static void
 update_allocation (GtkWidget            *w,
                    GtkInspectorMiscInfo *sl)
 {
-  GtkAllocation alloc;
+  graphene_rect_t bounds;
   char *size_label;
   GEnumClass *class;
   GEnumValue *value;
 
-  gtk_widget_get_allocation (w, &alloc);
-  size_label = g_strdup_printf ("%d × %d +%d +%d",
-                                alloc.width, alloc.height,
-                                alloc.x, alloc.y);
+  if (!gtk_widget_compute_bounds (w, gtk_widget_get_parent (w), &bounds))
+    graphene_rect_init (&bounds, 0, 0, 0, 0);
 
-  gtk_label_set_label (GTK_LABEL (sl->allocated_size), size_label);
+  size_label = g_strdup_printf ("%g × %g +%g +%g",
+                                bounds.size.width, bounds.size.height,
+                                bounds.origin.x, bounds.origin.y);
+
+  gtk_label_set_label (GTK_LABEL (sl->bounds), size_label);
   g_free (size_label);
 
-  size_label = g_strdup_printf ("%d", gtk_widget_get_allocated_baseline (w));
+  size_label = g_strdup_printf ("%d", gtk_widget_get_baseline (w));
   gtk_label_set_label (GTK_LABEL (sl->baseline), size_label);
   g_free (size_label);
 
@@ -165,15 +200,7 @@ update_allocation (GtkWidget            *w,
   gtk_label_set_label (GTK_LABEL (sl->request_mode), value->value_nick);
   g_type_class_unref (class);
 
-  if (gtk_widget_get_visible (sl->measure_row))
-    gtk_inspector_measure_graph_measure (GTK_INSPECTOR_MEASURE_GRAPH (sl->measure_graph), w);
-}
-
-static void
-measure_graph_measure (GtkWidget            *button,
-                       GtkInspectorMiscInfo *sl)
-{
-  gtk_inspector_measure_graph_measure (GTK_INSPECTOR_MEASURE_GRAPH (sl->measure_graph), GTK_WIDGET (sl->object));
+  measure_graph_measure (sl);
 }
 
 static void
@@ -245,73 +272,56 @@ show_frame_clock (GtkWidget *button, GtkInspectorMiscInfo *sl)
 static void
 update_surface (GtkInspectorMiscInfo *sl)
 {
+  gtk_widget_set_visible (sl->surface_row, GTK_IS_NATIVE (sl->object));
   if (GTK_IS_NATIVE (sl->object))
     {
       GObject *obj;
       char *tmp;
 
-      gtk_widget_show (sl->surface_row);
-
       obj = (GObject *)gtk_native_get_surface (GTK_NATIVE (sl->object));
       tmp = g_strdup_printf ("%p", obj);
       gtk_label_set_label (GTK_LABEL (sl->surface), tmp);
       g_free (tmp);
     }
-  else
-    {
-      gtk_widget_hide (sl->surface_row);
-    }
 }
 
 static void
 update_renderer (GtkInspectorMiscInfo *sl)
 {
+  gtk_widget_set_visible (sl->renderer_row, GTK_IS_NATIVE (sl->object));
   if (GTK_IS_NATIVE (sl->object))
     {
       GObject *obj;
       char *tmp;
 
-      gtk_widget_show (sl->renderer_row);
-
       obj = (GObject *)gtk_native_get_surface (GTK_NATIVE (sl->object));
       tmp = g_strdup_printf ("%p", obj);
       gtk_label_set_label (GTK_LABEL (sl->renderer), tmp);
       g_free (tmp);
     }
-  else
-    {
-      gtk_widget_hide (sl->renderer_row);
-    }
 }
 
 static void
 update_frame_clock (GtkInspectorMiscInfo *sl)
 {
+  gtk_widget_set_visible (sl->frame_clock_row, GTK_IS_ROOT (sl->object));
   if (GTK_IS_ROOT (sl->object))
     {
       GObject *clock;
 
-      gtk_widget_show (sl->frame_clock_row);
-
       clock = (GObject *)gtk_widget_get_frame_clock (GTK_WIDGET (sl->object));
+      gtk_widget_set_sensitive (sl->frame_clock_button, clock != NULL);
       if (clock)
         {
-          char *tmp;
-          tmp = g_strdup_printf ("%p", clock);
+          char *tmp = g_strdup_printf ("%p", clock);
           gtk_label_set_label (GTK_LABEL (sl->frame_clock), tmp);
           g_free (tmp);
-          gtk_widget_set_sensitive (sl->frame_clock_button, TRUE);
         }
       else
         {
           gtk_label_set_label (GTK_LABEL (sl->frame_clock), "NULL");
-          gtk_widget_set_sensitive (sl->frame_clock_button, FALSE);
         }
     }
-  else
-    {
-      gtk_widget_hide (sl->frame_clock_row);
-    }
 }
 
 static void
@@ -380,7 +390,6 @@ update_info (gpointer data)
           tmp = g_strdup_printf ("%p (%s)", l->data, g_type_name_from_instance ((GTypeInstance*)l->data));
           button = gtk_button_new_with_label (tmp);
           g_free (tmp);
-          gtk_widget_show (button);
           gtk_box_append (GTK_BOX (sl->mnemonic_label), button);
           g_object_set_data (G_OBJECT (button), "mnemonic-label", l->data);
           g_signal_connect (button, "clicked", G_CALLBACK (show_mnemonic_label), sl);
@@ -388,6 +397,10 @@ update_info (gpointer data)
       g_list_free (list);
 
       gtk_widget_set_visible (sl->tick_callback, gtk_widget_has_tick_callback (GTK_WIDGET (sl->object)));
+      gtk_widget_set_visible (sl->realized, gtk_widget_get_realized (GTK_WIDGET (sl->object)));
+      gtk_widget_set_visible (sl->mapped, gtk_widget_get_mapped (GTK_WIDGET (sl->object)));
+      gtk_widget_set_visible (sl->is_toplevel, GTK_IS_NATIVE (sl->object));
+      gtk_widget_set_visible (sl->child_visible, _gtk_widget_get_child_visible (GTK_WIDGET (sl->object)));
     }
 
   update_surface (sl);
@@ -437,6 +450,15 @@ update_info (gpointer data)
       sl->last_frame = frame;
     }
 
+  if (GDK_IS_SURFACE (sl->object))
+    {
+      char buf[64];
+
+      g_snprintf (buf, sizeof (buf), "%g", gdk_surface_get_scale (GDK_SURFACE (sl->object)));
+
+      gtk_label_set_label (GTK_LABEL (sl->scale), buf);
+    }
+
   return G_SOURCE_CONTINUE;
 }
 
@@ -462,28 +484,6 @@ measure_picture_drag_prepare (GtkDragSource *source,
   return gdk_content_provider_new_typed (GDK_TYPE_TEXTURE, texture);
 }
 
-static void
-update_measure_picture (GtkPicture      *picture,
-                        GtkToggleButton *toggle)
-{
-  GdkPaintable *paintable = gtk_picture_get_paintable (picture);
-
-  if (gtk_toggle_button_get_active (toggle) ||
-      (gdk_paintable_get_intrinsic_width (paintable) <= 200 &&
-       gdk_paintable_get_intrinsic_height (paintable) <= 100))
-    {
-      gtk_picture_set_can_shrink (picture, FALSE);
-      gtk_widget_set_size_request (GTK_WIDGET (picture), -1, -1);
-    }
-  else
-    {
-      gtk_picture_set_can_shrink (picture, TRUE);
-      gtk_widget_set_size_request (GTK_WIDGET (picture),
-                                   -1,
-                                   MIN (100, 200 / gdk_paintable_get_intrinsic_aspect_ratio (paintable)));
-    }
-}
-
 void
 gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
                                     GObject              *object)
@@ -496,28 +496,36 @@ gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
       sl->object = NULL;
     }
 
-  gtk_widget_show (GTK_WIDGET (sl));
+  gtk_widget_set_visible (GTK_WIDGET (sl), TRUE);
 
   sl->object = object;
   g_object_weak_ref (G_OBJECT (sl), disconnect_each_other, object);
   g_object_weak_ref (object, disconnect_each_other, sl);
 
+  gtk_widget_set_visible (sl->refcount_row, G_IS_OBJECT (object));
+  gtk_widget_set_visible (sl->state_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->direction_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->request_mode_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->bounds_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->baseline_row, GTK_IS_WIDGET (object));
+  /* Don't autoshow, it may be slow, we have a button for this */
+  if (!GTK_IS_WIDGET (object))
+    gtk_widget_set_visible (sl->measure_row, FALSE);
+  gtk_widget_set_visible (sl->measure_info_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->mnemonic_label_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->tick_callback_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->mapped_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->realized_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->is_toplevel_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->child_visible_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->frame_clock_row, GTK_IS_WIDGET (object));
+  gtk_widget_set_visible (sl->buildable_id_row, GTK_IS_BUILDABLE (object));
+  gtk_widget_set_visible (sl->framecount_row, GDK_IS_FRAME_CLOCK (object));
+  gtk_widget_set_visible (sl->framerate_row, GDK_IS_FRAME_CLOCK (object));
+  gtk_widget_set_visible (sl->scale_row, GDK_IS_SURFACE (object));
+
   if (GTK_IS_WIDGET (object))
     {
-      gtk_widget_show (sl->refcount_row);
-      gtk_widget_show (sl->state_row);
-      gtk_widget_show (sl->direction_row);
-      gtk_widget_show (sl->request_mode_row);
-      gtk_widget_show (sl->allocated_size_row);
-      gtk_widget_show (sl->baseline_row);
-      gtk_widget_show (sl->mnemonic_label_row);
-      gtk_widget_show (sl->tick_callback_row);
-      gtk_widget_show (sl->mapped_row);
-      gtk_widget_show (sl->realized_row);
-      gtk_widget_show (sl->is_toplevel_row);
-      gtk_widget_show (sl->is_toplevel_row);
-      gtk_widget_show (sl->frame_clock_row);
-
       g_signal_connect_object (object, "state-flags-changed", G_CALLBACK (state_flags_changed), sl, 0);
       state_flags_changed (GTK_WIDGET (sl->object), 0, sl);
 
@@ -526,40 +534,7 @@ gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
     }
   else
     {
-      gtk_widget_hide (sl->state_row);
-      gtk_widget_hide (sl->direction_row);
-      gtk_widget_hide (sl->request_mode_row);
-      gtk_widget_hide (sl->measure_row);
       gtk_inspector_measure_graph_clear (GTK_INSPECTOR_MEASURE_GRAPH (sl->measure_graph));
-      gtk_widget_hide (sl->mnemonic_label_row);
-      gtk_widget_hide (sl->allocated_size_row);
-      gtk_widget_hide (sl->baseline_row);
-      gtk_widget_hide (sl->tick_callback_row);
-      gtk_widget_hide (sl->mapped_row);
-      gtk_widget_hide (sl->realized_row);
-      gtk_widget_hide (sl->is_toplevel_row);
-      gtk_widget_hide (sl->child_visible_row);
-      gtk_widget_hide (sl->frame_clock_row);
-    }
-
-  if (GTK_IS_BUILDABLE (object))
-    {
-      gtk_widget_show (sl->buildable_id_row);
-    }
-  else
-    {
-      gtk_widget_hide (sl->buildable_id_row);
-    }
-
-  if (GDK_IS_FRAME_CLOCK (object))
-    {
-      gtk_widget_show (sl->framecount_row);
-      gtk_widget_show (sl->framerate_row);
-    }
-  else
-    {
-      gtk_widget_hide (sl->framecount_row);
-      gtk_widget_hide (sl->framerate_row);
     }
 
   update_info (sl);
@@ -636,11 +611,12 @@ gtk_inspector_misc_info_class_init (GtkInspectorMiscInfoClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, request_mode_row);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, request_mode);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, measure_row);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, measure_info_row);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, measure_expand_toggle);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, measure_picture);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, measure_graph);
-  gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, allocated_size_row);
-  gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, allocated_size);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, bounds_row);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, bounds);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, baseline_row);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, baseline);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, surface_row);
@@ -658,6 +634,8 @@ gtk_inspector_misc_info_class_init (GtkInspectorMiscInfoClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, framecount);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, framerate_row);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, framerate);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, scale_row);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, scale);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, mapped_row);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, mapped);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorMiscInfo, realized_row);
