@@ -33,14 +33,14 @@
 
 #include "gtkbuildable.h"
 #include "gtkbutton.h"
-#include "gtkcelllayout.h"
+#include "deprecated/gtkcelllayout.h"
 #include "gtkcolumnview.h"
-#include "gtkcomboboxprivate.h"
+#include "deprecated/gtkcomboboxprivate.h"
 #include "gtkfilterlistmodel.h"
 #include "gtkcustomfilter.h"
 #include "gtkflattenlistmodel.h"
 #include "gtkbuiltiniconprivate.h"
-#include "gtkiconview.h"
+#include "deprecated/gtkiconview.h"
 #include "gtkinscription.h"
 #include "gtklabel.h"
 #include "gtklistitem.h"
@@ -52,16 +52,16 @@
 #include "gtktogglebutton.h"
 #include "gtktreeexpander.h"
 #include "gtktreelistmodel.h"
-#include "gtktreeview.h"
-#include "gtktreeselection.h"
-#include "gtktreestore.h"
-#include "gtktreemodelsort.h"
-#include "gtktreemodelfilter.h"
+#include "deprecated/gtktreeview.h"
+#include "deprecated/gtktreeselection.h"
+#include "deprecated/gtktreemodelsort.h"
+#include "deprecated/gtktreemodelfilter.h"
 #include "gtkwidgetprivate.h"
 #include "gtksearchbar.h"
 #include "gtksearchentry.h"
 #include "gtkeventcontrollerkey.h"
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
 enum
 {
@@ -910,7 +910,11 @@ search (GtkInspectorObjectTree *wt,
         {
           if (match_object (child, text))
             {
-              gtk_single_selection_set_selected (priv->selection, row);
+              gtk_column_view_scroll_to (GTK_COLUMN_VIEW (wt->priv->list),
+                                         row,
+                                         NULL,
+                                         GTK_LIST_SCROLL_SELECT | GTK_LIST_SCROLL_FOCUS,
+                                         NULL);
               g_object_unref (child);
               g_object_unref (row_item);
               return TRUE;
@@ -1002,7 +1006,6 @@ bind_type_cb (GtkSignalListItemFactory *factory,
   expander = gtk_list_item_get_child (list_item);
   gtk_tree_expander_set_list_row (GTK_TREE_EXPANDER (expander), list_row);
   item = gtk_tree_list_row_get_item (list_row);
-  expander = gtk_list_item_get_child (list_item);
   inscription = gtk_tree_expander_get_child (GTK_TREE_EXPANDER (expander));
 
   gtk_inscription_set_text (GTK_INSCRIPTION (inscription), G_OBJECT_TYPE_NAME (item));
@@ -1132,11 +1135,19 @@ toplevel_filter_func (gpointer item,
                       gpointer data)
 {
   GdkDisplay *display = data;
+  gpointer iw;
 
   if (!GTK_IS_WINDOW (item))
     return FALSE;
 
-  return gtk_widget_get_display (item) == display;
+  if (gtk_widget_get_display (item) != display)
+    return FALSE;
+
+  iw = g_object_get_data (G_OBJECT (display), "-gtk-inspector");
+  if (iw == item)
+    return FALSE;
+
+  return TRUE;
 }
 
 static GListModel *
@@ -1295,12 +1306,28 @@ gtk_inspector_object_tree_select_object (GtkInspectorObjectTree *wt,
   if (row_item == NULL)
     return;
 
-  gtk_single_selection_set_selected (wt->priv->selection,
-                                     gtk_tree_list_row_get_position (row_item));
-  g_signal_emit (wt, signals[OBJECT_SELECTED], 0, object); // FIXME
+  gtk_column_view_scroll_to (GTK_COLUMN_VIEW (wt->priv->list),
+                             gtk_tree_list_row_get_position (row_item),
+                             NULL,
+                             GTK_LIST_SCROLL_SELECT | GTK_LIST_SCROLL_FOCUS,
+                             NULL);
+
+  g_signal_emit (wt, signals[OBJECT_SELECTED], 0, object);
   g_object_unref (row_item);
 }
 
+static void
+on_selected_item (GtkSingleSelection     *selection,
+                  GParamSpec             *pspec,
+                  GtkInspectorObjectTree *wt)
+{
+  GObject *selected = gtk_single_selection_get_selected_item (selection);
+  GtkTreeListRow *row = GTK_TREE_LIST_ROW (selected);
+  GObject *object = gtk_tree_list_row_get_item (row);
+  g_signal_emit (wt, signals[OBJECT_SELECTED], 0, object);
+  g_object_unref (object);
+}
+
 void
 gtk_inspector_object_tree_set_display (GtkInspectorObjectTree *wt,
                                        GdkDisplay *display)
@@ -1314,4 +1341,5 @@ gtk_inspector_object_tree_set_display (GtkInspectorObjectTree *wt,
   wt->priv->selection = gtk_single_selection_new (g_object_ref (G_LIST_MODEL (wt->priv->tree_model)));
   gtk_column_view_set_model (GTK_COLUMN_VIEW (wt->priv->list),
                              GTK_SELECTION_MODEL (wt->priv->selection));
+  g_signal_connect (wt->priv->selection, "notify::selected-item", G_CALLBACK (on_selected_item), wt);
 }
