@@ -797,9 +797,12 @@ font_changed (GObject *object, GParamSpec *pspec, gpointer data)
 
   font_desc = g_value_get_boxed (&val);
 
-  block_controller (G_OBJECT (fb));
-  gtk_font_dialog_button_set_font_desc (fb, font_desc);
-  unblock_controller (G_OBJECT (fb));
+  if (font_desc != NULL)
+    {
+      block_controller (G_OBJECT (fb));
+      gtk_font_dialog_button_set_font_desc (fb, font_desc);
+      unblock_controller (G_OBJECT (fb));
+    }
 
   g_value_unset (&val);
 }
@@ -933,7 +936,7 @@ property_editor (GObject                *object,
                                 G_PARAM_SPEC_INT (spec)->minimum,
                                 G_PARAM_SPEC_INT (spec)->maximum,
                                 1,
-                                MAX ((G_PARAM_SPEC_INT (spec)->maximum - G_PARAM_SPEC_INT (spec)->minimum) / 10, 1),
+                                MAX ((((double) G_PARAM_SPEC_INT (spec)->maximum - (double)G_PARAM_SPEC_INT (spec)->minimum)) / 10, 1),
                                 0.0);
 
       prop_edit = gtk_spin_button_new (adj, 1.0, 0);
