@@ -1,14 +1,23 @@
+#include "config.h"
 #include <string.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include "../reftests/reftest-compare.h"
 
+
 static char *arg_output_dir = NULL;
+static gboolean plain = FALSE;
 static gboolean flip = FALSE;
 static gboolean rotate = FALSE;
 static gboolean repeat = FALSE;
 static gboolean mask = FALSE;
+static gboolean replay = FALSE;
+static gboolean clip = FALSE;
+static gboolean colorflip = FALSE;
+
+extern void
+replay_node (GskRenderNode *node, GtkSnapshot *snapshot);
 
 static const char *
 get_output_dir (void)
@@ -155,10 +164,14 @@ deserialize_error_func (const GskParseLocation *start,
 
 static const GOptionEntry options[] = {
   { "output", 0, 0, G_OPTION_ARG_FILENAME, &arg_output_dir, "Directory to save image files to", "DIR" },
+  { "plain", 0, 0, G_OPTION_ARG_NONE, &plain, "Run test as-is", NULL },
   { "flip", 0, 0, G_OPTION_ARG_NONE, &flip, "Do flipped test", NULL },
   { "rotate", 0, 0, G_OPTION_ARG_NONE, &rotate, "Do rotated test", NULL },
   { "repeat", 0, 0, G_OPTION_ARG_NONE, &repeat, "Do repeated test", NULL },
   { "mask", 0, 0, G_OPTION_ARG_NONE, &mask, "Do masked test", NULL },
+  { "replay", 0, 0, G_OPTION_ARG_NONE, &replay, "Do replay test", NULL },
+  { "clip", 0, 0, G_OPTION_ARG_NONE, &clip, "Do clip test", NULL },
+  { "colorflip", 0, 0, G_OPTION_ARG_NONE, &colorflip, "Swap colors", NULL },
   { NULL }
 };
 
@@ -192,12 +205,29 @@ static GdkPixbuf *
 apply_mask_to_pixbuf (GdkPixbuf *pixbuf)
 {
   GdkPixbuf *copy;
+  int width, height;
 
   copy = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
-  for (unsigned int j = 0; j < gdk_pixbuf_get_height (copy); j++)
+  width = MIN (1000, gdk_pixbuf_get_width (pixbuf));
+  height = MIN (1000, gdk_pixbuf_get_height (pixbuf));
+  if (width < 25 || height < 25)
+    {
+      width = MIN (width, 25);
+      height = MIN (height, 25);
+    }
+  if (width != gdk_pixbuf_get_width (pixbuf) ||
+      height != gdk_pixbuf_get_height (pixbuf))
+    {
+      GdkPixbuf *sub;
+      sub = gdk_pixbuf_new_subpixbuf (copy, 0, 0, width, height);
+      g_object_unref (copy);
+      copy = sub;
+    }
+
+  for (unsigned int j = 0; j < height; j++)
     {
       guint8 *row = gdk_pixbuf_get_pixels (copy) + j * gdk_pixbuf_get_rowstride (copy);
-      for (unsigned int i = 0; i < gdk_pixbuf_get_width (copy); i++)
+      for (unsigned int i = 0; i < width; i++)
         {
           guint8 *p = row + i * 4;
           if ((i < 25 && j >= 25) || (i >= 25 && j < 25))
@@ -210,6 +240,65 @@ apply_mask_to_pixbuf (GdkPixbuf *pixbuf)
   return copy;
 }
 
+static GdkPixbuf *
+apply_colorflip_to_pixbuf (GdkPixbuf *pixbuf)
+{
+  GdkPixbuf *copy;
+  int width, height;
+
+  copy = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
+  width = gdk_pixbuf_get_width (pixbuf);
+  height = gdk_pixbuf_get_height (pixbuf);
+
+  for (unsigned int j = 0; j < height; j++)
+    {
+      guint8 *row = gdk_pixbuf_get_pixels (copy) + j * gdk_pixbuf_get_rowstride (copy);
+      for (unsigned int i = 0; i < width; i++)
+        {
+          guint8 *p = row + i * 4;
+          guint8 q;
+
+          q = p[0];
+          p[0] = p[1];
+          p[1] = q;
+        }
+    }
+
+  return copy;
+}
+
+static void
+make_random_clip (const graphene_rect_t *bounds,
+                  cairo_rectangle_int_t *int_clip)
+{
+  int_clip->width = g_test_rand_int_range (1, (int) floor (bounds->size.width));
+  int_clip->height = g_test_rand_int_range (1, (int) floor (bounds->size.height));
+
+  int_clip->x = g_test_rand_int_range ((int) ceil (bounds->origin.x), (int) floor (bounds->origin.x + bounds->size.width - int_clip->width));
+  int_clip->y = g_test_rand_int_range ((int) ceil (bounds->origin.y), (int) floor (bounds->origin.y + bounds->size.height - int_clip->height));
+}
+
+static void
+gsk_rect_from_cairo (graphene_rect_t              *rect,
+                      const cairo_rectangle_int_t *int_rect)
+{
+  rect->origin.x = int_rect->x;
+  rect->origin.y = int_rect->y;
+  rect->size.width = int_rect->width;
+  rect->size.height = int_rect->height;
+}
+
+static GdkPixbuf *
+apply_clip_to_pixbuf (GdkPixbuf                   *pixbuf,
+                      const cairo_rectangle_int_t *int_clip)
+{
+  return gdk_pixbuf_new_subpixbuf (pixbuf,
+                                   int_clip->x,
+                                   int_clip->y,
+                                   int_clip->width,
+                                   int_clip->height);
+}
+
 /*
  * Non-option arguments:
  *   1) .node file to compare
@@ -218,8 +307,8 @@ apply_mask_to_pixbuf (GdkPixbuf *pixbuf)
 int
 main (int argc, char **argv)
 {
-  GdkTexture *reference_texture;
-  GdkTexture *rendered_texture;
+  GdkTexture *reference_texture = NULL;
+  GdkTexture *rendered_texture = NULL;
   GskRenderer *renderer;
   GdkSurface *window;
   GskRenderNode *node;
@@ -228,7 +317,7 @@ main (int argc, char **argv)
   gboolean success = TRUE;
   GError *error = NULL;
   GOptionContext *context;
-  GdkTexture *diff_texture;
+  GdkTexture *diff_texture = NULL;
 
   (g_test_init) (&argc, &argv, NULL);
 
@@ -250,6 +339,9 @@ main (int argc, char **argv)
 
   g_option_context_free (context);
 
+  if (!plain && !flip && !rotate && !repeat && !mask && !replay && !clip && !colorflip)
+    plain = TRUE;
+
   gtk_init ();
 
   node_file = argv[1];
@@ -266,33 +358,36 @@ main (int argc, char **argv)
   if (!node)
     return 1;
 
-  /* Render the .node file and download to cairo surface */
-  rendered_texture = gsk_renderer_render_texture (renderer, node, NULL);
-  g_assert_nonnull (rendered_texture);
-
-  save_image (rendered_texture, node_file, ".out.png");
-
-  /* Load the given reference png file */
-  reference_texture = gdk_texture_new_from_filename (png_file, &error);
-  if (reference_texture == NULL)
+  if (plain)
     {
-      g_print ("Error loading reference surface: %s\n", error->message);
-      g_clear_error (&error);
+      /* Render the .node file and download to cairo surface */
+      rendered_texture = gsk_renderer_render_texture (renderer, node, NULL);
+      g_assert_nonnull (rendered_texture);
+
       save_image (rendered_texture, node_file, ".out.png");
-      return 0;
-    }
 
-  /* Now compare the two */
-  diff_texture = reftest_compare_textures (rendered_texture, reference_texture);
-  if (diff_texture)
-    {
-      save_image (diff_texture, node_file, ".diff.png");
-      g_object_unref (diff_texture);
-      success = FALSE;
-    }
+      /* Load the given reference png file */
+      reference_texture = gdk_texture_new_from_filename (png_file, &error);
+      if (reference_texture == NULL)
+        {
+          g_print ("Error loading reference surface: %s\n", error->message);
+          g_clear_error (&error);
+          save_image (rendered_texture, node_file, ".out.png");
+          return 0;
+        }
 
-  g_clear_object (&reference_texture);
-  g_clear_object (&rendered_texture);
+      /* Now compare the two */
+      diff_texture = reftest_compare_textures (rendered_texture, reference_texture);
+      if (diff_texture)
+        {
+          save_image (diff_texture, node_file, ".diff.png");
+          success = FALSE;
+        }
+
+      g_clear_object (&diff_texture);
+      g_clear_object (&reference_texture);
+      g_clear_object (&rendered_texture);
+    }
 
   if (flip)
     {
@@ -322,10 +417,10 @@ main (int argc, char **argv)
       if (diff_texture)
         {
           save_image (diff_texture, node_file, "-flipped.diff.png");
-          g_object_unref (diff_texture);
           success = FALSE;
         }
 
+      g_clear_object (&diff_texture);
       g_clear_object (&rendered_texture);
       g_clear_object (&reference_texture);
       gsk_render_node_unref (node2);
@@ -338,27 +433,16 @@ main (int argc, char **argv)
       int width, height;
       graphene_rect_t node_bounds;
       graphene_rect_t bounds;
-      float offset_x, offset_y;
 
       gsk_render_node_get_bounds (node, &node_bounds);
 
-      if (node_bounds.size.width > 32768. || node_bounds.size.height > 32768.)
-        {
-          g_print ("Avoiding repeat test that exceeds cairo image surface dimensions");
-          exit (77);
-        }
+      node_bounds.size.width = ceil (node_bounds.size.width);
+      node_bounds.size.height = ceil (node_bounds.size.height);
 
-      bounds.origin.x = 0.;
-      bounds.origin.y = 0.;
-      bounds.size.width = 2 * node_bounds.size.width;
-      bounds.size.height = 2 * node_bounds.size.height;
-
-      offset_x = floorf (fmodf (node_bounds.origin.x, node_bounds.size.width));
-      offset_y = floorf (fmodf (node_bounds.origin.y, node_bounds.size.height));
-      if (offset_x < 0)
-        offset_x += node_bounds.size.width;
-      if (offset_y < 0)
-        offset_y += node_bounds.size.height;
+      bounds.size.width = MIN (1000, 3 * node_bounds.size.width);
+      bounds.size.height = MIN (1000, 3 * node_bounds.size.height);
+      bounds.origin.x = node_bounds.origin.x + floorf (node_bounds.size.width / 2);
+      bounds.origin.y = node_bounds.origin.y + floorf (node_bounds.size.height / 2);
 
       node2 = gsk_repeat_node_new (&bounds, node, &node_bounds);
       save_node (node2, node_file, "-repeated.node");
@@ -373,14 +457,18 @@ main (int argc, char **argv)
       pixbuf2 = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
                                 gdk_pixbuf_get_has_alpha (pixbuf),
                                 gdk_pixbuf_get_bits_per_sample (pixbuf),
-                                width * 3,
-                                height * 3);
+                                width * 4,
+                                height * 4);
 
-      for (int i = 0; i < 3; i++)
-        for (int j = 0; j < 3; j++)
-          gdk_pixbuf_copy_area (pixbuf, 0, 0, width, height, pixbuf2, i * width,  j * height);
+      for (int i = 0; i < 4; i++)
+        {
+          for (int j = 0; j < 4; j++)
+            {
+              gdk_pixbuf_copy_area (pixbuf, 0, 0, width, height, pixbuf2, i * width,  j * height);
+            }
+        }
 
-      pixbuf3 = gdk_pixbuf_new_subpixbuf (pixbuf2, width - (int) offset_x, height - (int) offset_y, 2 * width, 2 * height);
+      pixbuf3 = gdk_pixbuf_new_subpixbuf (pixbuf2, width / 2, height / 2, MIN (1000, 3 * width), MIN (1000, 3 * height));
 
       reference_texture = gdk_texture_new_for_pixbuf (pixbuf3);
 
@@ -395,10 +483,10 @@ main (int argc, char **argv)
       if (diff_texture)
         {
           save_image (diff_texture, node_file, "-repeated.diff.png");
-          g_object_unref (diff_texture);
           success = FALSE;
         }
 
+      g_clear_object (&diff_texture);
       g_clear_object (&rendered_texture);
       g_clear_object (&reference_texture);
       gsk_render_node_unref (node2);
@@ -432,10 +520,10 @@ main (int argc, char **argv)
       if (diff_texture)
         {
           save_image (diff_texture, node_file, "-rotated.diff.png");
-          g_object_unref (diff_texture);
           success = FALSE;
         }
 
+      g_clear_object (&diff_texture);
       g_clear_object (&rendered_texture);
       g_clear_object (&reference_texture);
       gsk_render_node_unref (node2);
@@ -452,14 +540,25 @@ main (int argc, char **argv)
       gsk_render_node_get_bounds (node, &bounds);
       nodes[0] = gsk_color_node_new (&(GdkRGBA){ 0, 0, 0, 1},
                                      &GRAPHENE_RECT_INIT (bounds.origin.x, bounds.origin.y, 25, 25));
-      nodes[1] = gsk_color_node_new (&(GdkRGBA){ 0, 0, 0, 1},
-                                     &GRAPHENE_RECT_INIT (bounds.origin.x + 25, bounds.origin.y + 25, bounds.size.width - 25, bounds.size.height - 25));
+      if (bounds.size.width > 25 && bounds.size.height > 25)
+        {
+          nodes[1] = gsk_color_node_new (&(GdkRGBA){ 0, 0, 0, 1},
+                                         &GRAPHENE_RECT_INIT (
+                                             bounds.origin.x + 25,
+                                             bounds.origin.y + 25,
+                                             MIN (1000, bounds.size.width) - 25,
+                                             MIN (1000, bounds.size.height) - 25));
+          mask_node = gsk_container_node_new (nodes, G_N_ELEMENTS (nodes));
+          gsk_render_node_unref (nodes[0]);
+          gsk_render_node_unref (nodes[1]);
+        }
+      else
+        {
+          mask_node = nodes[0];
+        }
 
-      mask_node = gsk_container_node_new (nodes, G_N_ELEMENTS (nodes));
       node2 = gsk_mask_node_new (node, mask_node, GSK_MASK_MODE_ALPHA);
       gsk_render_node_unref (mask_node);
-      gsk_render_node_unref (nodes[0]);
-      gsk_render_node_unref (nodes[1]);
       save_node (node2, node_file, "-masked.node");
 
       rendered_texture = gsk_renderer_render_texture (renderer, node2, NULL);
@@ -478,22 +577,158 @@ main (int argc, char **argv)
       if (diff_texture)
         {
           save_image (diff_texture, node_file, "-masked.diff.png");
-          g_object_unref (diff_texture);
           success = FALSE;
         }
 
+      g_clear_object (&diff_texture);
+      g_clear_object (&rendered_texture);
+      g_clear_object (&reference_texture);
+      gsk_render_node_unref (node2);
+    }
+
+  if (replay)
+    {
+      GskRenderNode *node2;
+      GdkTexture *rendered_texture2;
+      graphene_rect_t node_bounds, node2_bounds;
+      GtkSnapshot *snapshot = gtk_snapshot_new ();
+
+      replay_node (node, snapshot);
+      node2 = gtk_snapshot_free_to_node (snapshot);
+      /* If the whole render node tree got eliminated, make sure we have
+         something to work with nevertheless.  */
+      if (!node2)
+        node2 = gsk_container_node_new (NULL, 0);
+
+      save_node (node2, node_file, "-replayed.node");
+      gsk_render_node_get_bounds (node, &node_bounds);
+      gsk_render_node_get_bounds (node2, &node2_bounds);
+      /* Check that the node didn't grow.  */
+      success = success && graphene_rect_contains_rect (&node_bounds, &node2_bounds);
+
+      rendered_texture = gsk_renderer_render_texture (renderer, node, &node_bounds);
+      save_image (rendered_texture, node_file, "-replayed.ref.png");
+      rendered_texture2 = gsk_renderer_render_texture (renderer, node2, &node_bounds);
+      save_image (rendered_texture2, node_file, "-replayed.out.png");
+      g_assert_nonnull (rendered_texture);
+      g_assert_nonnull (rendered_texture2);
+
+      diff_texture = reftest_compare_textures (rendered_texture, rendered_texture2);
+
+      if (diff_texture)
+        {
+          save_image (diff_texture, node_file, "-replayed.diff.png");
+          success = FALSE;
+        }
+
+      g_clear_object (&diff_texture);
+      g_clear_object (&rendered_texture);
+      g_clear_object (&rendered_texture2);
+      gsk_render_node_unref (node2);
+    }
+
+  if (clip)
+    {
+      GskRenderNode *node2;
+      GdkPixbuf *pixbuf, *pixbuf2;
+      graphene_rect_t bounds;
+      cairo_rectangle_int_t int_clip;
+      graphene_rect_t clip_rect;
+
+      gsk_render_node_get_bounds (node, &bounds);
+
+      if (bounds.size.width <= 1 || bounds.size.height <= 1)
+        {
+          g_test_skip ("Can't make a random clip");
+          goto skip_clip;
+        }
+
+      make_random_clip (&bounds, &int_clip);
+      g_print ("Random clip rectangle %d %d %d %d\n",
+               int_clip.x, int_clip.y, int_clip.width, int_clip.height);
+      gsk_rect_from_cairo (&clip_rect, &int_clip);
+      g_assert_true (graphene_rect_contains_rect (&bounds, &clip_rect));
+      g_assert_true (graphene_rect_get_area (&clip_rect) != 0);
+
+      node2 = gsk_clip_node_new (node, &clip_rect);
+      save_node (node2, node_file, "-clipped.node");
+
+      rendered_texture = gsk_renderer_render_texture (renderer, node2, NULL);
+      save_image (rendered_texture, node_file, "-clipped.out.png");
+
+      pixbuf = gdk_pixbuf_new_from_file (png_file, &error);
+
+      int_clip.x -= (int) bounds.origin.x;
+      int_clip.y -= (int) bounds.origin.y;
+
+      pixbuf2 = apply_clip_to_pixbuf (pixbuf, &int_clip);
+      reference_texture = gdk_texture_new_for_pixbuf (pixbuf2);
+      g_object_unref (pixbuf2);
+      g_object_unref (pixbuf);
+
+      save_image (reference_texture, node_file, "-clipped.ref.png");
+
+      diff_texture = reftest_compare_textures (rendered_texture, reference_texture);
+
+      if (diff_texture)
+        {
+          save_image (diff_texture, node_file, "-clipped.diff.png");
+          success = FALSE;
+        }
+
+      g_clear_object (&diff_texture);
+      g_clear_object (&rendered_texture);
+      g_clear_object (&reference_texture);
+      gsk_render_node_unref (node2);
+    }
+
+skip_clip:
+
+  if (colorflip)
+    {
+      GskRenderNode *node2;
+      GdkPixbuf *pixbuf, *pixbuf2;
+      graphene_matrix_t matrix;
+
+      graphene_matrix_init_from_float (&matrix,
+                                       (const float []) { 0, 1, 0, 0,
+                                                          1, 0, 0, 0,
+                                                          0, 0, 1, 0,
+                                                          0, 0, 0, 1 });
+
+      node2 = gsk_color_matrix_node_new (node, &matrix, graphene_vec4_zero ());
+
+      save_node (node2, node_file, "-colorflipped.node");
+
+      rendered_texture = gsk_renderer_render_texture (renderer, node2, NULL);
+      save_image (rendered_texture, node_file, "-colorflipped.out.png");
+
+      pixbuf = gdk_pixbuf_new_from_file (png_file, &error);
+      pixbuf2 = apply_colorflip_to_pixbuf (pixbuf);
+      reference_texture = gdk_texture_new_for_pixbuf (pixbuf2);
+      g_object_unref (pixbuf2);
+      g_object_unref (pixbuf);
+
+      save_image (reference_texture, node_file, "-colorflipped.ref.png");
+
+      diff_texture = reftest_compare_textures (rendered_texture, reference_texture);
+
+      if (diff_texture)
+        {
+          save_image (diff_texture, node_file, "-colorflipped.diff.png");
+          success = FALSE;
+        }
+
+      g_clear_object (&diff_texture);
       g_clear_object (&rendered_texture);
       g_clear_object (&reference_texture);
       gsk_render_node_unref (node2);
     }
 
   gsk_render_node_unref (node);
-
   gsk_renderer_unrealize (renderer);
   g_object_unref (renderer);
   gdk_surface_destroy (window);
 
-  gdk_display_close (gdk_display_get_default ());
-
   return success ? 0 : 1;
 }
