@@ -15,8 +15,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __REFTEST_MODULE_H__
-#define __REFTEST_MODULE_H__
+#pragma once
 
 #include <gtk/gtk.h>
 
@@ -40,5 +39,3 @@ GCallback       reftest_module_lookup           (ReftestModule  *module,
                                                  const char     *function_name);
 
 G_END_DECLS
-
-#endif /* __REFTEST_MODULE_H__ */
